Unit CPWinControls;

{$IfDef FPC}
  {$Mode Delphi}
{$EndIf}

interface

Uses SysUtils, Classes, CPWindows;

Procedure FreeAndNil(var Obj);

{const
  SS_LineWrap = $10000; }

Type
  TCPWinControl = Class(TComponent)
  Protected
    FLeft, FTop,
    FWidth, FHeight : Integer;
    FWindowText : PChar;
    ObjectType : String;
    Procedure SetWindowText(Value : PChar); Virtual;

    Function GetLeft : Integer; Virtual;
    Function GetTop : Integer; Virtual;
    Function GetWidth : Integer; Virtual;
    Function GetHeight : Integer; Virtual;
  Public
    Visible : Boolean;
    BKBrush : hBrush;
    BKFont  : hFont;
    BKPen   : hPen;

    hwndParent : hwnd;
    Painting : Boolean;
    childhwnd : TStrings;
    PaintRect : TRect;
    WndProc : TWndProc;
    WNDCLASS : TWNDCLASS;
    WNDCLASSEX : TWNDClASSEX;
    WName : PChar;
    Extended : Boolean;
    Function CanFocus : Boolean; Virtual;
    Procedure SetFocus; Virtual;
    Procedure SetAlign(Val : Integer); Virtual;
    Procedure SetParentWindow(Val : TCPWinControl); Virtual;
    Procedure SetSize(W,H : Integer); Virtual;
    Procedure SetOrigin(X,Y : Integer); Virtual;
    Procedure SetWidth(Val : Integer);
    Procedure SetHeight(Val : Integer);
    Procedure SetLeft(Val : Integer);
    Procedure SetTop(Val : Integer); 
    Procedure Show; Virtual;
    Procedure Hide; Virtual;
    Procedure UpdateWindow; Virtual;
    Constructor Create(theObjectType : String); Virtual;
    Destructor Destroy; Override; 
    Function SendMessage(Msg: UINT; wParam: WPARAM;
      lParam: LPARAM): LRESULT;
    Property WindowText : PChar read FWindowText write SetWindowText;
    Property Left : Integer read GetLeft write FLeft;
    Property Top : Integer read GetTop write FTop;
    Property Width : Integer read GetWidth write FWidth;
    Property Height : Integer read GetHeight write FHeight;
  end;

  function DestroyWindow(var Wnd: HWND): BOOL;
  function ShowWindow(hWnd: HWND; nCmdShow: Integer): BOOL;
  function UpdateWindow(hWnd: HWND): BOOL;

  function SetParent(hWndChild, hWndNewParent: HWND): HWND;
  function SetWindowPos(hWnd: HWND; X, Y, cx, cy: Integer): BOOL;
  function SetWindowLong(hWnd: HWND; nIndex: Integer; dwNewLong: Longint): Longint;

  function SendMessage(hWnd: HWND; Msg: UINT; wParam: WPARAM;
    lParam: LPARAM): LRESULT;  
  function CallWindowProc(lpPrevWndFunc: TFNWndProc; hWnd: HWND;
    Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT;

  function GetWindowText(hWnd: HWND): PChar;
  function SetWindowText(hWnd: HWND; const lpString: PChar): BOOL;
  function SetFocus(hWnd: HWND): HWND;
  function GetActiveWindow: HWND;
  function GetFocus: HWND;

Var
  dummyP : TComponent;

implementation

Procedure FreeAndNil(var Obj);
var
  Tmp : TObject;
begin
  If Pointer(Obj) <> nil then begin
    Tmp := TObject(Obj);
    Pointer(Obj) := nil;
    Tmp.Free;
  end;
end;

Procedure TCPWinControl.Show;
var
  I :integer;
begin
  Visible := True;
  If Extended then
    WNDClassEx.Style := WNDClassEx.Style or WS_Visible;
  For I := 0 to Childhwnd.Count - 1 do
    If TCPWinControl(StrToInt(childhwnd.Strings[I])).Visible then
      TCPWinControl(StrToInt(childhwnd.Strings[I])).Show;
end;

Procedure TCPWinControl.Hide;
begin
  Visible := False;
  If Extended then
    WNDClassEx.Style := WNDClassEx.Style or not WS_Visible;
end;

Function TCPWinControl.CanFocus : Boolean;
begin
  Result := False;
end;

Procedure TCPWinControl.SetFocus;
begin
end;

Destructor TCPWinControl.Destroy;
begin
  childhwnd.Free;
  Inherited Destroy;
end;

Procedure TCPWinControl.SetParentWindow(Val : TCPWinControl);
begin
  If hwndParent <> 0 then
    With TCPWinControl(hwndParent) do
      If childhwnd.IndexOf(intToStr(Hwnd(self))) > -1 then
        childhwnd.Delete(childhwnd.IndexOf(intToStr(Hwnd(self))));
  hwndParent := hwnd(Val);
  If Val <> nil then
    With Val do
      ChildhWnd.Append(intToStr(hwnd(Self)));
end;

Procedure TCPWinControl.SetAlign(Val : Integer);
begin
end;

Procedure TCPWinControl.SetWindowText(Value : PChar);
begin
  FWindowText := Value;
end;

Procedure TCPWinControl.SetWidth(Val : Integer);
begin
  SetSize(Val,Height)
end;

Function TCPWinControl.GetLeft : Integer;
begin
  Result := FLeft;
end;

Function TCPWinControl.GetTop : Integer;
begin
  Result := FTop;
end;

Function TCPWinControl.GetWidth : Integer;
begin
  Result := FWidth;
end;

Function TCPWinControl.GetHeight : Integer;
begin
  Result := FHeight;
end;

Procedure TCPWinControl.SetSize(W,H : Integer);
begin
  FWidth := W;
  FHeight := H;
end;

Procedure TCPWinControl.SetHeight(Val : Integer);
begin
  SetSize(Width,Val)
end;

Procedure TCPWinControl.SetLeft(Val : Integer);
begin
  SetOrigin(Val, Top);
end;

Procedure TCPWinControl.SetTop(Val : Integer);
begin
  SetOrigin(Left, Val);
end;

Procedure TCPWinControl.SetOrigin(X,Y : Integer);
begin
  FLeft := X;
  FTop := Y;
end;

Procedure TCPWinControl.UpdateWindow;
var
  ST,SW : Longint;

  Function StyleHas(Styles : Integer) : Boolean;
  begin
    Result := ST or Styles = st;
  end;

begin
  If Extended then begin
    st := WNDClassEX.Style;
    If STYLEHAS(WS_VISIBLE) then begin
      If STYLEHAS(WS_Maximize) then
        SW := SW_MAXIMIZE
      else
        If STYLEHAS(WS_Minimize) then
	  SW := SW_MINIMIZE
        else
	  SW := SW_Show;
    end
    else
      SW := SW_Hide;
    If STYLEHAS(SS_Center) then
      SetAlign(SS_Center)
    else
      If STYLEHAS(SS_Right) then
        SetAlign(SS_Right)
    else
      If STYLEHAS(SS_LEFT) then
        SetAlign(SS_Left);
    ShowWindow(Longint(Self), SW);
  end;
end;

Constructor TCPWinControl.Create(theObjectType : String);
begin
  inherited Create(DummyP);
  childhwnd := TStringlist.Create;
  theObjectType := Uppercase(theObjectType);
  ObjectType := theObjectType;
  Name := IntToStr(Longint(Self));
end;

Function TCPWinControl.SendMessage(Msg: UINT; wParam: WPARAM;
  lParam: LPARAM): LRESULT;
begin
  If Assigned(WndProc) then
    Result := WndProc(Longint(Self), Msg, wParam, lParam)
  else
    Result := 0;
end;

function DestroyWindow(var Wnd: HWND): BOOL;
var
  I : Integer;
  L : hwnd;
begin
  Result := False;
  If WND <> 0 then
    Try
      Try
	ShowWindow(wnd, SW_Hide);  
        SetParent(Wnd,0);
	For I := TCPWinControl(wnd).Childhwnd.Count - 1 downto 0 do begin
	  l := StrToINt(TCPWinControl(wnd).Childhwnd.strings[I]);
	  DestroyWindow(L);
	end;
      Finally
        FreeAndNil(Wnd);
	wnd := 0;
      end;
      Result := Wnd = 0;
    except
    end;
end;

function ShowWindow(hWnd: HWND; nCmdShow: Integer): BOOL;
begin
  Result := False;
  If HWND <> 0 then
    Try
      With TCPWinControl(hWnd) do
        Case nCmdShow of
          SW_HIDE:
            begin
              Hide;
              Result := True;
            end;
          SW_SHOWNORMAL:
            begin
              ShowWindow(hWnd,SW_Show);
            end;
          SW_SHOWMINIMIZED:
            begin
            end;
          SW_SHOWMAXIMIZED:
            begin
            end;
          SW_SHOWNOACTIVATE:
            begin
            end;
          SW_SHOW:
            begin
              Show;
              Result := True;
            end;
          SW_MINIMIZE:
            begin
            end;
          SW_SHOWMINNOACTIVE:
            begin
            end;
          SW_SHOWNA:
            begin
            end;
          SW_RESTORE:
            begin
            end;
          SW_SHOWDEFAULT:
            begin
              ShowWindow(hWnd,SW_Show);
            end;
      end;
    except
   end;
end;

function UpdateWindow(hWnd: HWND): BOOL;
var
  I : Integer; 
begin
  Result := False;
  If hWnd <> 0 then
    With TCPWinControl(hWnd) do begin
      For I := 0 To ChildhWnd.Count - 1 do
        CPWinControls.UpdateWindow(StrToInt(ChildhWnd.Strings[I]));
      UpdateWindow;
      Result := True;
    end;	 
end;

function SetParent(hWndChild, hWndNewParent: HWND): HWND;
begin
  Result := hWndChild;
  If hWndChild <> 0 then
    Try
      With TCPWinControl(hWndChild) do begin
        If hWndNewParent <> 0 then
          SetParentWindow(TCPWinControl(hWndNewParent))
        else
          SetParentWindow(nil);
      end;
    Except
      Result := 0;
    end;
end;

function SetWindowPos(hWnd: HWND; X, Y, cx, cy: Integer): BOOL;
begin
  Result := False;
  If HWND <> 0 then
    Try
      With TCPWinControl(hWnd) do begin
        SetOrigin(X, Y);
        SetSize(cx, cy);
      end;
      Result := True;
    Except
    end;
end;

function SendMessage(hWnd: HWND; Msg: UINT; wParam: WPARAM;
  lParam: LPARAM): LRESULT;
begin
  Result := 0;
  If HWND <> 0 then
    Try
      If TObject(hWnd) is TCPWinControl then
        Result := TCPWinControl(hWnd).SendMessage(Msg,
          wParam,lParam);
    except
      Result := 0;
    end;
end;

function CallWindowProc(lpPrevWndFunc: TFNWndProc; hWnd: HWND;
  Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT;
begin
  //I am fairly sure this wrong, but I am not sure how to do it right
  Result := TWNDPROC(lpPrevWndFunc)(hWnd,Msg,wParam,lParam);
end;

function GetWindowText(hWnd: HWND): PChar;
begin
  Result := '';
  If hWnd <> 0 then
    Try
      With TCPWinControl(hWnd) do
        Result := WindowText;
    except
      Result := '';
    end;
end;

function SetWindowText(hWnd: HWND; const lpString: PChar): BOOL;
begin
  Result := False;
  If hWnd <> 0 then
    Try
      With TCPWinControl(hWnd) do begin
        WindowText := lpString;
        Result := True;
      end;
    except
      Result := False;
    end;
end;

function SetWindowLong(hWnd: HWND; nIndex: Integer; dwNewLong: Longint): Longint;
begin
  Result := 0;
  If hWnd <> 0 then
    With TCPWinControl(hWnd), WNDCLASSEX do begin
      case nIndex Of
        GWL_WNDPROC:
          begin
            Result := Longint(@lpfnWndProc);
            lpfnWndProc := TWNDPROC(dwNewLong);
            wndproc := lpfnWndProc;
          end;
        GWL_HINSTANCE:
          begin
            Result := WNDCLASSEX.hInstance;
            WNDCLASSEX.hInstance := dwNewLong;
          end;
        GWL_HWNDPARENT:
          begin
            Result := hwndParent;
            SetParent(hWnd,dwnewLong);
          end;
        GWL_STYLE:
          begin
            Result := Style;
            style := dwnewLong;
          end;
      end;
    end;
end;

function SetFocus(hWnd: HWND): HWND;
begin
  Result := 0;
  If hWnd <> 0 then
    With TCPWinControl(hWnd) do
      If CanFocus then begin
        Result := CPWinControls.GetFocus;
        SetFocus;
      end;
end;

function GetActiveWindow: HWND;
begin
  Result := 0;
end;

function GetFocus: HWND;
begin
  Result := 0;
end;

initialization
  dummyP := TComponent.Create(nil);

end.