Unit CPWinDevices;

{$IfDef Fpc}
  {$Mode Delphi}
{$EndIf}

interface

Uses Classes, SysUtils, CPWindows, CPWinControls, CPWinObjects;

Type
  TCPhDevice = Class(TComponent)
  Protected
    wnd      : TCPWinControl;
    Function GetOrigin : TPoint; Virtual;
    Function GetSize : TSize; Virtual;

    Class Function GuiBitBlt(DestDC: HDC; X, Y, Width, Height: Integer; SrcDC: HDC;
      XSrc, YSrc: Integer; Rop: DWORD): BOOL; Virtual;
    Class Function GuiGetPixel(DC: HDC; X, Y: Integer): TCOLORREF; Virtual;
    Class Function GuiDrawPixel(DC: HDC; X, Y: Integer) : Integer; Virtual;

    Class Procedure GuiDrawArc(hDC: HDC; Filled : Boolean; left, top, right, bottom, startX, startY,
      endX, endY: Integer); Virtual; Overload;
    Class Procedure GuiDrawArc(hDC: HDC; Filled : Boolean; left, top, right, bottom,
      Angle1, Angle2: Integer); Virtual; Overload;
    Class Procedure GuiDrawChord(hDC: HDC; Filled : Boolean; left, top, right, bottom, startX, startY,
      endX, endY: Integer); Virtual;
    Class Procedure GuiDrawEllipse(hDC: HDC; Filled : Boolean; left, top, right, bottom : Integer); Virtual;
    Class Procedure GuiDrawLine(DC : hDC; X1,Y1,X2,Y2 : Integer); Virtual;
    Class Procedure GuiDrawRectangle(DC : hDC; Filled : Boolean; X, Y, Width, Height : Integer); Virtual;
    Class Procedure GuiDrawPolyline(DC: HDC; var Points; Count: Integer; FRMTO : PPoint); Virtual;
    Class Procedure GuiDrawPolygon(DC: HDC; Filled : Boolean; var Points;
      Count: Integer); Virtual;
    Class Procedure GuiInvertRect(hDC: HDC; const lprc: TRect); Virtual;
  Public
    lRect   : TRect;
    DCPen     : hPen;
    DCBrush   : hBrush;
    Pen     : hPen;
    Brush   : hBrush;
    Font    : hFont;
    Bitmap  : hBitmap;
    BKColor : TColorRef;
    ROPMode,
    BKMode  : Integer;
    PenPos  : TPoint;
    ArcDir  : Integer;
    Procedure UpdatePlaceP; Virtual;
    Procedure SelectPen; Virtual;
    Procedure SelectBrush; Virtual;
    Procedure SelectFont; Virtual;
    Procedure SelectBKColor; Virtual;
    Procedure SelectBKMode; Virtual;
    Procedure SelectRop2; Virtual;
    Constructor Create; Overload; Virtual;
    Constructor Create(ARect : TRect); Overload; Virtual;
    destructor  Destroy; Override;
  end;

  TCPhDeviceClass = Class of TCPhDevice;

(* Objects *)
  Function SelectObject(DC: HDC; p2: HGDIOBJ): HGDIOBJ;
  Function DeleteObject(var p1: HGDIOBJ): BOOL;

  function GetClipBox(DC: HDC; var lRect: TRect): Integer;
  Function IntersectClipRect(DC : hDC; Left, Top, Right,
    Bottom : Integer) : Integer;

  Function GetDCBrushColor(DC: HDC): COLORREF;
  Function GetDCPenColor(DC: HDC): COLORREF;
  Function GetBkColor(DC: HDC): COLORREF;
  Function GetBkMode(DC: HDC): Integer;
  Function SetBkColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
  Function SetBkMode(DC: HDC; theBkMode: Integer): Integer;
  Function SetDCBrushColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
  Function SetDCPenColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
  Function SetRop2(DC : HDC; p2: Integer) : Integer;
  function SetTextColor(DC: HDC; Color: TCOLORREF): TCOLORREF;

(* Bitmaps *)
  Function BitBlt(DestDC: HDC; X, Y, Width, Height: Integer; SrcDC: HDC;
    XSrc, YSrc: Integer; Rop: DWORD): BOOL;
  Function GetPixel(DC: HDC; X, Y: Integer): COLORREF;
  Function SetPixel(DC: HDC; X, Y: Integer; Color: COLORREF): COLORREF;
  Function SetPixelV(DC: HDC; X, Y: Integer; Color: COLORREF): Integer;

(* Lines and Curves *)
  Function Arc(hDC: HDC; left, top, right, bottom, startX, startY,
    endX, endY: Integer): BOOL;
  Function MoveToEx(DC: HDC; p2, p3: Integer; p4: PPoint): BOOL;
  Function LineTo(DC: HDC; X, Y: Integer): BOOL;
  Function Polyline(DC: HDC; var Points; Count: Integer): BOOL;
  Function PolylineTo(DC: HDC; var Points; Count: Integer): BOOL;
  
(* Filled Shapes *)  
  Function Chord(DC: HDC; X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer): BOOL;
  Function Ellipse(DC: HDC; X1, Y1, X2, Y2: Integer): BOOL;
  Function FillRect(hDC: HDC; const lprc: TRect; hbr: HBRUSH): Integer;
  Function FrameRect(hDC: HDC; const lprc: TRect; hbr: HBRUSH): Integer;
  Function InvertRect(hDC: HDC; const lprc: TRect): Integer;
  Function Pie(hDC: HDC; left, top, right, bottom, startX, startY,
    endX, endY: Integer): BOOL;
  Function Polygon(DC: HDC; var Points; Count: Integer): BOOL;
  Function Rectangle(DC: HDC; X1, Y1, X2, Y2: Integer): BOOL;
  Function RoundRect(DC: HDC; X1, Y1, X2, Y2, X3, Y3: Integer): BOOL;

  Procedure InitializeDevices(NewDeviceClass : TCPhDeviceClass);

var
  hDevices : TList;

  hDeviceClass : TCPhDeviceClass;

implementation

Function TCPhDevice.GetOrigin : TPoint;
begin
  Result.x := 0;
  Result.y := 0;
end;

Function TCPhDevice.GetSize : TSize;
begin
  Result.cx := 0;
  Result.cy := 0;
end;

Class Function TCPhDevice.GUIBitBlt(DestDC: HDC; X, Y, Width, Height: Integer; SrcDC: HDC;
  XSrc, YSrc: Integer; Rop: DWORD): BOOL;
begin
end;

Class Function TCPhDevice.GuiGetPixel(DC: HDC; X, Y: Integer): TCOLORREF;
begin
end;

Class Function TCPhDevice.GuiDrawPixel(DC: HDC; X, Y: Integer): Integer;
begin
end;

Class Procedure TCPhDevice.GuiDrawArc(hDC: HDC; Filled : Boolean; left, top, right, bottom, startX, startY,
  endX, endY: Integer);
begin
end;

Class Procedure TCPhDevice.GuiDrawArc(hDC: HDC; Filled : Boolean; left, top, right, bottom,
  Angle1, Angle2: Integer);
begin
end;

Class Procedure TCPhDevice.GuiDrawChord(hDC: HDC; Filled : Boolean; left, top, right, bottom, startX, startY,
  endX, endY: Integer);
begin
end;

Class Procedure TCPhDevice.GuiDrawEllipse(hDC: HDC; Filled : Boolean; left, top, right, bottom : Integer);
begin
end;

Class Procedure TCPhDevice.GuiDrawLine(DC : hDC; X1,Y1,X2,Y2 : Integer);
begin
end;

Class Procedure TCPhDevice.GuiDrawRectangle(DC : hDC; Filled : Boolean; X, Y, Width, Height : Integer);
begin
end;

Class Procedure TCPhDevice.GuiDrawPolyline(DC: HDC; var Points; Count: Integer; FRMTO : PPoint);
begin
end;

Class Procedure TCPhDevice.GuiDrawPolygon(DC: HDC; Filled : Boolean; var Points;
  Count: Integer);
begin
end;

Class Procedure TCPhDevice.GuiInvertRect(hDC: HDC; const lprc: TRect);
begin

end;

Constructor TCPhDevice.Create;
begin
  Create(Rect(0,0,0,0));
end;

Constructor TCPhDevice.Create(ARect : TRect);
begin
  inherited Create(DummyP);
  hDevices.Add(Self);
  PenPos.X := 0;
  PenPos.Y := 0;
  SelectBrush;
  SelectPen;
  SelectFont;
  RopMode := 0;
  lRect.Left := ARect.Left;
  lRect.Top := ARect.Top;
  lRect.Right := ARect.Right;
  lRect.Bottom := ARect.Bottom;
end;

destructor  TCPhDevice.Destroy;
begin
  inherited Destroy;
end;

Procedure TCPhDevice.SelectPen;
begin
  SelectBKColor;
  If DCPen = 0 then
    DCPen := CreatePenIndirect(Default_Pen);
  If Pen = 0 then
    Pen := DCPen;
  TCPhObject(Pen).hDCSelect(hDC(Self));
end;

Procedure TCPhDevice.SelectBKColor;
begin
end;

Procedure TCPhDevice.SelectBKMode;
begin
end;

Procedure TCPhDevice.UpdatePlaceP;
begin

end;

Procedure TCPhDevice.SelectBrush;
begin
  SelectBKColor;
  If DCBrush = 0 then
    DCBrush := CreateBrushIndirect(Default_Brush);
  If Brush = 0 then
    Brush := DCBrush;
  TCPhObject(Brush).hDCSelect(hDC(Self));
end;

Procedure TCPhDevice.SelectFont;
begin
  If Font = 0 then
    Font := DefaultFont;
  TCPhObject(Font).hDCSelect(hDC(Self));
end;

Procedure TCPhDevice.SelectRop2;
begin
end;

Function SelectObject(DC: HDC; p2: HGDIOBJ): HGDIOBJ;
begin
  Result := 0;
  If DC <> 0 then
    If TObject(DC) is TCPhDevice then
      If p2 <> 0 then
        if TObject(p2) is TCPhPen then begin
          With TCPhDevice(DC) do begin
            If Pen <> 0 then
              Result := Pen;
            Pen := p2;
          end;
        end
        else
          if TObject(p2) is TCPhBrush then begin
            With TCPhDevice(DC) do begin
              If Brush <> 0 then
                Result := Brush;
              Brush := p2;
            end;
          end
        else
          if TObject(p2) is TCPhFont then begin
            With TCPhDevice(DC) do begin
              If Font <> 0 then
                Result := Font;
              Font := p2;
            end;
          end;
end;

function DeleteObject(var p1: HGDIOBJ): BOOL;
var
  L,I : Longint;
  T : hGDIOBJ;
begin
 Result := False;
 T := p1;
 Result := CPWinObjects.DeleteObject(p1); 
 For I := 0 to DummyP.ComponentCount - 1 do begin
    If DummyP.Components[I] is TCPhDevice then
      If TCPhDevice(DummyP.Components[I]).Pen = T then
        TCPhDevice(DummyP.Components[I]).Pen := 0
      else
        If TCPhDevice(DummyP.Components[I]).Brush = T then
          TCPhDevice(DummyP.Components[I]).Brush := 0
      else
        If TCPhDevice(DummyP.Components[I]).Font = T then
          TCPhDevice(DummyP.Components[I]).Font := 0;
  end;
end;

function GetDCBrushColor(DC: HDC): COLORREF;
begin
  Result := $FFFFFF;
  If (DC <> 0)then
    If (TObject(DC) is TCPhDevice) then
      With TCPhDevice(DC) do begin
        SelectBrush;
        Result := TCPhBrush(Brush).Color;
      end;
end;

Function GetDCPenColor(DC: HDC): COLORREF;
begin
  Result := $0;
  If (DC <> 0)then
    If (TObject(DC) is TCPhDevice) then
      With TCPhDevice(DC) do begin
        SelectPen;
        Result := TCPhPen(Pen).Color;
      end;
end;

function GetBkColor(DC: HDC): COLORREF;
begin
  Result := $FFFFFF;
  If (DC <> 0)then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        Result := BKColor;
      end;
end;

function GetBkMode(DC: HDC): Integer;
begin
  Result := 0;
  If (DC <> 0)then
    If (TObject(DC) is TCPhDevice) then
      Result := TCPhDevice(DC).BKMode;
end;

function SetBkColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
begin
  Result := 0;
  If DC <> 0 then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        BKColor := Color;
        SelectBKColor;
        Result := BKColor;
      end;
end;

function SetBkMode(DC: HDC; theBkMode: Integer): Integer;
begin
  Result := 0;
  If DC <> 0 then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        BKMode := theBKMode;
        SelectBKMode;
        Result := BKMode;
      end;
end;

function SetDCBrushColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
begin
  Result := 0;
  If DC <> 0 then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        SelectBrush;
        TCPhBrush(Brush).Color := Color;
        SelectBrush;
        Result := TCPhBrush(Brush).Color;
      end;
end;

function SetDCPenColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
begin
  Result := 0;
  If DC <> 0 then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        SelectPen;
        Result := TCPhPen(Pen).Color;
        TCPhPen(Pen).Color := Color;
        SelectPen;
      end;
end;

function GetClipBox(DC: HDC; var lRect: TRect): Integer;
begin
  Result := 0;
  If (DC <> 0)then
    If (TObject(DC) is TCPhDevice) then
      With TCPhDevice(DC).lRect do begin
        lRect.Left := Left;
        lRect.Right := Right;
        lRect.Top := Top;
        lRect.Bottom := Bottom;
        Result := 1;
      end;
end;

Function IntersectClipRect(DC : hDC; Left, Top, Right,
  Bottom : Integer) : Integer;
begin
  Result := 0;
  If DC <> 0 then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
       lRect.Left := Left;
       lRect.Top := Top;
       lRect.Right := Right;
       lRect.Bottom := Bottom;
       UpdatePlaceP;
      end;
end;

function SetROP2(DC: HDC; p2: Integer): Integer;
begin
  Result := 0;
  If (DC <> 0)then
    If (TObject(DC) is TCPhDevice) then
      With TCPhDevice(DC) do begin
        Result := ROPMode;
        ROPMode := p2;
        SelectRop2;
      end;
end;

function SetTextColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
begin
  Result := 0;
  If DC <> 0 then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        SelectFont;
        Result := TCPhFont(Font).Color;
        TCPhFont(Font).Color := Color;
        SelectFont;
      end;
end;

function BitBlt(DestDC: HDC; X, Y, Width, Height: Integer; SrcDC: HDC;
  XSrc, YSrc: Integer; Rop: DWORD): BOOL;
var
  Tmp : Longint;
begin
  Result := False;
  If (DestDC <> 0) and (SrcDC <> 0) then
    If (TObject(DestDC) is TCPhDevice) and
      (TObject(SrcDC) is TCPhDevice)
    then begin
      Case Rop of
        R2_White, R2_Black:
          begin
            Tmp := TCPhDevice(DestDC).ROPMode;
            SetRop2(DestDC,Rop);
	    Rectangle(DestDC, X, Y, Width, Height);
            SetRop2(DestDC,Tmp);
            Result := True;
          end;
        else begin
          Result := hDeviceClass.GUIBitBlt(DestDC, X, Y, width, height, SrcDC, XSrc, YSrc,Rop);
        end;
      end;
    end;
end;

function GetPixel(DC: HDC; X, Y: Integer): COLORREF;
begin
  Result := GetDCPenColor(DC);
  If (DC <> 0)then
    If (TObject(DC) is TCPhDevice) then
      Result := hDeviceClass.GUIGetPixel(DC,X,Y);
end;

function SetPixel(DC: HDC; X, Y: Integer; Color: COLORREF): COLORREF;
begin
  Result := GetPixel(DC,X,Y);
  SetPixelV(DC,X,Y,Color);
end;

function SetPixelV(DC: HDC; X, Y: Integer; Color: COLORREF): Integer;
var
  C : ColorRef;
  OldP,
  thePen : hPen;
begin
  Result := 1;
  If (DC <> 0)then
    If (TObject(DC) is TCPhDevice) then
      With TCPhDevice(DC) do begin
        SelectPen;
        thePen := CreatePen(PS_Solid,1,Color);
        OldP := SelectObject(DC,thePen);
        Result := hDeviceClass.GUIDrawPixel(DC,X,Y);
        SelectObject(DC,OldP);
        DeleteObject(thePen);
      end;
end;

function Arc(hDC: HDC; left, top, right, bottom, startX, startY,
  endX, endY: Integer): BOOL;
begin
  Result := False;
  If (hDC <> 0) then
    If TObject(hDC) is TCPhDevice then
      With TCPhDevice(hDC) do begin
        UpdatePlaceP;
        SelectPen;
        If TCPhPen(Pen).Style <> ps_null then
           hDeviceClass.GUIDrawArc(hDC, False, Left, Top, Right, Bottom,
             startx,starty,endx,endy);
        Result := True;
      end;
end;

function MoveToEx(DC: HDC; p2, p3: Integer; p4: PPoint): BOOL;
begin
  Result := False;
  If DC <> 0 then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        If P4 <> nil then begin
          P4^.X := PenPos.X;
          P4^.Y := PenPos.Y;
        end;
        PenPos.x := p2;
        PenPos.y := p3;
        Result := True;
      end;
end;

function LineTo(DC: HDC; X, Y: Integer): BOOL;
begin
  Result := False;
  If DC <> 0 then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        SelectPen;
        If TCPhPen(Pen).Style <> ps_null then
          hDeviceClass.GuiDrawLine(DC, PenPos.X,PenPos.Y,X,Y);
        MoveToEx(DC,X,Y,nil);
        Result := True;
      end;
end;

function Polyline(DC: HDC; var Points; Count: Integer): BOOL;
begin
  Result := False;
  If (DC <> 0) then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        UpdatePlaceP;
        SelectPen;
        If TCPhPen(Pen).Style <> ps_null then
          hDeviceClass.GUIDrawPolyline(DC,Points,Count,nil);
	Result := True;
      end;
end;

function PolylineTo(DC: HDC; var Points; Count: Integer): BOOL;
begin
  Result := False;
  If (DC <> 0) then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        UpdatePlaceP;
        SelectPen;
        If TCPhPen(Pen).Style <> ps_null then
          hDeviceClass.GUIDrawPolyline(DC,Points,Count,@PenPos);
	Result := True;
      end;
end;

function Chord(DC: HDC; X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer): BOOL;
begin
  Result := False;
  If (DC <> 0) then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        UpdatePlaceP;
        SelectBrush;
        If TCPhBrush(Brush).Style <> bs_null then
          hDeviceClass.GUIDrawChord(DC, True, X1, Y1, X2, Y2, X3,Y3,X4,Y4);
        SelectPen;
        If TCPhPen(Pen).Style <> ps_null then
           hDeviceClass.GUIDrawChord(DC, False, X1, Y1, X2, Y2, X3,Y3,X4,Y4);
        Result := True;
      end;
end;

function Ellipse(DC: HDC; X1, Y1, X2, Y2: Integer): BOOL;
begin
  Result := False;
  If (DC <> 0) then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        UpdatePlaceP;
        SelectBrush;
        If TCPhBrush(Brush).Style <> bs_null then
          hDeviceClass.GUIDrawEllipse(DC, True, X1, Y1, ABS(X2 - X1), ABS(Y2 - Y1));
        SelectPen;
        If TCPhPen(Pen).Style <> ps_null then
          hDeviceClass.GUIDrawEllipse(DC, False, X1, Y1, ABS(X2 - X1), ABS(Y2 - Y1));
        Result := True;
      end;
end;

function FillRect(hDC: HDC; const lprc: TRect; hbr: HBRUSH): Integer;
var
  Left,Top,
  Right,Bottom,
  Width, Height,
  PW   : Integer;
  OLDB : HBRUSH;
begin
  Result := 0;
  If (hDC <> 0) then
    If TObject(hDC) is TCPhDevice then
      With TCPhDevice(hDC) do begin
        Left := lprc.Left;
        Top := lprc.Top;
        Right := lprc.Right;
        Bottom := lprc.Bottom;
        UpdatePlaceP;
        Width := Right - Left;
        Height := Bottom - Top;
        OLDB := SelectObject(hDC, hBR);
        If TCPhBrush(hBR).Style = BS_HOLLOW then begin
          Result := 1;
          SelectObject(hDC, OLDB);
          exit;
        end;
        SelectPen;
        PW := TCPhPen(Pen).Width;
        TCPhPen(Pen).Width := 1;
        SelectPen;
        SelectBrush;
        hDeviceClass.GUIDrawRectangle(hDC, True, Left,
          Top, Width, Height);
        SelectObject(hDC, OLDB);
        TCPhPen(Pen).Width := PW;
        Result := 1;
      end;
end;

function FrameRect(hDC: HDC; const lprc: TRect; hbr: HBRUSH): Integer;
var
  Left,Top,
  Right,Bottom,
  Width,
  Height,
  PW   : Integer;
  OLDB : HBRUSH;
begin
  Result := 0;
  If (hDC <> 0) then
    If TObject(hDC) is TCPhDevice then
      With TCPhDevice(hDC) do begin
        Left := lprc.Left;
        Top := lprc.Top;
        Right := lprc.Right;
        Bottom := lprc.Bottom;
        UpdatePlaceP;
        Width := Right - Left;
        Height := Bottom - Top;
        OLDB := SelectObject(hDC, hBR);
        SelectPen;
        PW := TCPhPen(Pen).Width;
        TCPhPen(Pen).Width := 1;
        SelectPen;
        If TCPhPen(Pen).Style <> ps_null then
          hDeviceClass.GUIDrawRectangle(hDC, False, Left,
            Top, Width, Height);
        SelectBrush;
        If TCPhBrush(Brush).Style <> bs_null then
          hDeviceClass.GUIDrawRectangle(hDC, False, Left,
            Top, Width, Height);
        SelectObject(hDC, OLDB);
        TCPhPen(Pen).Width := PW;
        Result := 1;
      end;
end;

function InvertRect(hDC: HDC; const lprc: TRect): Integer;
begin
  Result := 0;
  If (hDC <> 0) then
    If TObject(hDC) is TCPhDevice then
      hDeviceClass.GUIInvertRect(hDC, lprc);
end;

function Pie(hDC: HDC; left, top, right, bottom, startX, startY,
  endX, endY: Integer): BOOL;
begin
  Result := False;
  If (hDC <> 0) then
    If TObject(hDC) is TCPhDevice then
      With TCPhDevice(hDC) do begin
        UpdatePlaceP;
        SelectBrush;
        If TCPhBrush(Brush).Style <> bs_null then
          hDeviceClass.GUIDrawArc(hDC, True, Left, Top, Right, Bottom,
            startx,starty,endx,endy);
        SelectPen;
        If TCPhPen(Pen).Style <> ps_null then begin
          hDeviceClass.GUIDrawArc(hDC, False, Left, Top, Right, Bottom,
            startx,starty,endx,endy);
        end;
	Result := True;
      end;
end;

function Polygon(DC: HDC; var Points; Count: Integer): BOOL;
begin
  Result := False;
  If (DC <> 0) then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        UpdatePlaceP;
        SelectBrush;
        If TCPhBrush(Brush).Style <> bs_null then begin
          hDeviceClass.GUIDrawPolygon(DC, True,Points,Count);
        end;
        SelectPen;
        If TCPhPen(Pen).Style <> ps_null then
          hDeviceClass.GUIDrawPolygon(DC, False,Points,Count);
        Result := True;
      end;
end;

function Rectangle(DC: HDC; X1, Y1, X2, Y2: Integer): BOOL;
var
  Width,
  Height : Integer;
begin
  Result := False;
  If DC <> 0 then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        Width := X2 - X1;
        Height := Y2 - Y1;
        SelectBrush;
        If TCPhBrush(Brush).Style <> bs_null then
          hDeviceClass.GUIDrawRectangle(DC, True, X1, Y1, Width, Height);
        SelectPen;
        If TCPhPen(Pen).Style <> ps_null then
          hDeviceClass.GUIDrawRectangle(DC, False, X1, Y1, Width, Height);
        Result := True;
      end;
end;

function RoundRect(DC: HDC; X1, Y1, X2, Y2, X3, Y3: Integer): BOOL;
var
  pt : TPoint;
  
  Procedure Switch(Var F,T : Integer);
  var
    Tmp : Integer;
  begin
    Tmp := F;
    F := T;
    T := Tmp
  end;
  
begin
  Result := False;
  If (DC <> 0) then
    If TObject(DC) is TCPhDevice then
      With TCPhDevice(DC) do begin
        UpdatePlaceP;
        If X2 < X1 then
	  Switch(X2,X1);
        If Y2 < Y1 then
	  Switch(Y2,Y1);
	If ((X2 - X1) < 0) or ((Y2 - Y1) < 0) then begin
	  Result := True;
	  exit;
	end;
	If not ((X3 <= 0) or (Y3 <= 0)) then begin
  	  If ((X2 - X1) <= X3) or ((X2 - X1) div 2 < X3)  then begin
	    X3 := (X2 - X1) div 2;
	  end;
	  If ((Y2 - Y1) <= Y3) or ((Y2 - Y1) div 2 < Y3) then begin
	    Y3 := (Y2 - Y1) div 2;
	  end;
	  SelectBrush;
	  If TCPhBrush(Brush).Style <> bs_null then begin
  	    GUIDrawArc(DC, True, X1, Y1, X1 + X3 + X3, Y1 + Y3 + Y3, 90,90);
            GUIDrawArc(DC, True, X2 - X3 - X3, Y1, X2, Y1 + Y3 + Y3, 0, 90);
            GUIDrawArc(DC, True, X1, Y2 - Y3 - Y3, X1 + X3 + X3, Y2, 180,90);
            GUIDrawArc(DC, True, X2 - X3 - X3, Y2 - Y3 - Y3, X2, Y2, 270,90);
  	    GUIDrawRectangle(DC, True, X1 + X3, Y1, X2 - X1 - X3*2, Y3);
	    GUIDrawRectangle(DC, True, X1, Y1 + Y3, X2 - X1, Y2 - Y1 - Y3*2);
	    GUIDrawRectangle(DC, True, X1 + X3, Y2 - Y3, X2 - X1 - X3*2, Y3);
	  end;
	  SelectPen;
	  If TCPhPen(Pen).Style <> ps_null then begin
  	    GUIDrawArc(DC, False, X1, Y1, X1 + X3 + X3, Y1 + Y3 + Y3, 90,90);
            GUIDrawArc(DC, False, X2 - X3 - X3, Y1, X2, Y1 + Y3 + Y3, 0, 90);
            GUIDrawArc(DC, False, X1, Y2 - Y3 - Y3, X1 + X3 + X3, Y2, 180,90);
            GUIDrawArc(DC, False, X2 - X3 - X3, Y2 - Y3 - Y3, X2, Y2, 270,90);
          end;
	end
	else begin
	  X3 := 0;
	  Y3 := 0;
	end;
	MoveToEx(DC, X1 + X3, Y1,@pt);
        LineTo(DC, X2 - X3, Y1);
        MoveToEx(DC, X1, Y1 + Y3,nil);
        LineTo(DC, X1, Y2 - Y3);
        MoveToEx(DC, X1 + X3, Y2,@pt);
        LineTo(DC, X2 - X3, Y2);
        MoveToEx(DC, X2, Y1 + Y3,nil);
        LineTo(DC, X2, Y2 - Y3);
        MoveToEx(DC, pt.X, pt.Y,nil);
	Result := True;
     end;
end;

Procedure InitializeDevices(NewDeviceClass : TCPhDeviceClass);
begin
  hDevices := TList.Create;
  hDeviceClass := NewDeviceClass;
end;

end.