{***********************************************************************}

{FileName  : cpwindows.pas                                              }

{Version*  : 1.0                                                        } 

{Author(s) : Andrew Johnson <AJ_Genius@Hotmail.com>                     }

{Copyright* 2001 Andrew Johnson                                         }

{***********************************************************************}

{Comments  : The Purpose of this file is to help make porting Delphi    }

{  code to non-windows platforms easier by emulating a small portion    }

{  of the routines in the Delphi Windows.pas file. For now this is      }

{  only going to include non-visual routines, as this is not an attempt }

{  to compete with or replace(as-if) the Wine project.			}

{***********************************************************************}

{Problems  :                              			        }

{  1.)Library Routines for OS/2 don't exist                             }

{  2.)OS/2 Time routines don't exist                                    }

{  3.)OS/2 Get*Thread Routines don't exist                              }

{  4.)Linux SetTime Routines don't exist yet(needs current 1.0.5)       }

{  5.)Linux GetTime Routines incomplete, they don't get proper time     }

{     relative to current TimeZone or UTC/GMC                           }

{  6.)Linux Get*Thread Routines don't exist                             }

{***********************************************************************}

{*See end of file for history and Copyright/License information         }

{***********************************************************************}

Unit CPWindows;



{$IfDef FPC}

  {$Mode Delphi}

{$EndIf}



{$IfDef Linux}

  {$Define libdl.so}

{$EndIf}



{$IfDef FreeBSD}

  {$Define libdl.so}

{$EndIf}



interface

 {$I CPWinTypes.Inc}



  Function GetBValue(rgb : longint) : BYTE;

  Function GetGValue(rgb : longint) : BYTE;

  Function GetRValue(rgb : longint) : BYTE;

  Function RGB(r,g,b : longint) : DWORD;



  Function GetProcAddress(Handle : hModule; ProcName : PChar) : FARPROC;

  Function LoadLibraryA(Name : PAnsiChar) : hModule;

  Function LoadLibrary(Name : PChar) : hModule;

  Function LoadLibraryW(Name : PWideChar) : hModule;



  Function FreeLibrary(Handle : hModule) : Boolean;



  Function TerminateThread(hThread : Handle; dwExitCode : DWORD):WINBOOL;

  Function SuspendThread(hThread : Handle):DWORD;

  Function ResumeThread(hThread : Handle):DWORD;

  Function GetCurrentThread: THandle;

  Function GetCurrentThreadId: DWORD;

  Function GetWindowThreadProcessId(hWnd: HWND; lpdwProcessId: Pointer): DWORD;



  Function MulDiv(nNumber, nNumerator, neDenominator : Longint): Longint;



  Procedure GetSystemTime(lpSystemTime:LPSYSTEMTIME);

  Function SetSystemTime(var lpSystemTime:SYSTEMTIME):WINBOOL;

  Procedure GetLocalTime(lpSystemTime:LPSYSTEMTIME);

  Function SetLocalTime(var lpSystemTime:SYSTEMTIME):WINBOOL;



  Function Rect(ALeft,ATop,ARight,ABottom : Integer): TRect;

  Function OffsetRect(lprc: PRect; dx, dy: Integer): BOOL;



  Function MakeWord(A, B: Byte): Word;

  Function MakeLong(A, B: Word): Longint;

  Function MakeWParam(l, h: Word): WPARAM;

  Function MakeLParam(l, h: Word): LPARAM;

  Function MakeLResult(l, h: Word): LRESULT;

  Function HiWord(L: DWORD): Word;

  Function HiByte(W: Word): Byte;



  function GetVersionEx(var lpVersionInformation: TOSVersionInfo): BOOL;



  procedure InitializeCriticalSection(lpCriticalSection: LPCRITICAL_SECTION); Overload;

  procedure EnterCriticalSection(lpCriticalSection: LPCRITICAL_SECTION); Overload;

  procedure LeaveCriticalSection(lpCriticalSection: LPCRITICAL_SECTION); Overload;

  procedure DeleteCriticalSection(lpCriticalSection: LPCRITICAL_SECTION); Overload;



  procedure InitializeCriticalSection(lpCriticalSection: CRITICAL_SECTION); Overload;

  procedure EnterCriticalSection(lpCriticalSection: CRITICAL_SECTION); Overload;

  procedure LeaveCriticalSection(lpCriticalSection: CRITICAL_SECTION); Overload;

  procedure DeleteCriticalSection(lpCriticalSection: CRITICAL_SECTION); Overload;



implementation



{$IfDef Win32}

Uses Windows;

{$EndIf}

{$IfDef OS2}

Uses DosCalls;

{$EndIf}

{$IfDef Linux}

Uses Linux, SysUtils{,DateRoutines};

{$Define LinuxTime}

{$EndIf}

{$IfDef FreeBSD}

Uses Linux, SysUtils{,DateRoutines};{I certainly hope that there will soon be a FreeBSD.pas}

{$Define LinuxTime}

{$EndIf}



Function GetBValue(rgb : longint) : BYTE;

begin

  Result := (rgb shr 16) and $000000ff;

end;



Function GetGValue(rgb : longint) : BYTE;

begin

  Result := (rgb shr 8) and $000000ff;

end;



Function GetRValue(rgb : longint) : BYTE;

begin

  Result := rgb and $000000ff;

end;



Function RGB(r,g,b : longint) : DWORD;

begin

  Result := r or (g shl 8) or (b shl 16);

end;



{$IfDef libdl.so}

  {$linklib c}

  {$linklib dl}

  Function dlopen(name: pchar;mode: longint):pointer;cdecl;external;

  Function dlsym(lib: pointer; name: pchar):pointer;cdecl;external;

  Function dlclose(lib: pointer):longint;cdecl;external;

{$EndIF}



Function GetProcAddress(Handle : hModule; ProcName : PChar) : FARPROC;

begin

  {$Ifdef Win32}

    Result := Windows.GetProcAddress(Handle, ProcName);

  {$EndIf}

  {$Ifdef OS2}

    This has not been implimented -

      Does OS2 Support Dynamic Loading?

  {$EndIf}

  {$Ifdef Linux}

    Result := dlsym(Pointer(Handle), ProcName);

  {$EndIf}

  {$Ifdef FreeBSD}

    Result := dlsym(Pointer(Handle), ProcName);

  {$EndIf}

end;



Function LoadLibraryA(Name : PAnsiChar) : hModule;

begin

  Result := LoadLibrary(Name);

end;



Function LoadLibrary(Name : PChar) : hModule;

begin

  {$Ifdef Win32}

    Result := Windows.LoadLibrary(Name);

  {$EndIf}

  {$Ifdef OS2}

    This has not been implimented -

      Does OS2 Support Dynamic Loading?

  {$EndIf}

  {$Ifdef Linux}

    Result := hModule(dlOpen(Name,1));

  {$EndIf}

  {$Ifdef FreeBSD}

    Result := hModule(dlOpen(Name,1));

  {$EndIf}

end;



Function LoadLibraryW(Name : PWideChar) : hModule;

begin

  Result := LoadLibrary(PChar(AnsiString(Name)));

end;



Function FreeLibrary(Handle : hModule) : Boolean;

begin

  {$Ifdef Win32}

    Result := Windows.FreeLibrary(Handle);

  {$EndIf}

  {$Ifdef OS2}

    This has not been implimented -

      Does OS2 Support Dynamic Loading?

  {$EndIf}

  {$Ifdef Linux}

    Result := dlClose(Pointer(Handle)) = 0;

  {$EndIf}

  {$Ifdef FreeBSD}

    Result := dlClose(Pointer(Handle)) = 0;

  {$EndIf}

end;



Function TerminateThread(hThread : Handle; dwExitCode : DWORD):WINBOOL;

begin

  {$IfDef Win32}

    Result := Windows.TerminateThread(hThread, dwExitCode);

  {$EndIf}

  {$IfDef OS2}

    Result := DosKillThread(hThread) = 0;

  {$EndIf}

  {$IfDef Linux}

    if hThread <> -1 then

      Result := Kill(hThread, SIGKILL) = 0

    else

      Result := False;  

  {$EndIf}

  {$IfDef FreeBSD}

    if hThread <> -1 then

      Result := Kill(hThread, SIGKILL) = 0 

    else

      Result := False;  

  {$EndIf}

end;



Function SuspendThread(hThread : Handle):DWORD;

begin

  {$IfDef Win32}

    Result := Windows.SuspendThread(hThread);

  {$EndIf}

  {$IfDef OS2}

    Result := DosSuspendThread(hThread);

  {$EndIf}

  {$IfDef Linux}

    Result := Linux.Kill(hThread, SIGSTOP);

  {$EndIf}

  {$IfDef FreeBSD}

    Result := Linux.Kill(hThread, SIGSTOP);

  {$EndIf}

end;



Function ResumeThread(hThread : Handle):DWORD;

begin

  {$IfDef Win32}

    Result := Windows.ResumeThread(hThread);

  {$EndIf}

  {$IfDef OS2}

    Result := DosResumeThread(hThread);

  {$EndIf}

  {$IfDef Linux}

    Result := Linux.Kill(hThread, SIGCONT);

  {$EndIf}

  {$IfDef FreeBSD}

    Result := Linux.Kill(hThread, SIGSTOP);

  {$EndIf}

end;



function GetCurrentThread: THandle;

begin

  {$IfDef Win32}

    Result := Windows.GetCurrentThread;

  {$EndIf}

  {$IfDef OS2}

    Result := ??;

  {$EndIf}

  {$IfDef Linux}

   // Result := ??;

  {$EndIf}

  {$IfDef FreeBSD}

    Result := ??;

  {$EndIf}

end;



function GetCurrentThreadId: DWORD;

begin

  {$IfDef Win32}

    Result := Windows.GetCurrentThreadID;

  {$EndIf}

  {$IfDef OS2}

    Result := ??;

  {$EndIf}

  {$IfDef Linux}

   // Result := ??;

  {$EndIf}

  {$IfDef FreeBSD}

    Result := ??;

  {$EndIf}

end;



function GetWindowThreadProcessId(hWnd: HWND; lpdwProcessId: Pointer): DWORD;

begin

  {$IfDef Win32}

  Result := Windows.GetWindowThreadProcessId(hWnd, lpdwProcessId);

  {$EndIf}

  {$IfDef OS2}

    Result := ??;

  {$EndIf}

  {$IfDef Linux}

    //Result := ??;

  {$EndIf}

  {$IfDef FreeBSD}

    Result := ??;

  {$EndIf}

end;



Function MulDiv(nNumber, nNumerator, neDenominator : Longint): Longint;

begin

  Result := (nNumber*nNumerator) div neDenominator;

end;



{$IfDef LinuxTime}

Procedure GetLinuxSystemTime(lpSystemTime:LPSYSTEMTIME);

var

  T : Timeval;

begin

  Writeln('GetLinuxSystemTime Called -- THIS DOES NOT GET UTC/GMC!!!!');

  GetTimeOfDay(T);

  With lpSystemTime^ do begin

    EpochToLocal(T.Sec, wYear, wMonth, wDay, wHour, wMinute, wSecond);

    wMilliseconds := T.usec div 1000;

    //wDayOfWeek := DayOfWeek(EncodeDate(wYear, wMonth, wDay));

  end;

end;



Procedure GetLinuxLocalTime(lpSystemTime:LPSYSTEMTIME);

begin

  Writeln('GetLinuxLocalTime Called -- THIS DOES NOT WORK!!!!');

end;



Procedure SetLinuxSystemTime(lpSystemTime:SYSTEMTIME);

begin

  Writeln('SetLinuxSystemTime Called -- THIS DOES NOT WORK!!!!');

end;



Procedure SetLinuxLocalTime(lpSystemTime:SYSTEMTIME);

begin

  Writeln('SetLinuxLocalTime Called -- THIS DOES NOT WORK!!!!');

end;

{$EndIf}



Procedure GetSystemTime(lpSystemTime:LPSYSTEMTIME);

begin

  {$IfDef Win32}

    Windows.GetSystemTime(

        {$IfDef FPC}

      Windows.LPSYSTEMTIME(lpSystemTime)

    {$Else}

      Windows.PSystemTime(lpSystemTime)^

    {$EndIf}

      );

  {$EndIf}

  {$IfDef OS2}

   Implement this!!

  {$EndIf}

  {$IfDef LinuxTime}

    GetLinuxSystemTime(lpSystemTime);

  {$EndIf}

end;



Function SetSystemTime(var lpSystemTime:SYSTEMTIME):WINBOOL;

begin

  {$IfDef Win32}

    Result := Windows.SetSystemTime(

    {$IfDef FPC}

      Windows.SYSTEMTIME(lpSystemTime)

    {$Else}

      Windows.TSystemTime(lpSystemTime)

    {$EndIf}

      );

  {$EndIf}

  {$IfDef OS2}

   Implement this!!

  {$EndIf}

  {$IfDef LinuxTime}

    SetLinuxSystemTime(lpSystemTime);

  {$EndIf}

end;



Procedure GetLocalTime(lpSystemTime:LPSYSTEMTIME);

begin

  {$IfDef Win32}

    Windows.GetLocalTime(

    {$IfDef FPC}

      Windows.LPSYSTEMTIME(lpSystemTime)

    {$Else}

      Windows.PSystemTime(lpSystemTime)^

    {$EndIf});

  {$EndIf}

  {$IfDef OS2}

   Implement this!!

  {$EndIf}

  {$IfDef LinuxTime}

    GetLinuxLocalTime(lpSystemTime);

  {$EndIf}

end;



Function SetLocalTime(var lpSystemTime:SYSTEMTIME):WINBOOL;

begin

  {$IfDef Win32}

    Result := Windows.SetLocalTime(

    {$IfDef FPC}

      Windows.SYSTEMTIME(lpSystemTime)

    {$Else}

      Windows.TSystemTime(lpSystemTime)

    {$EndIf}

      );

  {$EndIf}

  {$IfDef OS2}

   Implement this!!

  {$EndIf}

  {$IfDef LinuxTime}

    SetLinuxLocalTime(lpSystemTime);

  {$EndIf}

end;



Function Rect(ALeft,ATop,ARight,ABottom : Integer): TRect;

begin

  With Result do begin

    Left  := Aleft;

    Top   := ATop;

    Right := ARight;

    Bottom:= ABottom;

  end;

end;



function OffsetRect(lprc: PRect; dx, dy: Integer): BOOL;

begin

  Result := False;

  If lprc <> nil then

    exit;

  With lprc^ do begin

    Left := Left + dx;

    Right := Right + dx;

    Top := Top + dy;

    Bottom := Bottom + dy;

  end;

  Result := True;

end;



function MakeWord(A, B: Byte): Word;

begin

  Result := A or B shl 8;

end;



function MakeLong(A, B: Word): Longint;

begin

  Result := A or B shl 16;

end;



function MakeWParam(l, h: Word): WPARAM;

begin

  Result := MakeLong(l, h);

end;



function MakeLParam(l, h: Word): LPARAM;

begin

  Result := MakeLong(l, h);

end;



function MakeLResult(l, h: Word): LRESULT;

begin

  Result := MakeLong(l, h);

end;



function HiWord(L: DWORD): Word;

begin

  Result := L shr 16;

end;



function HiByte(W: Word): Byte;

begin

  Result := W shr 8;

end;



function GetVersionEx(var lpVersionInformation: TOSVersionInfo): BOOL;

begin

  Result := True;

  With lpVersionInformation do begin

    dwMajorVersion :=  4;

    dwMinorVersion := 10;

    dwBuildNumber := 1998;

    dwPlatformId := VER_PLATFORM_WIN32s;

  end;

end;



procedure InitializeCriticalSection(lpCriticalSection: LPCRITICAL_SECTION);

begin



end;



procedure EnterCriticalSection(lpCriticalSection: LPCRITICAL_SECTION);

begin



end;



procedure LeaveCriticalSection(lpCriticalSection: LPCRITICAL_SECTION);

begin



end;



procedure DeleteCriticalSection(lpCriticalSection: LPCRITICAL_SECTION);

begin



end;



procedure InitializeCriticalSection(lpCriticalSection: CRITICAL_SECTION);

begin



end;



procedure EnterCriticalSection(lpCriticalSection: CRITICAL_SECTION);

begin



end;



procedure LeaveCriticalSection(lpCriticalSection: CRITICAL_SECTION);

begin



end;



procedure DeleteCriticalSection(lpCriticalSection: CRITICAL_SECTION);

begin



end;



{$Warnings Off}

end.

{***********************************************************************}

{                      Copyright/License Information                    }

{***********************************************************************}

{  This file falls under the LGPL, or the Library General Public        }

{  License, aka the Lesser General Public License                       }

{                                                                       }

{for more information regarding the LGPL see the included COPYING.LIB   }

{***********************************************************************}

{                            FILE HISTORY                               }

{***********************************************************************}

{Version 1.0 (Andrew Johnson, 6/3/2001)                                 }

{                                                                       }

{  1.) Initial import of the library routines from my                   }

{      libraryroutines.pas, added flags for FreeBSD,                    }

{      and a place holder for OS2. Added LoadLibraryW,                  }

{      will probably need a RW.                                         }

{  2.) Initial import of the color routines from my                     }

{      colorroutines.pas - modifying the names/Types                    }

{  3.) Initial import of the Thread routines from my                    }

{      threadroutines.pas- very incomplete at the moment.               }

{  4.) Wrote MulDiv                                                     }

{  5.) Imported and modified System/LocalTime Procs from old projects   }

{***********************************************************************}

{Version 1.0.1 (Andrew Johnson, 7/26/2001)                              }

{                                                                       }

{  1.) Added Rect Routine                                               }

{***********************************************************************}

{Version 1.0.2 (Andrew Johnson, 7/30/2001)                              }

{                                                                       }

{  1.) Added MakeWord, MakeLong, MakewParam, MakeLParam,                }

{      MakeLResult, HiWord, HiByte routines                             }

{  2.) Added GetWindowThreadProcess, GetCurrentThreadId,                }

{      GetCurrentThreadHandle, Routines.                                }

{***********************************************************************}



