Unit CPWinObjects;

{$IfDef FPC}
  {$Mode Delphi}
{$EndIf}
 
interface

Uses Classes, CPwindows, CPWinControls;

Type
  TCPhObject = Class(TComponent)
  Public
    Procedure hDCSelect(DC : hDC); Virtual;  
  end;
  
  TCPhPen = Class(TCPhObject)
  Public
    Color : TCOLORREF;
    Width : Cardinal;
    Style : Longint;
    
    Class Function CreatePenIndirect(const LogPen: TLogPen): HPEN; Virtual;
    Class Function CreatePen(Style : Integer; Width : Integer;
      Color : ColorRef): HPEN; Virtual;
  end;

  TCPhPenClass = Class of TCPhPen;
 
  TCPhBrush = Class(TCPhObject)
  Public
    Color : TCOLORREF;
    Style : Longint;
    Hatch : Longint;
    Class Function CreateBrushIndirect(const LogBrush: TLogBrush): HBRUSH; Virtual;
  end;

  TCPhBrushClass = Class of TCPhBrush;

  TCPhFont = Class(TCPhObject)
  Public
     Italic,
     Underline,
     StrikeOut  : Boolean;
     Weight     : Longint;
     Name       : PChar;
     Angle      : Integer;
     Color      : TCOLORREF;
     Size       : Integer;
     Class Function CreateFontIndirect(const LogFont: PLogFont): HFont; Virtual;
  end;

  TCPhFontClass = Class of TCPhFont;

  TCaret = Record
    Win       : hwnd;
    Pos       : TPoint;
    Size      : TSize;
    BlinkTime : integer;
    Timer     : Longint;
    Enabled   : Boolean;
    Brush     : hBrush;
  end;

  Function CreatePenIndirect(const LogPen: TLogPen): HPEN;
  Function CreatePen(Style : Integer; Width : Integer;
    Color : ColorRef): HPEN;

  Function CreateBrushIndirect(const LogBrush: TLogBrush): HBRUSH;
  Function CreateHatchBrush(nIndex : Longint; Color : TColorRef): HBRUSH;
  Function CreateSolidBrush(Color : TColorRef): HBRUSH;
  
  Function CreateFontIndirect(const LogFont: PLogFont): HFont;

  function CreateCaret(hWnd: HWND; hBitmap: HBITMAP; nWidth, nHeight: Integer): BOOL;
  function GetCaretBlinkTime: UINT;
  function SetCaretBlinkTime(uMSeconds: UINT): BOOL;
  function DestroyCaret: BOOL;
  function HideCaret(hWnd: HWND): BOOL;
  function ShowCaret(hWnd: HWND): BOOL;
  function SetCaretPos(X, Y: Integer): BOOL;
  function GetCaretPos(lpPoint: PPoint): BOOL;

  function GetObject(p1: HGDIOBJ; p2: Integer; p3: Pointer): Integer;
  function GetStockObject(Index: Integer): HGDIOBJ;
  Function DeleteObject(var p1: HGDIOBJ): BOOL;
  function UnrealizeObject(hGDIObj: HGDIOBJ): BOOL;

  Procedure InitializePens(NewPenClass : TCPhPenClass);
  Procedure InitializeBrushes(NewBrushClass : TCPhBrushClass);
  Procedure InitializeFonts(NewFontClass : TCPhFontClass);

  Function AsLogFont(P : hFont) : TLOGFONT;

Const
  Default_Caret : TCaret =
    ( Win : 0;
      Pos : (X : 0; Y : 0);
      Size :(cX : 2; cY : 12);
      BlinkTime : 500;
      Timer : 0;
      Enabled : FALSE;
      Brush : 0;);
  Default_Brush : TLogBrush =
    (lbStyle : BS_SOLID;
     lbColor : $FFFFFF;
     lbHatch : 0;);
  Default_Pen : TLogPen =
    (lopnStyle : PS_SOLID;
     lopnWidth : (x : 1; y : 0);
     lopnColor : $FFFFFF;);
  Default_Font : TLogFont =
    (lfHeight : 8;
     lfWidth : 0;
     lfEscapement : 0;
     lfOrientation : 0;
     lfWeight : FW_DontCare;
     lfItalic : 0;
     lfUnderline : 0;
     lfStrikeOut : 0;
     lfCharSet : 0;
     lfOutPrecision : 0;
     lfClipPrecision : 0;
     lfQuality : 0;
     lfPitchAndFamily : 0;
     lfFaceName : 'Times';);

var
  hPens : TList;
  hBrushes : TList;
  hFonts : TList;

  hPenClass : TCPhPenClass;
  hBrushClass : TCPhBrushClass;
  hFontClass : TCPhFontClass;

  DEFAULTFONT : hFont;
  WHITEBRUSH, LTGRAYBRUSH,
  GRAYBRUSH, DKGRAYBRUSH,
  BLACKBRUSH, NULLBRUSH : hBrush;
  WHITEPEN,BLACKPEN,
  NULLPEN : hPen;
  Caret : TCaret;
  
implementation

Procedure TCPhObject.hDCSelect(DC : hDC);  
begin
end;

Class Function TCPhPen.CreatePenIndirect(const LogPen: TLogPen): HPEN;
var
  Obj : TCPhPen;
begin
  Result := 0;
  Obj :=hPenClass.Create(dummyP);
  hPens.Add(Obj);
  With Obj, LogPen do begin
    Style := lopnStyle;
    Width := lopnWidth.X;
    Color := lopnColor;
  end;
  Result := Longint(Obj);
end;

Class Function TCPhPen.CreatePen(Style : Integer; Width : Integer;
  Color : ColorRef): HPEN;
var
  Obj : TCPhPen;
begin
  Result := 0;
  Obj := hPenClass.Create(dummyP);
  hPens.Add(Obj);
  Obj.Style := Style;
  Obj.Width := Width;
  Obj.Color := Color;
  Result := Longint(Obj);
end;

Class Function TCPhBrush.CreateBrushIndirect(const LogBrush: TLogBrush): HBRUSH;
var
  Obj : TCPhBrush;
begin
  Result := 0;
  Obj := hBrushClass.Create(dummyP);
  hBrushes.Add(Obj);
  With Obj, LogBrush do begin
    Style := lbStyle;
    Hatch := lbHatch;
    Color := lbColor;
  end;
  Result := Longint(Obj);
end;

Class Function TCPhFont.CreateFontIndirect(const LogFont: PLogFont): HFont;
var
  Obj : TCPhFont;
begin
  Result := 0;
  Obj := hFontClass.Create(dummyP);
  hFonts.Add(Obj);
  With Obj, LogFont^ do begin
    Italic    := Boolean(lfItalic);
    Underline := Boolean(lfUnderline);
    StrikeOut := Boolean(lfStrikeOut);
    Weight    := lfWeight;
    Name      := lfFaceName;
    Angle     := lfEscapement;
    Color     := -1;
  end;
  Result := Longint(Obj);
end;

function CreateCaret(hWnd: HWND; hBitmap: HBITMAP; nWidth, nHeight: Integer): BOOL;
var
  theBrush : TLogBrush;
begin
  If (hBitmap = 0) or (Caret.Brush  <> 0) then
    exit;
  Caret.Win := hWnd;
  theBrush := Default_Brush;
  theBrush.lbHatch := hBitmap;
  theBrush.lbStyle := bs_Hatched;
  Caret.Brush := CreateBrushIndirect(Default_Brush);
  Caret.Size.cx := nWidth;
  Caret.Size.cy := nHeight;
  Result := True;
end;

function GetCaretBlinkTime: UINT;
begin
  If Caret.Brush = 0 then
    Result := Default_Caret.BlinkTime
  else
    Result := Caret.BlinkTime;
end;

function SetCaretBlinkTime(uMSeconds: UINT): BOOL;
begin
  If Caret.Brush = 0 then
    exit;
  Caret.BlinkTime := uMSeconds;
  Result := True;
end;

function DestroyCaret: BOOL;
begin
  If Caret.Brush = 0 then
    exit;
  Caret := Default_Caret;
  DeleteObject(Caret.Brush);
  Result := True;
end;

function HideCaret(hWnd: HWND): BOOL;
begin

end;

function ShowCaret(hWnd: HWND): BOOL;
begin
end;

function SetCaretPos(X, Y: Integer): BOOL;
begin
  If Caret.Brush = 0 then
    exit;
  Caret.Pos.X := X;
  Caret.Pos.Y := Y;
  Result := True;
end;

function GetCaretPos(lpPoint: PPoint): BOOL;
begin
  If (Caret.Brush = 0) or (lpPoint = nil) then
    exit;
  lpPoint.X := Caret.Pos.X;
  lpPoint.Y := Caret.Pos.Y;
  result := True;
end;

function GetObject(p1: HGDIOBJ; p2: Integer; p3: Pointer): Integer;
begin
  Result := 0;
  If P1 <> 0 then begin
    If TObject(p1) is TCPhFont then begin
      If P3 <> nil then
        With TCPhFont(p1),PlogFont(P3)^ do begin
          lfItalic := Integer(Italic);
          lfUnderline := Integer(Underline);
          lfStrikeOut := Integer(StrikeOut);
          lfWeight  := Weight;
          lfFaceName := Name;
          lfEscapement := Angle;
          lfHeight  := Size;
        end;
    end;
  end;
end;

function DeleteObject(var p1: HGDIOBJ): BOOL;
var
  L,I : Longint;
begin
 Result := False;
 If p1 <> 0 then
  If (TObject(p1) is hPenClass) then begin
    L := hPens.IndexOf(TObject(P1));
    If L > -1 then begin
      hPens.Delete(L);
      FreeAndNil(p1);
      Result := True;
    end;
  end
  else
    If (TObject(p1) is hBrushClass) then begin
      L := hBrushes.IndexOf(TObject(P1));
      If L > -1 then begin
        hBrushes.Delete(L);
        FreeAndNil(P1);
        Result := True;
      end;
    end
  else
    If (TObject(p1) is hFontClass) then begin
      L := hFonts.IndexOf(TObject(P1));
      If L > -1 then begin
        hFonts.Delete(L);
        FreeAndNil(P1);
        Result := True;
      end;
    end;
end;

function UnrealizeObject(hGDIObj: HGDIOBJ): BOOL;
begin
  Result := DeleteObject(hGDIObj);
end;

Function CreatePenIndirect(const LogPen: TLogPen): HPEN;
begin
  Result := hPenClass.CreatePenIndirect(LogPen);
end;

Function CreatePen(Style : Integer; Width : Integer;
  Color : ColorRef): HPEN;
begin
  Result := hPenClass.CreatePen(Style, Width, Color);
end;

Function CreateBrushIndirect(const LogBrush: TLogBrush): HBRUSH;
begin
  Result := hBrushClass.CreateBrushIndirect(LogBrush);
end;

Function CreateHatchBrush(nIndex : Longint; Color : TColorRef): HBRUSH;
var
  Logbrush : TLogBrush;
begin
  With LogBrush do begin
    lbStyle := bs_Hatched;
    lbHatch := nIndex;
    lbColor := Color;
  end;
  Result := CreateBrushIndirect(LogBrush);
end;

Function CreateSolidBrush(Color : TColorRef): HBRUSH;
var
  Logbrush : TLogBrush;
begin
  With LogBrush do begin
    lbStyle := bs_Solid;
    lbHatch := 0;
    lbColor := Color;
  end;
  Result := CreateBrushIndirect(LogBrush);
end;

Function CreateFontIndirect(const LogFont: PLogFont): HFont;
begin
  Result := hFontClass.CreateFontIndirect(LogFont);
end;

function GetStockObject(Index: Integer): HGDIOBJ;
begin
  Case Index Of
   WHITE_BRUSH:
      begin
        If WhiteBrush = 0 then
          WHITEBRUSH := CreateBrushIndirect(Default_Brush);
        Result := WHITEBRUSH;
      end;
    LTGRAY_BRUSH:
      begin
        If LTGRAYBRUSH = 0 then begin
          Default_Brush.lbColor := $C0C0C0;
          LTGRAYBRUSH := CreateBrushIndirect(Default_Brush);
          Default_Brush.lbColor := $FFFFFF;
        end;
        Result := LTGRAYBRUSH;
      end;
    GRAY_BRUSH:
      begin
        If GRAYBRUSH = 0 then begin
          Default_Brush.lbColor := $808080;
          GRAYBRUSH := CreateBrushIndirect(Default_Brush);
          Default_Brush.lbColor := $FFFFFF;
        end;
        Result := GRAYBRUSH;
      end;
    DKGRAY_BRUSH:
      begin
        If DKGRAYBRUSH = 0 then begin
          Default_Brush.lbColor := $7F7F7F;
          DKGRAYBRUSH := CreateBrushIndirect(Default_Brush);
          Default_Brush.lbColor := $FFFFFF;
        end;
        Result := DKGRAYBRUSH;
      end;
    BLACK_BRUSH:
      begin
        If BLACKBRUSH = 0 then begin
          Default_Brush.lbColor := $0;
          BLACKBRUSH := CreateBrushIndirect(Default_Brush);
          Default_Brush.lbColor := $FFFFFF;
        end;
        Result := BLACKBRUSH;
      end;
    NULL_BRUSH:
     begin
        If NULLBRUSH = 0 then begin
          Default_Brush.lbStyle := BS_Null;
          NULLBRUSH := CreateBrushIndirect(Default_Brush);
          Default_Brush.lbStyle := BS_Solid;
        end;
       Result := NULLBRUSH;
     end;
    WHITE_PEN:
      begin
        If WHITEPEN = 0 then
          WHITEPEN := CreatePenIndirect(Default_Pen);
        Result := WHITEPEN;
      end;
    BLACK_PEN:
      begin
        If BLACKPEN = 0 then begin
          Default_Pen.lopnColor := $0;
          BLACKPEN := CreatePenIndirect(Default_Pen);
          Default_Pen.lopnColor := $FFFFFF;
        end;
        Result := BLACKPEN;
      end;
    NULL_PEN:
     begin
        If NULLPEN = 0 then begin
          Default_Pen.lopnStyle := PS_Null;
          NULLPEN := CreatePenIndirect(Default_Pen);
          Default_Pen.lopnStyle := PS_Solid;
        end;
       Result := NULLPEN;
     end;
    SYSTEM_FONT:
      begin
        Result := DEFAULTFONT;
      end;
    else
      Result := 0;
  end;
end;

Procedure InitializePens(NewPenClass : TCPhPenClass);
begin
  hPens := TList.Create;
  hPenClass := NewPenClass;
  WHITEPEN := 0;
  GetStockObject(white_pen);
  BLACKPEN := 0;
  GetStockObject(Black_pen);
  NULLPEN  := 0;
  GetStockObject(null_pen);
end;

Procedure InitializeBrushes(NewBrushClass : TCPhBrushClass);
begin
  hBrushes := TList.Create;
  hBrushClass := NewBrushClass;
  Default_Brush.lbColor := $FFFFFF;
  WHITEBRUSH := 0;
  GetStockObject(white_Brush);
  LTGRAYBRUSH := 0;
  GetStockObject(LTGRAY_BRUSH);
  GRAYBRUSH := 0;
  GetStockObject(GRAY_BRUSH);
  DKGRAYBRUSH := 0;
  GetStockObject(DKGRAY_BRUSH);
  BLACKBRUSH := 0;
  GetStockObject(Black_BRUSH);
  NULLBRUSH := 0;
  GetStockObject(Null_BRUSH);
end;

Procedure InitializeFonts(NewFontClass : TCPhFontClass);
begin
  hFonts := TList.Create;
  hFontClass := NewFontClass;
  DEFAULTFONT := CreateFontIndirect(@Default_Font);
  TCPhFont(DEFAULTFONT).Color := $0;
end;

Function AsLogFont(P : hFont) : TLOGFONT;
begin
  With TCPhFont(P), Result do begin
    lfItalic      := Byte(Italic);
    lfUnderline   := Byte(Underline);
    lfStrikeOut   := Byte(StrikeOut);
    lfWeight      := Weight;
    lfFaceName    := Name;
    lfEscapement  := Angle;
    lfOrientation := Angle;
    lfHeight      := Size;
  end;
end;

end.