unit GDKRoutines;

{$IFDef fpc}
  {$Mode ObjFPC}
  {$IFDef Win32}
    {$Define ARRAY_OF_CHAR}
  {$Else}
    {$Define ARRAY_OF_CHAR}
  {$EndIf}
{$EndIf}

interface

Uses
  Classes, SysUtils, Math,toSTr,
  {$IFNDef Win32}
  X, Xlib,
  {$EndIF}
  GLIB, GDK, CPWindows, XRoutines;

Function GDKGetArcPoints(Left, Top, Right, Bottom,
  StartX, StartY, EndX, EndY : Integer) : PGDKPoint;
Procedure GDKUpdateFonts;
Function GDKGetPoints(var Points; Count : Integer; Close : Boolean; StartPt : PPoint) : PGDKPoint;
Procedure GDKSetDashStyle(GC : PGDKGC; Style : Longint);
Function GDKGetColor(Color : TCOLORREF) : PGDKColor;
Function GDKGetPenStyle(Style : Longint) : TGdkLineStyle;
procedure GDKSetBrushBitmap(GC : PGDKGC; BrushStyle, BrushHatch : Integer);
Procedure GDKSetRop2(GC : PGDKGC; ROP2 : Integer);
Function GDKGetFont(Font : TLogFont) : PGDKFont;
Function GDKGetPixel(Drawable : PGDKWindow; X, Y : Integer) : TColorRef;
Procedure GDKArc(drawable : PGdkDrawable; gc : PGdkGC; filled : Boolean;
  X1,Y1,X2,Y2,X3,Y3,X4,Y4 : Integer); Overload;
Procedure GDKArc(drawable : PGdkDrawable; gc : PGdkGC; filled : Boolean;
  X1,Y1,X2,Y2,Angle1,Angle2 : Integer); Overload;
Procedure GDKChord(drawable : PGdkDrawable; gc : PGdkGC; filled : Boolean;
  X1,Y1,X2,Y2,X3,Y3,X4,Y4 : Integer);
function GDKPolyline(drawable : PGdkDrawable; gc : PGdkGC; 
  var Points; Count: Integer; PenPos : PPoint): BOOL;
function GDKPolygon(drawable : PGdkDrawable; gc : PGdkGC; Filled : Boolean; 
  var Points; Count: Integer): BOOL;
function GDKBitBlt(Dest : PGdkDrawable; GC : PGdkGC; X, Y, Width, Height: Integer; 
  Src : PGdkDrawable; XSrc, YSrc: Integer): BOOL;
function GDKStrechBlt(Dest : PGdkDrawable; GC : PGdkGC; X, Y, Width, Height: Integer; 
  Src : PGdkDrawable; XSrc, YSrc, SrcWidth, SrcHeight: Integer): BOOL;
Procedure GDKSetForegroundColor(GC : PGDKGC; Color : TColorRef);
Procedure GDKSetBackgroundColor(GC : PGDKGC; Color : TColorRef);
Function GDKGetCursorPos(var pos : TPoint) : Boolean;
Procedure GDKSetCursorPos(X, Y : Integer);

implementation

Type
  PGDKWindowPrivate = ^ TGDKWindowPrivate;
  TGDKWindowPrivate = Record
    Window : TGDKWindow;
    Parent : PGDKWindow;
    XWindow : X.TWindow;
    Display : PDisplay;
    X : guint16;
    Y : guint16;
    Width : guint16;
    Height : guint16;
  end;

  PGDKColorMapPrivate = ^TGDKColorMapPrivate;
  TGDKColorMapPrivate = Record 
    ColorMap : TGDKColorMap;
    XColorMap : TColorMap;
    XDisplay : PDisplay;
  end;
  
  PGDKGCPrivate = ^TGDKGCPrivate;
  TGDKGCPrivate = Record
    GC : TGDKGC;
    XGC : Xlib.TGC;
    XDisplay : PDisplay;
    ref_count : guint;
  end;
  
Function GDKGetArcPoints(Left, Top, Right, Bottom,
  StartX, StartY, EndX, EndY : Integer) : PGDKPoint;
var
  CX,CY,
  Height,
  Width,
  R : Extended;
  XAS,YAS,
  XAE,YAE : Extended;
begin
  CircCenter(Left,Top,Right,Bottom,CX,CY);
  Height := ABS(Bottom - Top);
  Width := ABS(Right - Left);
  R := Height / 2;
  If Width / 2 >  R then
    R := Width /2;

  CPCircle(StartX, StartY, CX, CY, R,XAS, YAS);
  CPCircle(EndX, EndY, CX, CY, R,XAE, YAE);
  Result := nil;
  If (Round(XAS) = Round(XAE)) and (Round(YAS) = Round(YAE)) then
    exit;
  New(Result);
  Inc(Result,SizeOf(TGDKPoint)*3);
  Result[0].X := Round(XAS);
  Result[0].Y := Round(YAS);
  Result[1].X := Round(CX);
  Result[1].Y := Round(CY);
  Result[2].X := Round(XAE);
  Result[2].Y := Round(YAE);
end;

Function GDKGetPoints(var Points; Count : Integer; Close : Boolean; StartPt : PPoint) : PGDKPoint;
var
  I,SP : Integer;
  PPoints : PPoint;
begin
  PPoints := PPoint(@Points);
  SP := 0;
  If StartPt <> nil then begin
    SP := 1;
    Count := Count + 1;
  end;
  New(Result);
  Inc(Result,SizeOf(TGDKPoint)*(Count + 1));
  If SP = 1 then begin
    Result[0].X := StartPt^.X;
    Result[0].Y := StartPt^.Y;
  end;
  For I := SP to Count do begin
    If I = Count then begin
      If Close then
        If SP = 1 then begin
          Result[0].X := StartPt^.X;
          Result[0].Y := StartPt^.Y;
        end else begin
          Result[I].X := PPoints[0].X;
          Result[I].Y := PPoints[0].Y;
        end;
    end
    else begin
      Result[I].X := PPoints[I].X;
      Result[I].Y := PPoints[I].Y;
    end;
  end;
end;

Procedure GDKUpdateFonts;
begin
  If theDisplay = nil then
    InitDisplay(GDK_Get_Display);
  CreateFontList(theDisplay);
end;

Procedure GDKSetDashStyle(GC : PGDKGC; Style : Longint);
const
  {$IfDef ARRAY_OF_CHAR}
  Dash1 : Char = #16;
  Dash2 : Char = #12;
  Dash3 : Char = #8;
  Dot   : Char = #4;  
  {$Else}
  Dash1 : Integer = 16;
  Dash2 : Integer = 12;
  Dash3 : Integer = 8;
  Dot   : Integer = 4;
  {$EndIf}
begin
  Case style of
    PS_Dash :
      begin
        gdk_gc_set_dashes(gc,0,[Dash1, Dash3],2);
      end;
    PS_Dot :
      begin
        gdk_gc_set_dashes(gc,0,[Dot,Dot],2);
      end;
    PS_DashDot :
      begin
        gdk_gc_set_dashes(gc,0,[Dash2,Dash3,Dot,Dash3],4);
      end;
    PS_DashDotDot :
      begin
        gdk_gc_set_dashes(gc,0,[Dash3,Dot,Dot,Dot,Dot,Dot],6);
      end;
  end;
end;

Function GDKGetColor(Color : TCOLORREF) : PGDKColor;
begin
  New(Result);
  With Result^ do begin
    Red   := RGB(0,GetRValue(Color),0);
    Green := RGB(0,GetGValue(Color),0);
    Blue  := RGB(0,GetBValue(Color),0);
    Pixel := red*65536 + green*256 + blue;
  end;
  GDK_Color_Alloc(gdk_colormap_get_system,Result);
end;

Function GetPixelColor(Pixel : guint32) : TColorRef;
var
  Color : PXColor;
  CMP : PGDKColorMapPrivate;
begin
  New(Color);
  Color^.Pixel := Pixel;
  CMP := PGDKColorMapPrivate(gdk_colormap_get_system);
  XQueryColor(CMP^.XDisplay, CMP^.XColorMap,Color);
  Result := RGB(Color^.Red shr 8,Color^.Green shr 8,Color^.Blue shr 8);
  Release(Color);
end;

Function GDKGetPixel(Drawable : PGDKWindow; X, Y : Integer) : TColorRef;
var
  Image : PGDKImage;
  Pixel : guint32;
begin
  Result := 0;
  Pixel := 0;
  If DRawable = nil then
    exit;
  If X < 0 then
    exit;
  If Y < 0 then
    exit; 
  If X > PGDKWindowPrivate(Drawable)^.Width then
    exit;
  If Y > PGDKWindowPrivate(Drawable)^.Height then
    exit;
  Image := nil;
  Try
    GDK.gdk_error_trap_push;
    Image := gdk_image_get(Drawable,X,Y,1,1);
    GDK.gdk_error_trap_pop;
  except
    Image := nil;
  end;
  If Image <> nil then begin
    Pixel := gdk_image_get_pixel(Image, 0, 0);
    GDK_Image_destroy(Image);
  end;
  Result := GetPixelColor(Pixel);
end;

Procedure GDKSetForeGroundColor(GC : PGDKGC; Color : TColorRef);
var
  C : PGDKColor;
begin
  C := GDKGetColor(Color);
  gdk_gc_set_foreground(GC, C);
  Release(C);
end;

Procedure GDKSetBackGroundColor(GC : PGDKGC; Color : TColorRef);
var
  C : PGDKColor;
begin
  C := GDKGetColor(Color);
  gdk_gc_set_background(GC, C);
  Release(C);
end;

Function GDKGetPenStyle(Style : Longint) : TGdkLineStyle;
begin
  Case style of
    PS_Solid : Result := GDK_LINE_SOLID;
    PS_Dash : Result := GDK_LINE_ON_OFF_DASH;
    PS_Dot : Result := GDK_LINE_ON_OFF_DASH;
    PS_DashDot : Result := GDK_LINE_ON_OFF_DASH;
    PS_DashDotDot : Result := GDK_LINE_ON_OFF_DASH;
  end;
end;

procedure GDKSetBrushBitmap(GC : PGDKGC; BrushStyle, BrushHatch : Integer);
var
  pix : PGDKPixmap;
  FG : PGDKColor;
  BG : PGDKColor;
Const
  Horiz  : PChar = #0 + #0 + #0 + #255 + #0 + #0 + #0 + #0;
  Vert   : PChar = #8 + #8 + #8 + #8 + #8 + #8 + #8 + #8;
  FDiag  : PChar = #1 + #2 + #4 + #8 + #16 + #32 + #64 + #128;
  BDiag  : PChar = #128 + #64 + #32 + #16 + #8 + #4 + #2 + #1;
  Cross  : PChar = #8 + #8 + #8 + #255 + #8 + #8 + #8 + #8;
  DiagC  : PChar = #129 + #66 + #36 + #24 + #24 + #36 + #66 + #129;
begin
  Pix := nil;
  case BrushStyle of
    BS_HATCHED :
      begin
        gdk_gc_set_fill(GC,GDK_STIPPLED);
        Case BrushHatch of
          HS_HORIZONTAL:
            pix := gdk_Bitmap_create_from_data(nil,
              Horiz, 8, 8);
          HS_VERTICAL:
            pix := gdk_Bitmap_create_from_data(nil,
              Vert, 8, 8);
          HS_FDIAGONAL:
            pix := gdk_Bitmap_create_from_data(nil,
              FDiag, 8, 8);
          HS_BDIAGONAL:
            pix := gdk_Bitmap_create_from_data(nil,
              BDiag, 8, 8);
          HS_CROSS:
            pix := gdk_Bitmap_create_from_data(nil,
              Cross, 8, 8);
          HS_DIAGCROSS:
            pix := gdk_Bitmap_create_from_data(nil,
              DiagC, 8, 8);
        end;
        If Pix <> nil then
          gdk_gc_set_Stipple(GC,Pix);
      end;
    BS_PATTERN :
      begin
        gdk_gc_set_fill(GC,GDK_Tiled);
        If BrushHatch <> 0 then
          Try
            pix := PGDKBitmap(BrushHatch);
          except
          end;
       If Pix <> nil then   
         gdk_gc_set_Tile(GC,Pix);
      end;   
    BS_SOLID :
      gdk_gc_set_fill(GC,GDK_Solid);
  end;
  gdk_gc_set_ts_origin(GC,0,0);
end;

Procedure GDKSetRop2(GC : PGDKGC; ROP2 : Integer);
begin
  Case ROP2 of
    R2_NOP :
      GDK_GC_Set_Function(GC, GDK_AND_INVERT);
    R2_NOT :
      GDK_GC_Set_Function(GC, GDK_INVERT);
    R2_COPYPEN :
      GDK_GC_Set_Function(GC, GDK_Copy);
    R2_NOTCOPYPEN :
      GDK_GC_Set_Function(GC, GDK_OR_REVERSE);
    R2_MERGEPENNOT :
      GDK_GC_Set_Function(GC, GDK_Equiv);
    R2_MASKPENNOT :
      GDK_GC_Set_Function(GC, GDK_AND);
    R2_MERGENOTPEN :
      GDK_GC_Set_Function(GC, GDK_Copy_Invert);
    R2_MASKNOTPEN:
      GDK_GC_Set_Function(GC, GDK_AND_Reverse);
    R2_MERGEPEN :
      GDK_GC_Set_Function(GC, GDK_NOOP);
    R2_NOTMERGEPEN :
      GDK_GC_Set_Function(GC, GDK_NAND);
    R2_MASKPEN :
      GDK_GC_Set_Function(GC, GDK_Clear);
    R2_NOTMASKPEN :
      GDK_GC_Set_Function(GC, GDK_OR_Invert);
    R2_XORPEN :
      GDK_GC_Set_Function(GC, GDK_XOR);
    R2_NOTXORPEN :
      GDK_GC_Set_Function(GC, GDK_OR);
   end;
end;

Function GDKGetFont(Font : TLogFont) : PGDKFont;
begin
  Result := GDK_Font_Load(GetFontName(Font));
end;

Procedure SetArcMode(Drawable : PGDKDrawable; gc : PGDKGC; ArcMode : Longint);
var
  GCP : PGDKGCPrivate;
  DRP : PGDKWindowPrivate;
begin
  DRP := PGDKWindowPrivate(drawable);
  GCP := PGDKGCPrivate(GC);
  XSetArcMode(DRP^.Display,GCP^.xgc, ArcMode);
end;

Procedure GDKArc(drawable : PGdkDrawable; gc : PGdkGC; filled : Boolean;
  X1,Y1,X2,Y2,X3,Y3,X4,Y4 : Integer);
Var
  Width,Height,
  start_angle, end_angle : Integer;
begin
  GetArcVariables(X1,Y1,X2,Y2,X3,Y3,X4,Y4,Width,Height,Start_Angle,End_Angle);
  SetArcMode(Drawable,GC, ArcPieSlice);
  GDK_Draw_Arc(Drawable,gc,Longint(Filled),X1,Y1,
    Width, Height, start_angle, end_angle);
end;

Procedure GDKArc(drawable : PGdkDrawable; gc : PGdkGC; filled : Boolean;
  X1,Y1,X2,Y2,Angle1,Angle2 : Integer);
Var
  Width,Height,
  start_angle, end_angle : Integer;
begin
  GetArcVariables(X1,Y1,X2,Y2,1,1,1,1,Width,Height,Start_Angle,End_Angle);
  SetArcMode(Drawable,GC, ArcPieSlice);
  GDK_Draw_Arc(Drawable,gc,Longint(Filled),X1,Y1,
    Width, Height, Angle1, Angle2);
end;

Procedure GDKChord(drawable : PGdkDrawable; gc : PGdkGC; filled : Boolean;
  X1,Y1,X2,Y2,X3,Y3,X4,Y4 : Integer);
Var
  Width,Height,
  start_angle, end_angle : Integer;
begin
  GetArcVariables(X1,Y1,X2,Y2,X3,Y3,X4,Y4,Width,Height,Start_Angle,End_Angle);
  SetArcMode(Drawable,GC, ArcChord);
  GDK_Draw_Arc(Drawable,gc,Longint(Filled),X1,Y1,
    Width, Height, start_angle, end_angle);
end;

function GDKPolyline(drawable : PGdkDrawable; gc : PGdkGC; 
  var Points; Count: Integer; PenPos : PPoint): BOOL;
Var
  Pts : PGDKPoint;
begin
  Result := False;
  Pts := GDKGetPoints(Points,Count,False, PenPos);
  If Pts <> nil then begin
    GDK_Draw_Lines(Drawable,gc,Pts,Count);
    If PenPos <> nil then begin
      PenPos^.X := Pts[Count - 1].X;
      PenPos^.Y := Pts[Count - 1].Y;
    end;  
    Release(pts);
    Result := True;
  end;
end;

function GDKPolygon(drawable : PGdkDrawable; gc : PGdkGC; Filled : Boolean;
  var Points; Count: Integer): BOOL;
Var
  Pts : PGDKPoint;
begin
  Result := False;
  Pts := GDKGetPoints(Points,Count,True,nil);
  If Pts <> nil then
    GDK_Draw_Polygon(Drawable,gc,Longint(Filled),Pts, Count);
  Release(pts);
  Result := True;
end;

function GDKBitBlt(Dest : PGdkDrawable; GC : PGdkGC; X, Y, Width, Height: Integer;
  Src : PGdkDrawable; XSrc, YSrc: Integer): BOOL;
begin
  Result := False;
  If (Dest <> nil) and (Src <> nil) then begin
    gdk_draw_pixmap(Dest, gc, Src, X, Y, XSrc, YSrc, width, height);
    Result := True;
  end;
end;

function GDKStrechBlt(Dest : PGdkDrawable; GC : PGdkGC; X, Y, Width, Height: Integer;
  Src : PGdkDrawable; XSrc, YSrc, SrcWidth, SrcHeight: Integer): BOOL;
begin
  Result := False;
  If (Dest <> nil) and (Src <> nil) then begin
    gdk_draw_pixmap(Dest, gc, Src, X, Y, XSrc, YSrc, width, height);
    Result := True;
  end;
end;

Function GDKGetCursorPos(var pos : TPoint) : Boolean;
begin
  If theDisplay = nil then
    InitDisplay(GDK_GET_DISPLAY);
  Result := XGetCursorPos(theDisplay, pos);
end;

Procedure GDKSetCursorPos(X, Y : Integer);
begin
  If theDisplay = nil then
    InitDisplay(GDK_GET_DISPLAY);
  XSetCursorPos(theDisplay,X,Y);
end;

Finalization
  If TheDisplay <> nil then
    CloseDisplay;
end.
