Unit GTKRoutines;



interface



Uses Glib, GDK, GTK, CPWindows, XRoutines, GDKRoutines;



Procedure GTKGetWidgetSize(Widget : PGTKWidget; var W, H : Longint); 

Procedure GTKGetWidgetText(Widget : PGTKWidget; var aString : PChar);

Procedure SetGTKWidgetFont(Widget : PGTKWidget; LogFont : TLogFont; Color : TColorRef);

Procedure GTKSetWidgetSize(Widget : PGTKWidget; W,H : Longint);

Procedure GTKWidgetSetFocus(Widget : PGTKWidget);



implementation



Procedure GTKGetWidgetSize(Widget : PGTKWidget; var W, H : Longint); 

begin

  If GTK_IS_Window(Widget) and GTK_Widget_Realized(Widget) then begin

    GDK_Window_Get_Size(Widget^.Window, @W, @H);

  end;

end;



Procedure GTKGetWidgetText(Widget : PGTKWidget; var aString : PChar);

begin

  If Widget = nil then

    exit;

  If GTK_IS_Window(Widget) then

    aString := g_strdup(PGtkWINDOW(Widget)^.title)

  else

    If GTK_IS_Label(Widget) then

      aString := g_strdup(PGTKLabel(Widget)^.theLabel)

  else

    If GTK_IS_Entry(Widget) then

      aString := g_strdup(GTK_Entry_Get_Text(PGTKEntry(Widget)))

  else

    If GTK_IS_Combo(Widget) then begin

      If PGTKCombo(Widget)^.Entry <> nil then

        aString := g_strdup(GTK_Entry_Get_Text(PGTKEntry(

          PGTKCombo(Widget)^.Entry)));

    end

  else

    If GTK_IS_TEXT(Widget) then

      aString := g_strdup(GTK_Editable_Get_Chars(

        PGTKEditable(Widget), 0,-1))

  else

    If GTK_IS_FRAME(Widget) then

      aString := g_strdup(PGTKFrame(Widget)^.thelabel);

end;



Procedure SetGTKWidgetFont(Widget : PGTKWidget; LogFont : TLogFont; Color : TColorRef);

var

  I : integer;

  ST : PGTKRCStyle;

  CM : PGDKColorMap;

begin

  If Widget = nil then

    exit;

  St := GTK_RC_Get_Style(Widget)^.RC_style;

  ST^.Font_Name := GetFontName(LogFont);

  For I := 0 to 4 do begin

    st^.FG[I].Red := RGB(0,GetRValue(Color),0);

    st^.FG[I].Green := RGB(0,GetGValue(Color),0);

    st^.FG[I].Blue := RGB(0,GetBValue(Color),0);

    CM := gtk_widget_get_Colormap(Widget);

    GDK_ColorMap_alloc_Color(CM,@st^.FG[I],True,False);

    ST^.Color_Flags[I] := GTK_RC_FG;

  end;

  GTK_Widget_Modify_Style(Widget, ST);

end;



Procedure GTKSetWidgetSize(Widget : PGTKWidget; W,H : Longint);

begin

  If Widget = nil then

    exit;

  If GTK_IS_Table(Widget) then

    GTK_Table_resize(PGTKTable(Widget), W, H)

  else

    If GTK_IS_Window(Widget) then begin

      If GTK_Widget_Realized(Widget) then

        GDK_Window_Resize(Widget^.Window, W, H)

      else

        GTK_Window_Set_Default_Size(PGTKWindow(Widget), W, H);

    end

  else

    If GTK_IS_drawing_area(Widget) then

      gtk_drawing_area_size (GTK_DRAWING_AREA(Widget), W, H)

    else

      GTK_Widget_Set_uSize(Widget, W, H);

end;



Procedure GTKWidgetSetFocus(Widget : PGTKWidget);

begin

  If Widget <> nil then begin

    If GTK_IS_Window(Widget) then

      gtk_window_activate_focus(PGTKWindow(Widget));

    GTK_Widget_Grab_Focus(Widget);

    GTK_Widget_Draw_Focus(Widget);

    If GTK_IS_Combo(Widget) then

      GTK_Widget_Grab_Focus(PGTKWidget(PGTKCombo(Widget)^.Entry));

  end;

end;



end.