unit gtkwindevices;

{$Mode Delphi}

interface

Uses
  CPWindows, CPWinControls, CPWinObjects, CPWinDevices, GDKRoutines, GTKRoutines,
  GDK, GTK;

Type
  TGTKhDC = Class(TCPhDevice)
  Public
    FHandle  : PGDKDrawable;
    FGVisual : PGDKVisual;
    Procedure SetHandle(Value : PGdkDrawable); Virtual;
    Function GetHandle : PGdkDrawable; Virtual;
    Function GDKVisual : PGDKVisual; Virtual;
    Function GetOrigin : TPoint; Override;
    Function GetSize : TSize; Override;
  Protected
    Class Function GuiBitBlt(DestDC: HDC; X, Y, Width, Height: Integer; SrcDC: HDC;
      XSrc, YSrc: Integer; Rop: DWORD): BOOL; Override;
    Class Function GuiGetPixel(DC: HDC; X, Y: Integer): TCOLORREF; Override;
    Class Function GuiDrawPixel(DC: HDC; X, Y: Integer) : Integer;
      Override;
    Class Procedure GuiDrawArc(hDC: HDC; Filled : Boolean; left, top, right, bottom, startX, startY,
      endX, endY: Integer); Override; Overload;
    Class Procedure GuiDrawArc(hDC: HDC; Filled : Boolean; left, top, right, bottom,
      Angle1, Angle2: Integer); Override; Overload;
    Class Procedure GuiDrawChord(hDC: HDC; Filled : Boolean; left, top, right, bottom, startX, startY,
      endX, endY: Integer); Override;
    Class Procedure GuiDrawEllipse(hDC: HDC; Filled : Boolean; left, top, right, bottom : Integer);
      Override;
    Class Procedure GuiDrawLine(DC : hDC; X1,Y1,X2,Y2 : Integer); Override;
    Class Procedure GuiDrawRectangle(DC : hDC; Filled : Boolean; X, Y, Width, Height : Integer);
      Override;
    Class Procedure GuiDrawPolyline(DC: HDC; var Points; Count: Integer; FRMTO : PPoint);
      Override;
    Class Procedure GuiDrawPolygon(DC: HDC; Filled : Boolean; var Points;
      Count: Integer); Override;
    Class Procedure GuiInvertRect(hDC: HDC; const lprc: TRect); Override;
  Public
    GC      : PGDKGC;
    GCVals  : PGdkGCValues;
    Procedure UpdatePlaceP; Override;
    Procedure SelectBKColor; Override;
    Procedure SelectBKMode; Override;
    Procedure SelectRop2; Override;
    Constructor Create; Overload; Override;
    Constructor Create(ARect : TRect); Overload; Override;
    destructor  Destroy; Override;
    Property Handle : PGDKDrawable read GetHandle write SetHandle;
  End;

  TGTKRoot = Class(TGTKhDC)
  Private
    Function GetSize : TSize; Override;
    Procedure SetHandle(Value : PGdkDrawable); Override;
    Function GetHandle : PGdkDrawable; Override;
    Function GDKVisual : PGDKVisual; Override;
  Public
    Constructor Create; Overload; Override;
    Constructor Create(ARect : TRect); Overload; Override;
  end;

  Function CreateCompatibleDC(DC: HDC): HDC;
  Function GetDC(wnd : hwnd) : hdc;
  Function ReleaseDC(Wnd: HWND; hDC: HDC): Integer;

  Function BeginPaint(hWnd: HWND; lpPaint: lpPaintStruct): HDC;
  Function EndPaint(hWnd: HWND; const lpPaint: TPaintStruct): BOOL;


implementation

Uses GTKWindows, XRoutines;

Procedure DrawArcLines(DC : hDC; Left, Top, Right, Bottom,
  StartX, StartY, EndX, EndY : Integer);
var
  CX,CY,
  Height : Extended;
  XAS,YAS,
  XAE,YAE : Extended;
  PTS : Array [0..2] of TPoint;
begin
  CircCenter(Left,Top,Right,Bottom,CX,CY);
  Height := Bottom - Top;
  CPCircle(StartX, StartY, CX, CY, Height / 2,XAS, YAS);
  CPCircle(EndX, EndY, CX, CY, Height / 2,XAE, YAE);
  If (Round(XAS) = Round(XAE)) and (Round(YAS) = Round(YAE)) then
    exit;
  PTS[0].X := Round(XAS);
  PTS[0].Y := Round(YAS);
  PTS[1].X := Round(CX);
  PTS[1].Y := Round(CY);
  PTS[2].X := Round(XAE);
  PTS[2].Y := Round(YAE);
  PolyLine(DC,Pts[0],3);
end;

Procedure DrawChordLine(DC : hDC; Left, Top, Right, Bottom,
  StartX, StartY, EndX, EndY : Integer);
var
  CX,CY,
  Height : Extended;
  XAS,YAS,
  XAE,YAE : Extended;
  PTS : Array [0..2] of TPoint;
begin
  CircCenter(Left,Top,Right,Bottom,CX,CY);
  Height := Bottom - Top;
  CPCircle(StartX, StartY, CX, CY, Height / 2,XAS, YAS);
  CPCircle(EndX, EndY, CX, CY, Height / 2,XAE, YAE);
  If (Round(XAS) = Round(XAE)) and (Round(YAS) = Round(YAE)) then
    exit;
  PTS[0].X := Round(XAS);
  PTS[0].Y := Round(YAS);
  PTS[1].X := Round(XAE);
  PTS[1].Y := Round(YAE);
  PolyLine(DC,Pts[0],2);
end;

Procedure TGTKhDC.SetHandle(Value : PGdkDrawable);
begin
  FHandle := Value;
  If GC <> nil then
    gdk_gc_destroy(GC);
  GC := nil;
  If FHandle <> nil then
    GC := gdk_gc_new(Fhandle);
  SelectPen;
  SelectBrush;
  SelectFont;
  SetRop2(LongInt(Self),ROPMode);
end;

Function TGTKhDC.GDKVisual : PGDKVisual;
begin
  Try
    If FGVisual = nil then
      If Handle <> nil then
        FGVisual := gdk_window_get_visual(FHandle);
  Finally
    Result := FGVisual;
  end;
end;

Function TGTKhDC.GetHandle : PGdkDrawable;
begin
  Result := FHandle;
end;

Function TGTKhDC.GetOrigin : TPoint;
begin
  Result := Inherited GetOrigin;
  If Handle <> nil then
    Try
      gdk_window_get_position(FHandle, @Result.X, @Result.Y);
    except
    end;
end;

Function TGTKhDC.GetSize : TSize;
begin
  Result := Inherited GetSize;
  If Handle <> nil then
    Try
      gdk_window_get_size(FHandle, @Result.cX, @Result.cY);
    except
    end;
end;

Class Function TGTKhDC.GUIBitBlt(DestDC: HDC; X, Y, Width, Height: Integer; SrcDC: HDC;
  XSrc, YSrc: Integer; Rop: DWORD): BOOL;
var
  Tmp : Longint;
begin
  Result := False;
  If TGTKhDC(DestDC).Handle = nil then
    exit;
  If TGTKhDC(SrcDC).Handle = nil then
    exit;
  Tmp := TGTKhDC(DestDC).ROPMode;
  SetRop2(DestDC,Rop);
  GDKBitBlt(TGTKhDC(DestDC).Handle, TGTKhDC(DestDC).gc, X, Y,
    width, height, TGTKhDC(SrcDC).Handle, XSrc, YSrc);
  SetRop2(DestDC,Tmp);
  Result := True;
end;

Class Function TGTKhDC.GuiGetPixel(DC: HDC; X, Y: Integer): TCOLORREF;
begin
  With TGTKhDC(DC) do
    If Handle <> nil then
      Result := GDKGetPixel(Handle,X,Y);
end;

Class Function TGTKhDC.GuiDrawPixel(DC: HDC; X, Y: Integer) : Integer;
{var
  Error : Integer;}
begin
  Result := 0;
  If TGTKhDC(DC).Handle = nil then
    exit;
  With TGTKhDC(DC) do
    gdk_draw_point(Handle,GC,X,Y);
  //Result := Error;
end;

Class Procedure TGTKhDC.GuiDrawArc(hDC: HDC; Filled : Boolean; left, top, right, bottom, startX, startY,
  endX, endY: Integer);
begin
  If TGTKhDC(hDC).Handle = nil then
    exit;
  With TGTKhDC(hDC) do
    GDKArc(Handle, GC, Filled, Left, Top, Right, Bottom,
      startx,starty,endx,endy);
  If not Filled then
    DrawArcLines(hDC,Left,Top,Right,Bottom,
      StartX,StartY,EndX,EndY);
end;

Class Procedure TGTKhDC.GuiDrawArc(hDC: HDC; Filled : Boolean; left, top, right, bottom,
  Angle1, Angle2: Integer);
begin
  If TGTKhDC(hDC).Handle = nil then
    exit;
  With TGTKhDC(hDC) do
    GDKArc(Handle, GC, Filled, Left, Top, Right, Bottom,
      Angle1*64,Angle2*64);
end;

Class Procedure TGTKhDC.GuiDrawChord(hDC: HDC; Filled : Boolean; left, top, right, bottom, startX, startY,
  endX, endY: Integer);
begin
  If TGTKhDC(hDC).Handle = nil then
    exit;
  With TGTKhDC(hDC) do
    GDKChord(Handle, GC, Filled, Left, Top, Right, Bottom,
      startx,starty,endx,endy);
  If not Filled then
    DrawChordLine(hDC,Left,Top,Right,Bottom,
      StartX,StartY,EndX,EndY);   
end;

Class Procedure TGTKhDC.GuiDrawEllipse(hDC: HDC; Filled : Boolean; left, top, right, bottom : Integer);
begin
  If TGTKhDC(hDC).Handle = nil then
    exit;
  With TGTKhDC(hDC) do
    GDKArc(Handle, GC, Filled, Left, Top, Left + Right, Top + Bottom, 0,0,0,0);
end;


Class Procedure TGTKhDC.GuiDrawLine(DC : hDC; X1,Y1,X2,Y2 : Integer);
begin
  If TGTKhDC(DC).Handle = nil then
    exit;
  With TGTKhDC(DC) do
    GDK_Draw_Line(Handle, GC, X1, Y1, X2, Y2);
end;

Class Procedure TGTKhDC.GuiDrawRectangle(DC : hDC; Filled : Boolean; X, Y, Width, Height : Integer);
begin
  If TGTKhDC(DC).Handle = nil then
    exit;
  With TGTKhDC(DC) do
    GDK_Draw_Rectangle(Handle, GC, Integer(Filled), X, Y, Width, Height); 
end;

Class Procedure TGTKhDC.GuiDrawPolyline(DC: HDC; var Points; Count: Integer; FRMTO : PPoint);
begin
  If TGTKhDC(DC).Handle = nil then
    exit;
  With TGTKhDC(DC) do
    GDKPolyline(Handle,gc,Points,Count,FRMTO);
end;

Class Procedure TGTKhDC.GuiDrawPolygon(DC: HDC; Filled : Boolean; var Points; Count: Integer);
begin
  If TGTKhDC(DC).Handle = nil then
    exit;
  With TGTKhDC(DC) do
    GDKPolygon(Handle,gc,Filled,Points,Count);
end;

Class Procedure TGTKhDC.GuiInvertRect(hDC: HDC; const lprc: TRect);
var
  Left,Top,
  Right,Bottom,
  Width,
  Height,
  Tmp : Integer;
begin
  If TGTKhDC(hDC).Handle = nil then
    exit;
  With TGTKhDC(hDC) do begin
    Left := lprc.Left;
    Top := lprc.Top;
    Right := lprc.Right;
    Bottom := lprc.Bottom;
    Width := Right - Left;
    Height := Bottom - Top;
    UpdatePlaceP;
    Tmp := SetRop2(hDC,R2_Not);
    GDKBitBlt(Handle, gc, Left, Top, Width, Height, Handle, Left, Top);
    SetRop2(hDC,Tmp);
  end;
end;

Constructor TGTKhDC.Create;
begin
  Create(Rect(0,0,0,0));
end;

Constructor TGTKhDC.Create(ARect : TRect);
begin
  inherited Create(ARect);
  New(GCVals);
  GC := nil;
end;

destructor  TGTKhDC.Destroy;
begin
  SetHandle(nil);
  DeleteObject(DCPen);
  DeleteObject(DCBrush);
  inherited Destroy;
end;

Procedure TGTKhDC.UpdatePlaceP;
var
  area : TGdkRectangle;
begin
  If Handle = nil then
    exit;
  area.X := lRect.Left;
  area.Y := lRect.Top;
  area.Width := lRect.Right - lRect.Left;
  area.Height := lRect.Bottom - lRect.Top;
  gdk_gc_set_clip_rectangle (gc,  @area);
end;

Procedure TGTKhDC.SelectBKColor;
begin
  If Handle <> nil then begin
    GDKSetBackgroundColor(GC, BKColor);
    gdk_gc_get_values(GC, GCVals);
  end;
end;

Procedure TGTKhDC.SelectBKMode;
begin
  If Handle <> nil then begin
    gdk_gc_get_values(GC, GCVals);
  end;
end;

Procedure TGTKhDC.SelectRop2;
begin
  If GC <> nil then
    GDKSetRop2(GC,RopMode);
end;

Function TGTKRoot.GetSize : TSize;
begin
  Result.cx := GDK_Screen_Width;
  Result.cy := GDK_Screen_Height;
end;

Procedure TGTKRoot.SetHandle(Value : PGdkDrawable);
begin
end;

Function TGTKRoot.GetHandle : PGdkDrawable;
begin
  Result := PGdkDrawable(Self);
end;

Function TGTKRoot.GDKVisual : PGDKVisual;
begin
  Result := GDK_Visual_Get_System;
end;

Constructor TGTKRoot.Create;
begin
end;

Constructor TGTKRoot.Create(ARect : TRect);
begin
end;

function CreateCompatibleDC(DC: HDC): HDC;
var
  LDC : hdc;
  tDC : TGTKhDC;
begin
  If DC = 0 then begin
    LDC := GetDC(0);
    tDC := TGTKhDC.Create(TGTKhDC(lDC).lRect);
    ReleaseDC(0,Ldc);
  end
  else
    tDC := TGTKhDC.Create(TGTKhDC(DC).lRect);
  Result := Longint(tDC);
end;

Function GetDC(wnd : hwnd) : hdc;
var
  DC : TGTKhDC;
begin
  Result := 0;
  If wnd = 0 then begin
    DC := TGTKRoot.Create;
    Result := Longint(DC);
  end
  else
    If (TObject(wnd) is TGTKControl) then
      with TGTKControl(wnd) do begin
        DC := TGTKhDC.Create(Rect(Left,Top,
          Width + Left, Top + Height));
        If Widget <> nil then
          DC.Handle := Widget^.Window;
        Result := Longint(DC);
      end;
end;

function ReleaseDC(Wnd: HWND; hDC: HDC): Integer;
var
  DC : TGTKhDC;
  L : Longint;
begin
  Result := 0;
  L := hDevices.IndexOf(TGTKhDC(HDC));
  If L > -1 then begin
    DC := TGTKhDC(hDevices.Items[L]);
    hDevices.Delete(L);
    FreeAndNil(DC);
    Result := 1;
  end;
end;

function BeginPaint(hWnd: HWND; lpPaint: lpPaintStruct): HDC;
var
  DC : hDC;
begin
  Result := 0;
  If hwnd = 0 then
   exit;
  If lpPaint = nil then
    exit;
  With TCPWinControl(hwnd) do begin
    If Painting then
      exit;
    Painting := True;
    DC := GetDC(hWnd);
    lpPaint^.hdc := DC;
    Result := lpPaint^.hdc;
    lpPaint^.rcPaint := PaintRect;
  end;
end;

function EndPaint(hWnd: HWND; const lpPaint: TPaintStruct): BOOL;
begin
  Result := False;
  If hwnd = 0 then
   exit;
  If lpPaint.hdc = 0 then
    exit;
  With TCPWinControl(hwnd) do begin
    ReleaseDC(hWnd,lpPaint.hdc);
    Painting := False;
  end;
end;

end.
