unit GTKWindows;

{$Mode Delphi}
interface

Uses
  SysUtils, Classes, Math, GLIB, GTK, GDK,
  CPWindows, CPWinControls, CPWinDevices, CPWinObjects, 
  XRoutines, GDKRoutines, GTKCallbacks, GTKRoutines,
  GTKWinObjects, GTKWinDevices;

Type
  TGTKControl = Class(TCPWinControl)
  Protected
    Procedure CreateWidgets(theObject : Pointer);
    Procedure SetWindowText(Value : PChar); Override;

    Function GetLeft : Integer; Override;
    Function GetTop : Integer; Override;
    Function GetWidth : Integer; Override;
    Function GetHeight : Integer; Override;
  Public
    Function CanFocus : Boolean; Override;
    Procedure SetFocus; Override;
    Procedure SetAlign(Val : Integer); Override;
    Procedure SelectCursor;
    Procedure UpdateTable;
    Procedure SetParentWindow(Val : TCPWinControl); Override;
    Procedure SetSize(W,H : Integer); Override;
    Procedure SetOrigin(X,Y : Integer); Override;
    Procedure SetSignal(PName : String; Proc : Pointer);
    Function GetText(Len : integer) : String;
    Procedure Show; Override;
    Procedure Hide; Override;
    Procedure UpdateWindow; Override;
    Constructor Create(theObjectType : String); Override;
    Destructor Destroy; Override; 
    
    WindowDestroyed : Boolean;
    Widget,
    Table : PGTKWidget;
  end;

  TGTKhBitmap = Class(TGTKhDC)
  Private
    FHandle : PGDKPixmap;
    Procedure SetHandle(Value : PGdkDrawable); Override;
    Function GetHandle : PGdkDrawable; Override;
  Public
    theType : Longint;
    Width : Longint;
    Height : Longint;
    WidthBytes : Longint;
    Planes : Word;
    Depth : Word;
    Bits : Pointer;
    Procedure UpdateBitmap;
  end;

  function RegisterClass(const lpWndClass: TWndClass): ATOM;
  function CreateWindowEx(dwExStyle: DWORD; lpClassName: PChar;
    lpWindowName: PChar; dwStyle: DWORD; X, Y, nWidth, nHeight: Integer;
    hWndParent: HWND; hMenu: HMENU; hInstance: HINST; lpParam: Pointer): HWND;
  function UpdateWindow(hWnd: HWND): BOOL;
  function SetWindowLong(hWnd: HWND; nIndex: Integer; dwNewLong: Longint): Longint;
  function BeginPaint(hWnd: HWND; lpPaint: lpPaintStruct): HDC;
  function EndPaint(hWnd: HWND; const lpPaint: TPaintStruct): BOOL;

  function GetWindowText(hWnd: HWND): PChar;
  function SetWindowText(hWnd: HWND; const lpString: PChar): BOOL;
  function SetFocus(hWnd: HWND): HWND;
  function GetActiveWindow: HWND;
  function GetFocus: HWND;
  function DestroyWindow(var Wnd: HWND): BOOL;
  function ShowWindow(hWnd: HWND; nCmdShow: Integer): BOOL;
  function SetWindowPos(hWnd: HWND; X, Y, cx, cy: Integer): BOOL;
  function SetParent(hWndChild, hWndNewParent: HWND): HWND;
  function DefWindowProc(hWnd: HWND; Msg: UINT; wParam: WPARAM;
    lParam: LPARAM): LRESULT;
  function CallWindowProc(lpPrevWndFunc: TFNWndProc; hWnd: HWND;
    Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT;
  function SendMessage(hWnd: HWND; Msg: UINT; wParam: WPARAM;
    lParam: LPARAM): LRESULT;
  Function CreateWindow(theObjectType : String) : hWnd;

  function CreateCompatibleDC(DC: HDC): HDC;

  Function CreatePenIndirect(const LogPen: TLogPen): HPEN;
  Function CreatePen(Style : Integer; Width : Integer;
    Color : Colorref): HPEN;

  Function CreateBrushIndirect(const LogBrush: TLogBrush): HBRUSH;
  Function CreateHatchBrush(nIndex : Longint; Color : TColorRef): HBRUSH;
  Function CreateSolidBrush(Color : TColorRef): HBRUSH;

  function CreateBitmap(Width, Height: Integer; Planes, BitCount: Longint;
    Bits: Pointer): HBITMAP;
  function CreateBitmapIndirect(const p1: TBitmap): HBITMAP;
  function CreateCompatibleBitmap(DC: HDC; Width, Height: Integer): HBITMAP;
  function CreateDiscardableBitmap(DC: HDC; p2, p3: Integer): HBITMAP;
  Function CreateFontIndirect(const LogFont: PLogFont): HFont;

  function SetTextColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
  function SetBkColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
  function SetDCBrushColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
  function SetDCPenColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
  function SetBkMode(DC: HDC; theBkMode: Integer): Integer;

  function GetSysColor(nIndex: Integer): DWORD;
  function GetStockObject(Index: Integer): HGDIOBJ;
  function GetObject(p1: HGDIOBJ; p2: Integer; p3: Pointer): Integer;
  function SelectObject(DC: HDC; p2: HGDIOBJ): HGDIOBJ;
  function DeleteObject(var p1: HGDIOBJ): BOOL;
  function UnrealizeObject(hGDIObj: HGDIOBJ): BOOL;

  function GetBkColor(DC: HDC): COLORREF;
  function GetDCBrushColor(DC: HDC): COLORREF;
  function GetDCPenColor(DC: HDC): COLORREF;
  function GetBkMode(DC: HDC): Integer;

  function GetTextExtentPoint(DC: HDC; Str: PChar; Count: Integer;
    var Size: TSize): BOOL;
  function MoveToEx(DC: HDC; p2, p3: Integer; p4: PPoint): BOOL;
  function LineTo(DC: HDC; X, Y: Integer): BOOL;
  function Rectangle(DC: HDC; X1, Y1, X2, Y2: Integer): BOOL;
  function Polyline(DC: HDC; var Points; Count: Integer): BOOL;
  function PolylineTo(DC: HDC; var Points; Count: Integer): BOOL;
  function FrameRect(hDC: HDC; const lprc: TRect; hbr: HBRUSH): Integer;
  function FillRect(hDC: HDC; const lprc: TRect; hbr: HBRUSH): Integer;
  function InvertRect(hDC: HDC; const lprc: TRect): Integer;
  function DrawFocusRect(hDC: HDC; const lprc: TRect): BOOL;
  function Arc(hDC: HDC; left, top, right, bottom, startX, startY,
    endX, endY: Integer): BOOL;
  function Pie(hDC: HDC; left, top, right, bottom, startX, startY,
    endX, endY: Integer): BOOL;
  function Ellipse(DC: HDC; X1, Y1, X2, Y2: Integer): BOOL;
  function Chord(DC: HDC; X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer): BOOL;
  function Polygon(DC: HDC; var Points; Count: Integer): BOOL;
  function RoundRect(DC: HDC; X1, Y1, X2, Y2, X3, Y3: Integer): BOOL;
  function BitBlt(DestDC: HDC; X, Y, Width, Height: Integer; SrcDC: HDC;
    XSrc, YSrc: Integer; Rop: DWORD): BOOL;
  function StretchBlt(DestDC: HDC; X, Y, Width, Height: Integer; SrcDC: HDC;
    XSrc, YSrc, SrcWidth, SrcHeight: Integer; Rop: DWORD): BOOL;
  function ExtFloodFill(DC: HDC; X, Y: Integer;
    Color: COLORREF; FillType: UINT): BOOL;
  function DrawText(hDC: HDC; lpString: PChar; nCount: Integer;
    var lpRect: TRect; uFormat: UINT): Integer;
  Function TextOut(DC: HDC; X, Y: Integer; Str: PChar; Count: Integer): BOOL;
  function GetPixel(DC: HDC; X, Y: Integer): COLORREF;
  function SetPixel(DC: HDC; X, Y: Integer; Color: COLORREF): COLORREF;
  function SetPixelV(DC: HDC; X, Y: Integer; Color: COLORREF): Integer;

  Function GetDC(wnd : hwnd) : hdc;
  function ReleaseDC(Wnd: HWND; hDC: HDC): Integer;

  Function GetCursorPos(var lpPoint: TPoint): BOOL;
  Procedure SetCursorPos(X, Y : Integer);
  function CreateCaret(hWnd: HWND; hBitmap: HBITMAP; nWidth, nHeight: Integer): BOOL;
  function GetCaretBlinkTime: UINT;
  function SetCaretBlinkTime(uMSeconds: UINT): BOOL;
  function DestroyCaret: BOOL;
  function HideCaret(hWnd: HWND): BOOL;
  function ShowCaret(hWnd: HWND): BOOL;
  function SetCaretPos(X, Y: Integer): BOOL;
  function GetCaretPos(lpPoint: PPoint): BOOL;

  Function WindowFromPoint(Point: TPoint): HWND;
  Function LoadCursor(hInstance: HINST;
    lpCursorName: PAnsiChar): HCURSOR;
  function GetClipBox(DC: HDC; var lRect: TRect): Integer;
  Function IntersectClipRect(DC : hDC; Left, Top, Right,
    Bottom : Integer) : Integer;
  function SetROP2(DC: HDC; p2: Integer): Integer;
  Function GetDeviceCaps(dc : hDC; cap : Integer) : Integer;
  function GetSystemMetrics(nIndex: Integer): Integer;
  procedure PostQuitMessage(nExitCode: Integer);

  function MessageBox(hWnd: HWND; lpText, lpCaption: PChar; uType: UINT): Integer;

const
  HInstance = 0;

implementation

Var
  hClasses : TStrings;
  hObjects : TList;
  
Procedure GetUser_Data(Window : PGdkWindow; var user_data : gpointer);
var
  tmp : gPointer;
  w : PGTKWidget;
begin
  User_Data := nil;
  If Window <> nil then
    If Window^.User_Data <> nil then begin
      tmp := Window^.User_Data;
      W := PGTKWidget(Tmp);
      User_Data := gPointer(W^.name);
    end;
end;

Function GetClassWidgetType(ObjectType : String) : String;
begin
  If hClasses.IndexOf(ObjectType) < 0 then
    Result := UpperCase(ObjectType)
  else
    Result := 'CUSTOM';
end;

Function GetClassWindowProc(ObjectType : String) : TWndProc;
var
  Obj : TObject;
  wncl : TWNDClass;
begin
  If hClasses.IndexOf(ObjectType) < 0 then
    Result := DefWindowProc
  else begin
    Obj := hClasses.Objects[hClasses.IndexOf(ObjectType)];
    wncl := PWndClass(longint(Obj))^;
    Result := wncl.lpfnWndProc;
  end;
end;

Function GetClassBKBrush(ObjectType : String) : hBrush;
var
  Obj : TObject;
  wncl : TWNDClass;
begin
  If hClasses.IndexOf(ObjectType) < 0 then
    Result := GrayBrush
  else begin
    Obj := hClasses.Objects[hClasses.IndexOf(ObjectType)];
    wncl := PWndClass(longint(Obj))^;
    Result := wncl.hbrBackground;
    If Result = 0 then
      Result := WhiteBrush;
  end;
end;

Procedure FFDrawInvalidate(Widget : PGDKWindow; ARect : TRect; Data: gPointer);
var
  Control : TGTKControl;
  wnd : hWnd;
  DC : hDC;
begin
  Control := TGTKControl(Data);
  wnd := hwnd(Control);
  DC := GetDC(Wnd);
  TGTKhDC(DC).Handle := PGDKDrawable(Widget);
 // InterSectClipRect(DC,ARect.Left,ARect.Top,
 //   ARect.Right,ARect.Bottom);
  Control.PaintRect := ARect;
  Control.SendMessage(WM_ERASEBKGND,DC,0);
  FillRect(DC,ARect,Control.BKBrush);
  Control.SendMessage(WM_Paint,DC,0);
  ReleaseDC(Wnd, DC);
  Control.PaintRect := Rect(Control.Left,Control.Top,
    Control.Left + Control.Width,Control.Top +
    Control.Height);
end;

Function DAExpose_Event(Wid : PGTKWidget; Event : PGdkEventExpose;
  Data : gPointer) : gBoolean; cdecl;
begin
  FFDrawInvalidate(Wid^.Window, Rect(Event^.Area.X,Event^.Area.Y,
    Event^.Area.X + Event^.Area.Width,Event^.Area.Y +
    Event^.Area.Height),Data);
  Result := False;
end;

Function DASizeReq_Event(Wid : PGTKWidget; R : PGTkAllocation;
  Data : gPointer) : gBoolean; cdecl;
begin
  FFDrawInvalidate(Wid^.Window, Rect(0, 0, R^.Width,
    R^.Height),Data);
  Result := False;
end;

Procedure Callbacks(Msg : Longint; Vars : Array Of Const;
    Result : Pointer); cdecl;
  
  Procedure SetWindowKilled(Control : TGTKControl);
  var
    I : Integer;
    T : hwnd;
  begin
    Control.WindowDestroyed := True;
    For I := 0 to Control.childhwnd.count - 1 do begin
      T := Hwnd(StrToInt(Control.childhwnd.strings[I]));
      SetWindowKilled(TGTKControl(T));
    end;
  end;
  
var
  P : gPointer;
  Control : TGTKControl;
  cMsg : TMessage;
begin
  P := Vars[0].vPointer;
  cMsg := PMessage(Vars[1].vPointer)^;
  Control := TGTKControl(P);
  If Msg = WM_Command then begin
    If Control.hwndParent <> 0 then
      cMsg.Result := TGTKControl(Control.hwndParent).SendMessage(Msg, 0,
        hwnd(control));
    exit;
  end;
  If Control.WndProc <> nil then
    cMsg.Result := Control.WndProc(Hwnd(Control),Msg, cMsg.WParam,cMsg.LParam)
  else
    cMsg.Result := DefWindowProc(Hwnd(Control),Msg, cMsg.WParam,cMsg.LParam);
  If cMsg.Msg = WM_Close then begin
    SetWindowKilled(Control);
  end;
end;

Function TGTKControl.GetText(Len : integer) : String;
var
  STR : PChar;
  S : String;
begin
  STR := '';
  GTKGetWidgetText(Widget,Str);
  S := String(Str);
  If len < 0 then
    len := Length(S);
  Result := Copy(S,1,len);
end;

Procedure TGTKControl.Show;
begin
  gtk_widget_set_name(Widget,PChar(Self));
  gtk_widget_show(Widget);
  gtk_widget_show(Table);
  Inherited Show;
  If GTK_Is_Window(Widget) then
    SetSignal('size_allocate', @DASizeReq_Event);
end;

Procedure TGTKControl.Hide;
begin
  Inherited Hide;
  If GTK_IS_Widget(Table) then
    gtk_widget_hide(Table);
  If not WindowDestroyed then
    gtk_widget_hide(Widget);
end;

Function TGTKControl.CanFocus : Boolean;
begin
  Result := Inherited CanFocus;
  Result := Widget <> nil;
  If Result then
    Result := Result and GTK_WIDGET_CAN_FOCUS(Widget)
      and GTK_WIDGET_VISIBLE(Widget);
end;

Procedure TGTKControl.SetFocus;
begin
  Inherited SetFocus;
  GTKWidgetSetFocus(Widget);
end;

Destructor TGTKControl.Destroy;
begin
  If Table <> nil then
    If GTK_IS_Widget(Table) then
      If not WindowDestroyed then
        GTK_Widget_Destroy(Table);
  If Widget <> nil then
    If not WindowDestroyed then
      GTK_Widget_Destroy(Widget);  
  Inherited Destroy;
end;

Procedure TGTKControl.SetParentWindow(Val : TCPWinControl);
var
  Right,
  Bottom : integer;
begin
  Inherited SetParentWindow(Val);
  
  If (Widget = nil) or (Val = nil) then
    exit;
  If Not GTK_IS_WINDOW(Widget) then
  If TGTKControl(Val).Widget <> nil then begin
    Right := Width + Left;
    Bottom := Height + Top;
    Widget^.Parent := nil;
    If Left < 0 then FLeft := 0;
    If Top < 0 then FTop := 0;
    If (Bottom > Top) and (Right > Left) then
      GTK_Table_Attach(PGTKTable(TGTKControl(Val).Table),Widget, FLeft, Right,
        FTop, Bottom,0,0,0,0);
    UpdateTable;
  end
  else
    GTK_WIDGET_SET_Parent(Widget,TGTKControl(Val).Widget);
end;

Procedure TGTKControl.UpdateTable;
begin
  If ObjectType = UpperCase('Label') then
    exit;
  If ObjectType = UpperCase('Static') then
    exit;
  If Table^.Parent = nil then begin
    gtk_container_add (PGtkCONTAINER(Widget), Table);
  end else
    GTK_Widget_Reparent(Table, Widget);
end;

Procedure TGTKControl.SetAlign(Val : Integer);
var
  vObjectType : String;
begin
  Inherited SetAlign(Val);
  vObjectType := GetClassWidgetType(ObjectType);
  If Widget <> nil then begin
    If (vObjectType = UpperCase('Label')) or
      (vObjectType = UpperCase('Static'))
    then begin
      GTK_Misc_Set_Alignment(PGTKMisc(Widget),Val / 2,0.5);
    end;
    If vObjectType =  UpperCase('Memo') then begin
      GTK_Misc_Set_Alignment(PGTKMisc(Widget),Val / 2,0.0);
    end;
  end;
end;

Procedure TGTKControl.SetWindowText(Value : PChar);
var
  vObjectType : String;
  DC : hdc;
  area : TGDKRectangle;
begin
  Inherited SetWindowText(Value);
  vObjectType := GetClassWidgetType(ObjectType);
  If Widget <> nil then begin
    If vObjectType = UpperCase('Button') then begin
      GTKWindows.SetWindowText(StrToInt(childhwnd.Strings[0]),Value);
      SetSize(Width,Height);
    end; 
    If (vObjectType = UpperCase('Label')) or
      (vObjectType = UpperCase('Static'))
    then begin
      BKBrush := GetClassBKBrush(ObjectType);
      DC := GetDC(hwnd(self));
      FillRect(DC,Rect(Left, Top,
        Left + Width,  Top + Height),BKBrush);
      ReleaseDC(hwnd(self),dc);
      area.X := Left;
      area.Y := Top;
      area.Height := Top + Height;
      area.Width := Left + Width;
      GTK_Widget_draw(Widget,@area);
      GTK_Label_Set_Text(PGTKLabel(Widget),g_strdup(Value));
      GTK_Widget_draw(Widget,@area);
    end;
    If vObjectType = UpperCase('MenuLabel') then begin
      GTK_Label_Set_Text(PGTKLabel(Widget),g_strdup(Value));
      GTK_Misc_Set_Alignment(PGTKMisc(Widget),0.0,0.5);
    end;
    If vObjectType = UpperCase('EditBox') then begin
      GTK_Entry_Set_Text(PGTKEntry(Widget),g_strdup(Value));
    end;
    If vObjectType = UpperCase('ComboBox') then begin
      GTK_Entry_Set_Text(PGTKEntry(PGTKCombo(Widget)^.Entry), g_strdup(Value));
    end;
    If vObjectType =  UpperCase('Memo') then begin
	GTK_Text_Set_Point(PGTKText(Widget), 0);
	GTK_Text_Forward_Delete(PGTKText(Widget),
  	  GTK_Text_Get_Length(PGTKText(Widget)));
      If Value <> nil then
        GTK_Text_Insert(PGTKText(Widget), null,null,null,
	  g_strdup(Value), -1)
    end;
    If vObjectType = UpperCase('GroupBox') then begin
      GTK_Frame_Set_Label(PGTKFrame(Widget),g_strdup(Value));
    end;
    If vObjectType = UpperCase('Custom') then
      GTK_Window_Set_Title(PGTKWindow(widget), g_strdup(Value));
  end;
end;

Function TGTKControl.GetLeft : Integer;
begin
  If GTK_IS_Window(Widget) and GTK_Widget_Realized(Widget) then begin
  end;
  Result := FLeft;
end;

Function TGTKControl.GetTop : Integer;
begin
  If GTK_IS_Window(Widget) and GTK_Widget_Realized(Widget) then begin
  end;
  Result := FTop;
end;

Function TGTKControl.GetWidth : Integer;
var
  H : integer;
begin
  GTKGetWidgetSize(Widget, FWidth, H); 
  Result := FWidth;
end;

Function TGTKControl.GetHeight : Integer;
var
  W : integer;
begin
  GTKGetWidgetSize(Widget, W, FHeight);
  Result := FHeight;
end;

Procedure TGTKControl.SetSize(W,H : Integer);
begin
  Inherited SetSize(W,H);
  GTKSetWidgetSize(Table,W,H);
  GTKSetWidgetSize(Widget,W,H);
  SetGTKWidgetFont(Widget,AsLogFont(DEFAULTFONT),$0);
  If ObjectType = UpperCase('BUTTON') then begin
    SetWindowPos(StrToInt(ChildhWnd.Strings[0]),0,0,W,H);
  end;
end;

Procedure TGTKControl.SetOrigin(X,Y : Integer);
begin
  Inherited SetOrigin(X,Y);
  GTK_Widget_Set_uPosition(Widget, X, Y);
end;

Procedure TGTKControl.CreateWidgets(theObject : Pointer);
var
  P : Pointer;
begin
  If theObject = nil then
    Abort;
  Widget := PGTKWidget(theObject);
  GTK_Widget_Set_Style(Widget, GTK_Style_New);
  GTK_Widget_Set_RC_Style(Widget);
  Table  := GTK_Table_New(2,2,False);
  GTK_Widget_Set_uPosition(Table, 0, 0);
  GTK_Table_Set_Row_Spacings(PGTKTable(Table),0);
  GTK_Table_Set_Col_Spacings(PGTKTable(Table),0);
  If (Widget^.Parent <> nil) or (GTK_IS_Window(theObject)) then
    UpdateTable;
  If GTK_Is_Window(Widget) then begin
    SetSignal('destroy', @OnDestroy{gtk_exit});
    SetSignal('key-press-event',@OnKeyUpDown);
    SetSignal('expose_event', @DAExpose_Event);
    GTK_WIDGET_SET_APP_PAINTABLE(Widget,True);
  end;
  If GTK_Is_Button(Widget) then begin
    SetSignal('clicked', @OnClicked);
  end;
  If GTK_Is_Drawing_Area(Widget) then {This is Only Temporary!!}
    SetSignal('expose_event', @DAExpose_Event);
  If Widget <> nil then begin
    P := Pointer(Self);
    gtk_widget_set_name(Widget,PChar(@P));
  end;
end;
const
  SS_LineWrap = $10000;

Procedure TGTKControl.UpdateWindow;
var
  ST : Longint;

  Function StyleHas(Styles : Integer) : Boolean;
  begin
    Result := ST or Styles = st;
  end;

begin
  Inherited UpdateWindow;
  If Extended then begin
    st := WNDClassEX.Style;
    If GTK_IS_LABEL(Widget) then
      gtk_label_set_Line_Wrap(PGTKLabel(Widget),StyleHAS(SS_LineWrap));
  end;
end;

Constructor TGTKControl.Create(theObjectType : String);

  Function XNewButton : PGTKWidget;
  begin
    Result := PGTKWidget(GTK_Button_New);
  end;

  Function XNewRadioButton : PGTKWidget;
  {var
    Grp : PGSList;}
  begin
    {GRP := nil;
    If theObject^.Parent <> nil then
      Grp := GTK_Radio_Button_Group(GTK_Radio_Button(theObject^.Parent));
    Result := GTK_Radio_Button_New_With_Label(Grp, '');}
    Result := nil;
  end;

  Function XNewLabel : PGTKWidget;
  begin
    Result := PGTKWidget(GTK_Label_New(''));
  end;

  Function XNewEditBox : PGTKWidget;
  begin
    Result := PGTKWidget(GTK_Entry_New);
  end;

  Function XNewComboBox : PGTKWidget;
  begin
    Result := PGTKWidget(GTK_Combo_New);
  end;

  Function XNewMemo : PGTKWidget;
  begin
    Result := PGTKWidget(GTK_Text_New(null,null));
  end;

  Function XNewGroupBox : PGTKWidget;
  begin
    Result := PGTKWidget(GTK_Frame_New(''));
  end;

  Function XNewCheckBox : PGTKWidget;
  begin
    Result := PGTKWidget(GTK_Check_Button_New_With_Label(''));
  end;

  Function XNewPanel : PGTKWidget;
  begin
    Result := PGTKWidget(GTK_Viewport_New(nil,nil));
  end;

  Function XNewFFDraw : PGTKWidget;
  begin
    Result := PGTKWidget({gtk_FFDraw_new);}gtk_Drawing_Area_new);
   // PGtkFFDraw(Result)^.onPaint := FFDRAWPaint;
  end;

  Function XNewForm : PGTKWidget;
  begin
    Result := PGTKWidget(gtk_Window_new(1));
  end;

Var
  P : Pointer;
begin
  Inherited Create(theObjectType);
  If ObjectType = UpperCase('RadioButton') then
    CreateWidgets(XNewRadioButton);
  If ObjectType = UpperCase('Button') then begin
    CreateWidgets(XNewButton);
    CreateWindowEx( 0, 'Label', 'WindowText',
      WS_VISIBLE or WS_CHILD or SS_Center, 0, 0, 2,2,
        hwnd(Self), 0, HInstance, nil);
  end;
  If (ObjectType = UpperCase('Label')) or
    (ObjectType = UpperCase('Static'))
  then
    CreateWidgets(XNewLabel);
  If ObjectType = UpperCase('EditBox') then
    CreateWidgets(XNewEditBox);
  If ObjectType = UpperCase('ComboBox') then
    CreateWidgets(XNewComboBox);
  If ObjectType = UpperCase('Memo') then
    CreateWidgets(XNewMemo);
  If ObjectType = UpperCase('GroupBox') then
    CreateWidgets(XNewGroupBox);
  If ObjectType = UpperCase('CheckBox') then
    CreateWidgets(XNewCheckBox);
  If ObjectType = UpperCase('Panel') then
    CreateWidgets(XNewPanel);
  If ObjectType = UpperCase('CustomDraw') then
    CreateWidgets(XNewFFDraw);
  If ObjectType = UpperCase('Form') then
    CreateWidgets(XNewForm);
  If Widget <> nil then begin
    P := Pointer(Self);
    gtk_widget_set_name(Widget,PChar(@P));
  end;
end;

Procedure TGTKControl.SetSignal(PName : String; Proc : Pointer);
begin
  If Widget <> nil then
    gtk_signal_connect(PGtkOBJECT(Widget), PChar(PName), gtk_SIGNAL_FUNC (Proc),
      Pointer(Self));
end;

Procedure TGTKControl.SelectCursor;
begin
 { If Widget <> nil then
    If Widget^.Window <> nil then begin
      gdk_window_set_cursor(Widget^.Window, Cursor.Handle);  }
end;

Procedure TGTKhBitmap.SetHandle(Value : PGdkDrawable);
begin
  FHandle := Value;
  Inherited SetHandle(Value);
end;

Function TGTKhBitmap.GetHandle : PGdkDrawable;
begin
  Result := FHandle;
end;

Procedure TGTKhBitmap.UpdateBitmap;
begin
  If Handle = nil then begin
    If Bits <> nil then
      Handle := gdk_Pixmap_create_from_data(nil, Bits, Width, Height,Depth,
        GDKGetColor(0), GDKGetColor($FFFFFF))
    else
      Handle := gdk_Pixmap_new(nil, Width, Height,Depth);
  end;
end;

function RegisterClass(const lpWndClass: TWndClass): ATOM;
begin
  Result := 0;
  If lpWndClass.lpszClassName = '' then
    exit;
  If hClasses.IndexOf(lpWndClass.lpszClassName) > - 1 then
    exit;
  Result := hClasses.AddObject(lpWndClass.lpszClassName,
    TObject(Longint(@lpWndClass))) + 1;
end;

function CreateWindowEx(dwExStyle: DWORD; lpClassName: PChar;
  lpWindowName: PChar; dwStyle: DWORD; X, Y, nWidth, nHeight: Integer;
  hWndParent: HWND; hMenu: HMENU; hInstance: HINST; lpParam: Pointer): HWND;
var
  wnd : String;
  P : Pointer;
begin
  Wnd := GetClassWidgetType(lpClassName);
  If Wnd = 'CUSTOM' then
    Result := CreateWindow('Form')
  else
    Result := CreateWindow(Wnd);
  If Result = 0 then
    exit;
  SetWindowPos(Result, X, Y, nWidth, nHeight);
  SetParent(Result, hWndParent);
  With TGTKControl(Result) do begin
    Extended := True;
    WName := lpWindowName;
    SetWindowText(WName);
    WndProc := GetClassWindowProc(lpClassName);
    BKBrush := GetClassBKBrush(lpClassName); 
    With WNDCLASSEX do begin
      style := dwStyle;
      lpfnWndProc := WNDPROC;
      lpszClassName := lpClassName;
    end;
    UpdateWindow;
    If Widget <> nil then begin
      P := Pointer(Result);
      gtk_widget_set_name(Widget,PChar(@P));
    end;
  end;
end;

function UpdateWindow(hWnd: HWND): BOOL;
begin
  Result := CPWinControls.UpdateWindow(hWnd);
end;

function SetWindowLong(hWnd: HWND; nIndex: Integer; dwNewLong: Longint): Longint;
var
  P : Pointer;
begin
  Result := CPWinControls.SetWindowLong(hWnd, nIndex, dwNewLong);
  If hWnd <> 0 then
    With TGTKControl(hWnd) do
      If Widget <> nil then begin
        P := Pointer(hwnd);
        gtk_widget_set_name(Widget,PChar(@P));
      end;
end;

function GetWindowText(hWnd: HWND): PChar;
begin
  Result := CPWinControls.GetWindowText(hWnd);
end;

function SetWindowText(hWnd: HWND; const lpString: PChar): BOOL;
begin
  Result := CPWinControls.SetWindowText(hWnd,lpString);
end;

function SetFocus(hWnd: HWND): HWND;
begin
  Result := CPWinControls.SetFocus(hWnd);
end;

function GetActiveWindow: HWND;
begin
  Result := CPWinControls.GetActiveWindow;
end;

function GetFocus: HWND;
begin
  Result := CPWinControls.GetFocus;
end;

function DestroyWindow(var Wnd: HWND): BOOL;
begin
  Result := CPWinControls.DestroyWindow(Wnd);
end;

function ShowWindow(hWnd: HWND; nCmdShow: Integer): BOOL;
begin
  Result := CPWinControls.ShowWindow(hWnd,nCmdShow);
end;

function SetWindowPos(hWnd: HWND; X, Y, cx, cy: Integer): BOOL;
begin
  Result := CPWinControls.SetWindowPos(hWnd, X, Y, cx, cy);
end;

function SetParent(hWndChild, hWndNewParent: HWND): HWND;
begin
  Result := CPWinControls.SetParent(hWndChild,hWndNewParent);
end;

function SendMessage(hWnd: HWND; Msg: UINT; wParam: WPARAM;
  lParam: LPARAM): LRESULT;
begin
  Result := CPWinControls.SendMessage(hWnd, Msg, wParam, lParam);
end;

function CallWindowProc(lpPrevWndFunc: TFNWndProc; hWnd: HWND;
  Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT;
begin
  Result := CPWinControls.CallWindowProc(lpPrevWndFunc, hWnd, 
    Msg, wParam, lParam);
end;

function DefWindowProc(hWnd: HWND; Msg: UINT; wParam: WPARAM;
  lParam: LPARAM): LRESULT;
var
  I : Integer;
  Str : String;
  str2 : PChar;
begin
  Result := 0;
  If HWND <> 0 then
    Try
    If TObject(hWnd) is TGTKControl then
      With TGTKControl(hWnd) do begin
        Case Msg of
          WM_GETTEXT :
            begin
              If lParam = 0 then
                exit;
              str := GetText(-1);
              Str2 := PChar(Str);
              For I := 0 to wParam - 1 do
                PChar(lParam)[I] := Str2[I];
            End;

          WM_GETTEXTLENGTH :
            begin
              Result := Length(GetText(-1));
              lParam := Result;
            End;
          WM_SETTEXT:
            begin
              Str := String(PChar(lParam));
              SetWindowText(PChar(str));
            end;
          WM_SETCURSOR:
            begin
              {Why is WParam needed??}
              SelectCursor;
              {What is LPARAM for??}
            end;
          WM_NCHITTEST:
            begin
              {WHAT DOES THIS DO?!}
            end;  
	  WM_KeyDown:
            begin
            end;
        End;
      end;
    Except On E : Exception do
      writeln(E.Message);
    end;
end;

Function CreateWindow(theObjectType : String) : hWnd;
begin
  Result := Longint(TGTKControl.Create(theObjectType));
end;

function BeginPaint(hWnd: HWND; lpPaint: lpPaintStruct): HDC;
begin
  Result := GTKWinDevices.BeginPaint(hWnd,lpPaint);
end;

function EndPaint(hWnd: HWND; const lpPaint: TPaintStruct): BOOL;
begin
  Result := GTKWinDevices.EndPaint(hWnd,lpPaint);
end;

function CreateCompatibleDC(DC: HDC): HDC;
begin
  Result := GTKWinDevices.CreateCompatibleDC(DC);
end;

Function GetDC(wnd : hwnd) : hdc;
begin
  Result := GTKWinDevices.GetDC(Wnd);
end;

function ReleaseDC(Wnd: HWND; hDC: HDC): Integer;
begin
  Result := GTKWinDevices.ReleaseDC(Wnd, hDC);
end;

Function CreatePenIndirect(const LogPen: TLogPen): HPEN;
begin
  Result := CPWinObjects.CreatePenIndirect(logPen);
end;

Function CreatePen(Style : Integer; Width : Integer;
  Color : ColorRef): HPEN;
begin
  Result := CPWinObjects.CreatePen(Style,Width,Color);
end;

function CreateBitmap(Width, Height: Integer; Planes, BitCount: Longint;
  Bits: Pointer): HBITMAP;
var
  Tmp : TBitmap;
begin
  With Tmp do begin
    bmType := 0;
    bmWidth :=  Width;
    bmHeight := Height;
    bmWidthBytes := 0;
    bmPlanes := Planes;
    bmBitsPixel := BitCount;
    bmBits := Bits;
  end;
  Result := CreateBitmapIndirect(Tmp);
end;

function CreateBitmapIndirect(const p1: TBitmap): HBITMAP;
var
  Obj : TGTKhBitmap;
begin
  Result := 0;
  Obj := TGTKhBitmap.Create(Rect(0,0,p1.bmWidth,p1.bmHeight));
  With Obj, p1 do begin
    theType := bmType;
    Width :=  bmWidth;
    Height := bmHeight;
    WidthBytes := bmHeight;
    Planes := bmPlanes;
    Depth := bmBitsPixel;
    Bits := bmBits;
    UpdateBitmap;
  end;
  Result := Longint(Obj);
end;

function CreateCompatibleBitmap(DC: HDC; Width, Height: Integer): HBITMAP;
var
  Depth : Integer;
begin
  Depth := 1;
  If DC = 0 then begin
    DC := GetDC(0);
    If TGTKhDC(DC).GDKVisual <> nil then
      Depth := TGTKhDC(DC).GDKVisual^.Depth;
    ReleaseDC(0,DC);
  end else
    If TGTKhDC(DC).GDKVisual <> nil then
      Depth := TGTKhDC(DC).GDKVisual^.Depth;
  Result := CreateBitmap(Width, Height, 0, Depth, 0);
end;

function CreateDiscardableBitmap(DC: HDC; p2, p3: Integer): HBITMAP;
begin
  Result := 0;
end;

Function CreateBrushIndirect(const LogBrush: TLogBrush): HBRUSH;
begin
  Result := CPWinObjects.CreateBrushIndirect(LogBrush);
end;

Function CreateHatchBrush(nIndex : Longint; Color : TColorRef): HBRUSH;
begin
  Result := CPWinObjects.CreateHatchBrush(nIndex,Color);
end;

Function CreateSolidBrush(Color : TColorRef): HBRUSH;
begin
  Result := CPWinObjects.CreateSolidBrush(Color);
end;

Function CreateFontIndirect(const LogFont: PLogFont): HFont;
begin
  Result := CPWinObjects.CreateFontIndirect(LogFont);
end;

function CreateCaret(hWnd: HWND; hBitmap: HBITMAP; nWidth, nHeight: Integer): BOOL;
begin
  Result := CPWinObjects.CreateCaret(hWnd,hBitmap,nWidth,nHeight);
end;

function GetCaretBlinkTime: UINT;
begin
  Result := CPWinObjects.GetCaretBlinkTime;
end;

function SetCaretBlinkTime(uMSeconds: UINT): BOOL;
begin
  Result := CPWinObjects.SetCaretBlinkTime(uMSeconds);
end;

function DestroyCaret: BOOL;
begin
  Result := CPWinObjects.DestroyCaret;
end;

function HideCaret(hWnd: HWND): BOOL;
begin
  Result := CPWinObjects.HideCaret(hWnd);
end;

function ShowCaret(hWnd: HWND): BOOL;
begin
  Result := CPWinObjects.ShowCaret(hWnd);
end;

function SetCaretPos(X, Y: Integer): BOOL;
begin
  Result := CPWinObjects.SetCaretPos(X,Y);
end;

function GetCaretPos(lpPoint: PPoint): BOOL;
begin
  Result := CPWinObjects.GetCaretPos(lpPoint);
end;

function GetSysColor(nIndex: Integer): DWORD;
begin
  Case nIndex of
    0 : Result := $FFFFFF;
    else
      Result := $FFFFFF;
  end;
end;

function SetBkColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
begin
  Result := CPWinDevices.SetBkColor(DC,Color);
end;

function SetDCBrushColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
begin
  Result := CPWinDevices.SetDCBrushColor(DC,Color);
end;

function SetDCPenColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
begin
  Result := CPWinDevices.SetDCPenColor(DC,Color);
end;

function SetBkMode(DC: HDC; theBkMode: Integer): Integer;
begin
  Result := CPWinDevices.SetBkMode(DC,theBkMode);
end;

function SetTextColor(DC: HDC; Color: TCOLORREF): TCOLORREF;
begin
  Result := CPWinDevices.SetTextColor(DC,Color);
end;

function GetObject(p1: HGDIOBJ; p2: Integer; p3: Pointer): Integer;
begin
  Result := CPWinObjects.GetObject(p1,p2,p3);
end;

function GetStockObject(Index: Integer): HGDIOBJ;
begin
  Result := CPWinObjects.GetStockObject(Index);
end;

Function IntersectClipRect(DC : hDC; Left, Top, Right,
  Bottom : Integer) : Integer;
begin
  Result := CPWinDevices.IntersectClipRect(DC, Left, Top, Right, Bottom);
end;

Function SelectObject(DC: HDC; p2: HGDIOBJ): HGDIOBJ;
begin
  Result := CPWinDevices.SelectObject(DC,p2);
end;

function DeleteObject(var p1: HGDIOBJ): BOOL;
begin
 Result := CPWinDevices.DeleteObject(p1);
end;

function UnrealizeObject(hGDIObj: HGDIOBJ): BOOL;
begin
  Result := CPWinObjects.UnrealizeObject(hGDIObj);
end;

function GetTextExtentPoint(DC: HDC; Str: PChar; Count: Integer;
  var Size: TSize): BOOL;
var
  S : String;
begin
  Size.cX := 0;
  Size.cY := 0;
  Result := False;
  If DC <> 0 then
    If TObject(DC) is TGTKhDC then
      With TGTKhDC(DC) do begin
        SelectFont;
        S := Copy(String(Str),1,Count);
        Size.cx := GDK_String_Width(GCVals^.Font,PChar(S));
        Size.cy := GDK_String_Height(GCVals^.Font,PChar(S));
        Result := True;
      end;
end;

function Arc(hDC: HDC; left, top, right, bottom, startX, startY,
  endX, endY: Integer): BOOL;
begin
  Result := CPWinDevices.Arc(hDC,Left,Top,Right,Bottom,StartX,StartY,
    EndX,EndY);
end;

function MoveToEx(DC: HDC; p2, p3: Integer; p4: PPoint): BOOL;
begin
  Result := CPWinDevices.MoveToEx(DC,p2,p3,p4);
end;

function LineTo(DC: HDC; X, Y: Integer): BOOL;
begin
  Result := CPWinDevices.LineTo(DC,X,Y);
end;

function Polyline(DC: HDC; var Points; Count: Integer): BOOL;
begin
  Result := CPWinDevices.PolyLine(DC, Points,Count);
end;

function PolylineTo(DC: HDC; var Points; Count: Integer): BOOL;
begin
  Result := CPWinDevices.PolyLineTo(DC, Points,Count);
end;

function Chord(DC: HDC; X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer): BOOL;
begin
  Result := CPWinDevices.Chord(DC,X1,Y1,X2,Y2,X3,Y3,X4,Y4);
end;

function Ellipse(DC: HDC; X1, Y1, X2, Y2: Integer): BOOL;
begin
  Result := CPWinDevices.Ellipse(DC,X1,Y1,X2,Y2);
end;

function FillRect(hDC: HDC; const lprc: TRect; hbr: HBRUSH): Integer;
begin
  Result := CPWinDevices.FillRect(hDC,lprc,hbr);
end;

function FrameRect(hDC: HDC; const lprc: TRect; hbr: HBRUSH): Integer;
begin
  Result := CPWinDevices.FrameRect(hDC,lprc,hbr);
end;

function Pie(hDC: HDC; left, top, right, bottom, startX, startY,
  endX, endY: Integer): BOOL;
begin
  Result := CPWinDevices.Pie(hDC,Left,Top,Right,Bottom,StartX,StartY,
    EndX,EndY);
end;

function Polygon(DC: HDC; var Points; Count: Integer): BOOL;
begin
  Result := CPWinDevices.Polygon(DC,Points,Count);
end;

function Rectangle(DC: HDC; X1, Y1, X2, Y2: Integer): BOOL;
begin
  Result := CPWinDevices.Rectangle(DC,X1,Y1,X2,Y2);
end;

function InvertRect(hDC: HDC; const lprc: TRect): Integer;
begin
  Result := CPWinDevices.InvertRect(hDC,lprc);
end;

function DrawFocusRect(hDC: HDC; const lprc: TRect): BOOL;
var
  FC, Color : ColorRef;

   Procedure DrawDottedHLine(X1,Y,X2 : Integer);
   var
     I : Integer;
   begin
     For I := X1 to X2 do begin
       If not Odd(I) then
         SetPixel(hDC,I, Y, Color);
     end;
   end;

   Procedure DrawDottedVLine(X,Y1,Y2 : Integer);
   var
     I : Integer;
   begin
     For I := Y1 to Y2 do begin
       If not Odd(I) then
         SetPixel(hDC,X, I, Color);
     end;
   end;

   Procedure Win32Draw;
   begin
    Color := $0;
    FC := SetTextColor(hDC,Color);
    TGTKhDC(hDC).SelectFont;
    Color := not TCPhPen(TGTKhDC(hDC).Brush).Color;
    DrawDottedHLine(lprc.Left + 1, lprc.Top,
      lprc.right - 1);
    DrawDottedVLine(lprc.Left, lprc.Top + 1,
      lprc.Bottom - 1);
    DrawDottedVLine(lprc.Right, lprc.Top + 1,
      lprc.Bottom - 1);
    DrawDottedHLine(lprc.Left + 1, lprc.Bottom,
      lprc.right - 1);
    SetTextColor(hDC,FC);
  end;

  Procedure GTKDraw;
  begin
    with TGTKhDC(hDC) do begin
     { gtk_draw_focus(gtk_widget_get_style(Wnd.Widget),
        Handle,lprc.Left,lprc.Top, lprc.Right - lprc.Left,
        lprc.Bottom - lprc.Top);  }
    end;
  end;

begin
  Result := False;
  If (hDC <> 0) then
    If TObject(hDC) is TGTKhDC then
      With TGTKhDC(hDC) do begin
        Win32Draw;
        Result := True;
      end;
end;

function RoundRect(DC: HDC; X1, Y1, X2, Y2, X3, Y3: Integer): BOOL;
begin
  Result := CPWinDevices.RoundRect(DC,X1,Y1,X2,Y2,X3,Y3);
end;

function BitBlt(DestDC: HDC; X, Y, Width, Height: Integer; SrcDC: HDC;
  XSrc, YSrc: Integer; Rop: DWORD): BOOL;
begin
  Result := CPWinDevices.BitBlt(DestDC,X,Y,Width,Height,SrcDC,
    XSrc,YSrc,Rop);
end;

function StretchBlt(DestDC: HDC; X, Y, Width, Height: Integer; SrcDC: HDC;
  XSrc, YSrc, SrcWidth, SrcHeight: Integer; Rop: DWORD): BOOL;
var
  Tmp : Longint;
begin
  Result := False;
  If (DestDC <> 0) and (SrcDC <> 0) then
    If (TObject(DestDC) is TGTKhDC) and
      (TObject(SrcDC) is TGTKhDC)
    then begin
      Case Rop of
        R2_White, R2_Black:
	  Rectangle(DestDC, X, Y, Width, Height);
      else begin
          Tmp := TGTKhDC(DestDC).ROPMode;
          SetRop2(DestDC,Rop);
          GDKStrechBlt(TGTKhBitmap(DestDC).Handle, TGTKhDC(DestDC).gc, X, Y, 
            width, height, TGTKhDC(SrcDC).Handle, XSrc, YSrc, SrcWidth, SrcHeight);
          SetRop2(DestDC,Tmp);
        end;
      end;
      Result := True;
    end;
end;

function ExtFloodFill(DC: HDC; X, Y: Integer; Color: COLORREF; FillType: UINT): BOOL;
begin
  Result := False;
end;

function DrawText(hDC: HDC; lpString: PChar; nCount: Integer;
  var lpRect: TRect; uFormat: UINT): Integer;
const
  Buffer : PChar = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
var
  AP, CP : TSize;
  FinalStrings : TStrings;
    
  Function LeftOffset : UINT;
  begin
    If (uFormat and DT_Right) = DT_Right then 
      Result := DT_Right
    else
      If (uFormat and DT_CENTER) = DT_CENTER then 
        Result := DT_CENTER
    else
        Result := DT_LEFT;
  end;  

  Function TopOffset : UINT;
  begin
    If (uFormat and DT_BOTTOM) = DT_BOTTOM then 
      Result := DT_BOTTOM
    else
      If (uFormat and DT_CENTER) = DT_CENTER then 
        Result := DT_CENTER
    else
      If (uFormat and DT_VCENTER) = DT_VCENTER then
        Result := DT_VCENTER
    else
      Result := DT_Top;
  end;  

  Procedure AddLine(TMPLine : String; CW : Integer);
  var
    ACPL,
    I,TW,ft,st,
    lns : Integer;
    FP : TSize;
  begin
    GetTextExtentPoint(hDC, PChar(TMPLine), Length(TMPLine), FP);
    TW := FP.cX;
    lns := 0;
    If (CW < TW) and  (uFormat and DT_WORDBREAK = DT_WORDBREAK) then begin
      ACPL := TW div FP.cX;
      St := 1;
      Repeat
        ft := st + ACPL;
        If ft > Length(TMPLine) then
          ft := Length(TMPLine);
        For I := ft downto st do
          Case TMPLine[I] of
            ' ' :
              begin    
                If (not (Length(TMPLine) - ST <= ACPL)) then begin
	          FinalStrings.Add(Copy(TMPLine,ST,I));
  		  ST := I + 1;
		  FT := -1;
  		end;
       	        Break;
              end;
          end;
        If FT <> -1 then
          For I := ft to Length(TMPLine) do
            Case TMPLine[I] of
              ' ' :
                begin    
                  If (not (Length(TMPLine) - ST <= ACPL)) then begin
   	            FinalStrings.Add(Copy(TMPLine,ST,I));  
  		    ST := I + 1;
		    FT := -1;
  		  end;
       	          Break;
                end;
            end;
        If FT = -1 then begin
          Lns := lns + 1;
	end else
	  ST := FT;
      until (st >= Length(TMPLine));
    end
    else
      FinalStrings.Add(Copy(TMPLine,0,Length(TMPLine)));  
  end;

  Procedure CalcRect(var lRect : TRect);
  var
    I,
    FirstChar,
    CW,
    LastWidth, LastHeight,
    LongLine, TW : Integer;
    Buffer : PChar;
    TXT : AnsiString;
    FP : TSize;
    Strings : Tstrings;
  begin
    Strings := TStringList.Create;
    GetTextExtentPoint(hDC, lpString, Length(AnsiString(lpString)), FP);
    TW := FP.cX;
    Buffer := 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' +
      'abcdefghijklmnoqrstuvwxyz';
    GetTextExtentPoint(hDC,Buffer,52,FP);
    FP.cX := FP.cX div 52;
    If TW > lRect.Right - lRect.Left then
      CW := lRect.Right - lRect.Left
    else
      CW := TW;
    TXT := AnsiString(lpString);
    FirstChar := 1;
    For I := 1 to Length(Txt) do
      If TXT[I] = #13 then begin
	Strings.Add(Copy(Txt,FirstChar,I - 1));
	Strings.Add(#13#10);
	FirstChar := I + 1;
	If FirstChar <= Length(Txt) then
	  If Txt[FirstChar] = #10 then
	    FirstChar := FirstChar + 1;
      end else
        If I >= Length(Txt) then begin
  	  Strings.Add(Copy(Txt,FirstChar,I - 1));
        end;
    If Strings.Count > 0 then begin
      For I := 0 to Strings.Count - 1 do
        If Strings.Strings[I] <> #13#10 then
	  AddLine(Strings.Strings[I],CW); 
    end else begin
      AddLine(TXT,CW);
    end;
    LastWidth := 0;
    LastHeight := AP.CY * FinalStrings.Count;
    LongLine := 0;
    For I := 0 to FinalStrings.Count - 1 do begin
      GetTextExtentPoint(hDC, PChar(FinalStrings.Strings[I]),
        Length(FinalStrings.Strings[I]), FP);
      If LastWidth < FP.CX then begin
        LastWidth := FP.CX;
        LongLine := I;
      end;
    end;
    Case TopOffset of
      DT_Top : LRect.Bottom := LRect.Top + LastHeight + AP.CY;
      DT_Bottom : LRect.Top := LRect.Bottom - LastHeight;
      DT_Center,
      DT_VCenter : 
        begin
          LRect.Top := ((LRect.Bottom - LRect.Top) div 2) - (LastHeight) div 2;
	  LRect.Bottom := LRect.Bottom + LRect.Top;// + LastHeight + AP.CY;
	end;
    end;
    Case LeftOffset of
      DT_Left :
        begin
          GetTextExtentPoint(hDC, PChar(FinalStrings.Strings[LongLine]),
            Length(FinalStrings.Strings[LongLine]), FP);
          LRect.Right := LRect.Left + FP.CX;
        end;
      DT_Right :
        begin
          GetTextExtentPoint(hDC, PChar(FinalStrings.Strings[LongLine]),
            Length(FinalStrings.Strings[LongLine]), FP);
          LRect.Left := LRect.Right - FP.CX;
        end;
     end;
    Strings.Free;
  end;

  Procedure TextSingleLine(aRect : TRect; AString : PChar; Count : Integer);
  var
    X,Y : Integer;
    tRect : TRect;
    VP : TSize;
  begin
    GetTextExtentPoint(hDC, AString, Count, VP);
    Case TopOffset of
      DT_Top : Y := aRect.Top;
      DT_Bottom : Y := aRect.Bottom - VP.cY;
      DT_Center,
      DT_VCenter : Y := (aRect.Bottom div 2) - (VP.cY div 2);
    end;
    Case LeftOffset of
      DT_Left : X := ARect.Left;
      DT_Right : X := ARect.Right - VP.cX;
      DT_Center : X := (ARect.Right div 2) - (VP.cX div 2);
    end;
    tRect := Rect(TGTKhDC(hDC).lRect.Left,
      TGTKhDC(hDC).lRect.Top,
      TGTKhDC(hDC).lRect.Right,
      TGTKhDC(hDC).lRect.Bottom);
    TGTKhDC(hDC).lRect := Rect(aRect.Left,  
      aRect.Top, aRect.Right, aRect.Bottom);
    TextOut(hDC,X,Y,aString,Count);
    TGTKhDC(hDC).lRect := Rect(tRect.Left,
      tRect.Top, tRect.Right, tRect.Bottom);
    Result := 1;
  end;

  Procedure TextMultiLine;
  var
    I, X,Y : Integer;
    LastHeight : Integer;
    VRect,
    tRect : TRect;
    FP : TSize;
  begin
    FinalStrings := TStringList.Create;
    tRect := Rect(lpRect.Left, lpRect.Top, lpRect.Right, lpRect.Bottom);
    If uFormat and DT_CalcRect = DT_CalcRect then begin
      CalcRect(LPRect);
      Result := 1;
      exit;
    end
    else
      CalcRect(TRect);
    LastHeight := tRect.Top;
    VRect := Rect(TGTKhDC(hDC).lRect.Left,
      TGTKhDC(hDC).lRect.Top,
      TGTKhDC(hDC).lRect.Right,
      TGTKhDC(hDC).lRect.Bottom);
    TGTKhDC(hDC).lRect := Rect(tRect.Left,
       tRect.Top, tRect.Right, tRect.Bottom);
    For I := 0 to FinalStrings.Count - 1 do begin
      GetTextExtentPoint(hDC, PChar(FinalStrings.Strings[I]),
        Length(FinalStrings.Strings[I]), FP);
      Case LeftOffset of
        DT_Left : X := tRect.Left;
        DT_Right : X := tRect.Right - FP.cX;
        DT_Center : X := tRect.Left + ((tRect.Right - tRect.Left) div 2) - (FP.cX div 2);
      end;
      Y := LastHeight;
      TextOut(hDC,X,Y,PChar(FinalStrings.Strings[I]),Length(FinalStrings.Strings[I]));
      With TGTKhDC(hDC) do begin
        SelectFont;
        SelectBrush;
        LastHeight := Y + GCVals^.Font.ascent + GCVals^.Font.descent;
      end;
      Result := 1;
    end;
    TGTKhDC(hDC).lRect := Rect(VRect.Left,
      VRect.Top, VRect.Right, VRect.Bottom);
    FinalStrings.Free;
  end;

begin
  Result := 0;
  Try
    GetTextExtentPoint(hDC,Buffer,52,CP);
    CP.cX := CP.cX div 52;
    GetTextExtentPoint(hDC, lpString, nCount, AP);
    If uFormat and DT_SingleLine = DT_SingleLine then
      TextSingleLine(lpRect, lpString, nCount)
    else
      TextMultiLine;
  finally
  end;      
end;

function TextOut(DC: HDC; X, Y: Integer; Str: PChar; Count: Integer): BOOL;
var
  xt,xt2 : Longint;
  ARect : TRect;
begin
  Result := False;
  If (DC <> 0) then
    If TObject(DC) is TGTKhDC then
      With TGTKhDC(DC) do begin
        If Handle = nil then
          exit;
        UpdatePlaceP;
        SelectFont;
        xt := GCVals^.Font.ascent;
        xt2 := GCVals^.Font.descent;
        SelectBrush;
        ARect.Left := X;
        ARect.Top := Y;
        ARect.Bottom := Y + xt + xt2;
        ARect.Right := X + GDK_String_Width(GCVals^.Font,str);
        FillRect(DC,ARect,Brush);
        SelectFont;
        gdk_draw_text(Handle, TGTKhFont(Font).Handle,
          gc, X, Y + xt, Str, Count);
        If TCPhFont(Font).Underline then
          GDK_Draw_Line(Handle, GC, ARect.Left, ARect.Bottom - 2,
            ARect.Right, ARect.Bottom - 2);
        If TCPhFont(Font).StrikeOut then
          GDK_Draw_Line(Handle, GC, ARect.Left, 2 + Y + (xt + xt2) div 2,
            ARect.Right, 2 + Y + (xt + xt2) div 2);
        Result := True;
      end;
end;

function GetBkColor(DC: HDC): COLORREF;
begin
  Result := CPWinDevices.GetBkColor(DC);
end;

function GetDCBrushColor(DC: HDC): COLORREF;
begin
  Result := CPWinDevices.GetDCBrushColor(DC);
end;

function GetDCPenColor(DC: HDC): COLORREF;
begin
  Result := CPWinDevices.GetDCPenColor(DC);
end;

function GetBkMode(DC: HDC): Integer;
begin
  Result := CPWinDevices.GetBkMode(DC);
end;

function GetPixel(DC: HDC; X, Y: Integer): COLORREF;
begin
  Result := CPWinDevices.GetPixel(DC,X,Y);
end;

function SetPixel(DC: HDC; X, Y: Integer; Color: COLORREF): COLORREF;
begin
  Result := CPWinDevices.SetPixel(DC,X,Y,Color);
end;

function SetPixelV(DC: HDC; X, Y: Integer; Color: COLORREF): Integer;
begin
  Result := CPWinDevices.SetPixelV(DC,X,Y,Color);
end;

{var
  W : PGTKWidget;}

Function GetCursorPos(var lpPoint: TPoint): BOOL;
{var
  X,Y : integer;}
begin
{  X := 0;
  Y := 0;
  Result := True;
  Try
    If W <> nil then begin
      W := GTK_Window_New(GTK_Window_Popup);
      GTK_Window_Set_Default_Size(PGTKWindow(W), 1, 1);
    end;
    GTK_Window_Set_Position(PGTKWindow(W), GTK_Win_Pos_Mouse);
    GTK_Widget_Show(W);
    GTK_Widget_Get_Pointer(W,@X,@Y);
    GTK_Widget_Hide(W);
    GTK_Widget_Unrealize(W);
  except
    Result := False;
  end;
  If Result then begin
    lpPoint.X := X;
    lpPoint.Y := Y;
  end;}
  Result := GDKGetCursorPos(lpPoint);
end;

Procedure SetCursorPos(X, Y : Integer);
begin
  GDKSetCursorPos(X,Y);
end;

function WindowFromPoint(Point: TPoint): HWND;
var
  TMP : PGDKWindow;
  UD  : gPointer;
  Obj : TObject;
begin
  Result := 0;
  Tmp := gdk_window_at_pointer(@Point.X, @Point.Y);
  If TMP <> nil then begin
    If TMP^.user_data <> nil then
      Try
        GetUser_Data(Tmp, UD);
        If UD <> nil then
          Obj := TObject(UD)
        else
          Obj := nil;
      except
        Obj := nil;
      end;
      If Obj <> nil then
        Try
          If Obj is TGTKControl then
            Result := longint(Obj);
        except
        end;
  end;
end;

Function LoadCursor(hInstance: HINST; lpCursorName: PAnsiChar): HCURSOR;
begin
  Result := 0;
  {Result := Longint(TGTKhCursor.Create(lpCursorName))}
end;

function GetClipBox(DC: HDC; var lRect: TRect): Integer;
begin
  Result := CPWinDevices.GetClipBox(DC,lRect);
end;

function SetROP2(DC: HDC; p2: Integer): Integer;
begin
  Result := CPWinDevices.SetRop2(DC,P2);
end;

Function GetDeviceCaps(dc : hDC; cap : Integer) : Integer;
var
  Obj : TGTKhDC;
  I : integer;
  TMP1,TMP2 : Integer;
begin
  Result := 0;
  If Dc = 0 then
    exit;
  Try
    Obj := TGTKhDC(DC);
  except
    exit;
  end;
  Case cap of
    HORZSIZE:
      begin
        Tmp1 := GetDeviceCaps(DC, HORZRES);
        Tmp2 := GetDeviceCaps(DC,LOGPIXELSX);
        If Tmp2 <> 0 then
          Result := MulDiv(Tmp1, 254, Tmp2 * 10 )
        else
          result := 0;
      end;
    VERTSIZE:
      begin
        Tmp1 := GetDeviceCaps(DC, VERTRES);
        Tmp2 := GetDeviceCaps(DC, LOGPIXELSY);
        If Tmp2 <> 0 then
          Result := MulDiv(Tmp1, 254, Tmp2 * 10 )
        else
          result := 0;
      end;
    HORZRES:
      begin
        With Obj do
          If Handle <> nil then
            Result := GetSize.cX;
      end;
    VERTRES:
      begin
        With Obj do
          If Handle <> nil then
            Result := GetSize.cY;
      end;
    BITSPIXEL:
      begin
        With Obj do
          If GDKVisual <> nil then
            Result := GDKVisual.depth
      end;
    NUMBRUSHES:
      Result := hBrushes.Count;
    NUMPENS:
      Result := hPens.Count;
    NUMFONTS:
      Result := hFonts.Count;
    NUMCOLORS:
      begin
        Tmp1 := GetDeviceCaps(DC,BITSPIXEL);
        If Tmp1 <= 8 then
          Result := 1 SHL I
        else
          Result := -1
      end;
    COLORRES:
      begin
      //  Result := GetDeviceCaps(DC,BITSPIXEL);
      end;
    LOGPIXELSX:
      begin
        Tmp1 := GDK_Screen_Width;
        Tmp2 := GDK_Screen_Width_MM;
        Result := MulDiv(Tmp1, 254, Tmp2 * 10 );
      end;
    LOGPIXELSY:
      begin
        Tmp1 := GDK_Screen_Height;
        Tmp2 := GDK_Screen_Height_MM;
        Result := MulDiv(Tmp1, 254, Tmp2 * 10 );
      end;
    PHYSICALWIDTH:
      begin
      end;
    PHYSICALHEIGHT:
      begin
      end;
    else
       Writeln('GetDeviceCaps : an unknown/unimplemented ' +
         'variable(' + intToStr(cap) + ') was requested!');
       Result := 0;
    end;
end;

function GetSystemMetrics(nIndex: Integer): Integer;
begin
  Case nIndex of
    SM_CXSCREEN :
      begin
        Result := gdk_screen_width;
      end;
    SM_CYSCREEN :
      begin
        Result := gdk_screen_height;
      end;
    else
       Writeln('GetSystemMetrics : an unknown/unimplemented ' +
         'variable(' + intToStr(nIndex) + ') was requested(so go fix/impliment it)!');
       Result := 0;
  end;
end;

procedure PostQuitMessage(nExitCode: Integer);
begin
  GTK_Exit(nExitCode);
end;

var
  MessageBoxWindow: TWndClass = (
    style : CS_DBLCLKS or CS_HREDRAW or CS_VREDRAW;
    lpfnWndProc  : @DefWindowProc;
    cbClsExtra   : 0;
    cbWndExtra   : 0;
    hInstance    : 0;
    hIcon        : 0;
    hCursor      : 0;
    hbrBackground: 0;
    lpszMenuName : nil;
    lpszClassName: 'MessageBoxWindow' );
  Box, Lab,
  OK, CANCEL, YES, NO, ABORT,
  RETRY, IGNORE, HELP : CPWindows.hwnd;
  BOXDone : Boolean;
  MResult : Integer;
  
  Function MSGWindowProc(Wnd : CPWindows.hwnd; Msg : UINT; WParam : WParam;
    LParam : LParam) : LResult;
  begin
    Case Msg of
      WM_Close :
        begin
 	  BoxDone := True;
        end;
      WM_Command :
        begin
          If lParam = 0 then
            exit
          else
	    If LParam = OK then
              MResult:=1
          else
	    If LParam = CANCEL then
              MResult:=2
          else
	    If LParam = ABORT then
              MResult:=3
          else
	    If LParam = RETRY then
              MResult:=4
          else
	    If LParam = IGNORE then
              MResult:=5
          else
	    If LParam = YES then
              MResult:=6
          else
	    If LParam = NO then
              MResult:=7
          else
	    If LParam = HELP then
              begin
                //MResult:=;
                exit;
              end
          else
            exit;
          BoxDone := True;
	  exit;
        end;  
      else
        Result := DefWindowProc(Wnd,Msg,WParam,LParam);
    end;
  end;

function MessageBox(hWnd: HWND; lpText, lpCaption: PChar; uType: UINT): Integer;
var
  X, HasHelp : Integer;
  FP : TSize;
  Width,Height,
  BWTotal,ButtonWidth,
  ButtonHeight,ButtonSpace : Integer;
  
  Function CreateButton(Caption : PChar) : CPWindows.hwnd;
  begin
    Result:=CreateWindowEx( 0, 'BUTTON', Caption,
      WS_VISIBLE or WS_CHILD or BS_PUSHLIKE,
      X, Height - 4 - ButtonHeight, ButtonWidth,
      ButtonHeight, Box, 0, HInstance, nil);
    Inc(X, ButtonWidth + ButtonSpace);
  end;

  Procedure CreateOK;
  begin
    OK := CreateButton('OK');
  end;

  Procedure CreateCancel;
  begin
    Cancel := CreateButton('Cancel');
  end;

  Procedure CreateAbort;
  begin
    Abort := CreateButton('Abort');
  end;

  Procedure CreateRetry;
  begin
    Retry := CreateButton('Retry');
  end;

  Procedure CreateIgnore;
  begin
    Ignore:= CreateButton('Ignore');
  end;

  Procedure CreateYes;
  begin
    Yes := CreateButton('Yes');
  end;

  Procedure CreateNo;
  begin
    No := CreateButton('No');
  end;

  Procedure CreateHelp;
  begin
    Help := CreateButton('Help');
  end;

  Procedure SetBWTotal(Int : Integer);
  var
    TDC : hDC;
    SW, SH,
    CW, CH,
    TW   : Integer;
    Buffer : PChar;
    TXT : AnsiString;
    ARect : TRect;
  begin
    SW := GetSystemMetrics(SM_CXSCREEN);
    SH := GetSystemMetrics(SM_CYSCREEN) div 2;
    TDC := GetDC(BOX);
    GetTextExtentPoint(TDC, lpText, Length(AnsiString(lpText)), FP);
    TW := FP.cX;
    Buffer := 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' +
      'abcdefghijklmnoqrstuvwxyz';
    GetTextExtentPoint(TDC,Buffer,52,FP);
    FP.cX := FP.cX div 52;
    If TW > Round(Muldiv(SW,2,3)) then
      CW := Round(Muldiv(SW,2,3))
    else
      CW := TW;
    ButtonWidth := (50 * FP.cX) div 4;
    ButtonHeight := (14 * FP.cY) div 8;
    ButtonSpace := (4 * FP.cX) div 4;
    BWTotal := ButtonWidth * (Int + HasHelp) +
        ButtonSpace * (Int + HasHelp - 1);
    If BWTotal > CW then
      CW := BWTotal;
    TXT := AnsiString(lpText);
    ARect.Left := 0;
    ARect.Top := 0;
    ARect.Right := cw;
    Arect.Bottom := SH * 2;
    DrawText(TDC,lpText,Length(Txt), ARect,DT_WordBreak or
      DT_ExpandTabs or DT_CalcRect);
    CW := ARect.Right;
    CH := Arect.Bottom;
    If BWTotal > CW then
      CW := BWTotal;
    Width :=  MulDiv(8, FP.cX, 4) + CW + MulDiv(8, FP.cX, 4);
    If Width < 85 then
      Width := 85;
    Height := MulDiv(8, FP.cY, 4) + CH + MulDiv(6, FP.cY, 8) +
      ButtonHeight;
    SetWindowPos(Box,(SW div 2) - (Width div 2),SH - (Height div 2),
      Width, Height);
    SetWindowPos(LAB, MulDiv(8, FP.cX, 4),MulDiv(8, FP.cX, 4),
       Width - MulDiv(8, FP.cX, 4), CH);
    SetWindowPos(Box,(SW div 2) - (Width div 2),SH - (Height div 2),
      Width, Height);
    If Int + HasHelp = 1 then
      X := Round((Width div 2) - (ButtonWidth div 2))
    else
      X := (Width - BWTotal) div 2;
    ReleaseDC(BOX, TDC);
  end;

begin
  If not BoxDone then
    exit;
  Result := 0;
  MResult := 0;
  BoxDone := False;
  Box:=CreateWindowEx(WS_EX_PALETTEWINDOW or WS_EX_CONTROLPARENT,
    MESSAGEBOXWindow.lpszClassName,lpCaption,
    WS_POPUP or WS_CAPTION,
    0, 0, 1, 1, hwnd, 0, HInstance, nil);
  ShowWindow(Box,SW_Show);
  lab := CreateWindowEx( 0, 'STATIC', lpText,
    WS_VISIBLE or WS_CHILD or SS_LEFT or SS_LineWrap,
    10, 10, 300, 15, Box, 0, HInstance, nil);
  HasHelp := Integer((UType and MB_HELP) = MB_HELP);
  If (UType and MB_OKCANCEL) = MB_OKCANCEL then begin
    SetBWTotal(2);
    CreateOK;
    CreateCancel;
  end
  else
    If (UType and MB_ABORTRETRYIGNORE) = MB_ABORTRETRYIGNORE then begin
      SetBWTotal(3);
      CreateAbort;
      CreateRetry;
      CreateIgnore;
    end
  else
    If (UType and MB_YESNOCANCEL) = MB_YESNOCANCEL then begin
      SetBWTotal(3);
      CreateYes;
      CreateNo;
      CreateCancel;
    end
  else
    If (UType and MB_YESNO) = MB_YESNO then begin
      SetBWTotal(2);
      CreateYes;
      CreateNo;
    end
  else
    If (UType and MB_RETRYCANCEL) = MB_RETRYCANCEL then begin
      SetBWTotal(2);
      CreateRetry;
      CreateCancel;
    end
  else begin
    SetBWTotal(1);
    CreateOK;
  end;

  If Boolean(HasHelp) then
    CreateHelp;

  SetWindowLong(Box, GWL_WNDPROC, Longint(@MSGWindowProc));
  UpdateWindow(Box);
  While Not BoxDone do
    GTK_Main_Iteration;
  Result := MResult;
  DestroyWindow(OK); 
  DestroyWindow(Cancel);
  DestroyWindow(Yes);
  DestroyWindow(NO);
  DestroyWindow(Abort);
  DestroyWindow(Retry);
  DestroyWindow(Ignore); 
  DestroyWindow(Lab);
  DestroyWindow(Help);
  DestroyWindow(Box);
  Box := 0;
end;

initialization
  gtk_init (@argc, @argv);
  gtk_rc_init;
  GDKUpdateFonts;
  hClasses := TStringList.Create;
  hObjects := TList.Create;
  dummyP := TComponent.Create(nil);
  InitializeDevices(TGTKhDC);
  InitializeBrushes(TGTKhBrush);
  InitializePens(TGTKhPen);
  InitializeFonts(TGTKhFont);
  CBH := Callbacks;
  RegisterClass(MessageBoxWindow);
  MESSAGEBOXWindow.hbrBackground := GRAYBRUSH;
  BoxDone := True;
Finalization
  {$IfNDef Win32}
  FontList.Free;
  {$EndIf}
end.