unit gtkwinobjects;

{$Mode Delphi}

interface

Uses CPWindows, CPWinObjects, CPWinDevices, GDKRoutines, GDK;

Type
  TGTKhPen = Class(TCPhPen)
  Public
    Procedure hDCSelect(DC : hDC); Override;
  end;

  TGTKhBrush = Class(TCPhBrush)
  Public
    Procedure hDCSelect(DC : hDC); Override;
  end;

  TGTKhFont = Class(TCPhFont)
  Public
    Handle : PGDKFont;
    Class Function CreateFontIndirect(const LogFont: PLogFont): HFont; Override;
    Procedure hDCSelect(DC : hDC); Override;
  end;

implementation

Uses GTKWindows, GTKWinDevices;

Procedure TGTKhPen.hDCSelect(DC : hDC);
var
  Obj : TGTKhDC;
  theColor : TColorRef;
begin
  Inherited hDCSelect(DC);
  If DC = 0 then
    exit;
  Obj := TGTKhDC(DC);
  With Obj do
    If Handle <> nil then begin
      GDKSetBrushBitmap(GC,BS_Solid,0);
      Case RopMode of
        R2_BLACK : theColor := $0;
	R2_WHITE : theColor := $FFFFFF;
	else begin
	  theColor := Self.Color;
	end;
      end;
      GDKSetForegroundColor(GC, theColor);
      gdk_gc_set_line_attributes(GC,Self.Width,
        GDKGetPenStyle(Self.Style),GDK_CAP_BUTT,
        GDK_JOIN_BEVEL);
      GDKSetDashStyle(GC,Self.Style);
      gdk_gc_get_values(GC, GCVals);
    end;
end;

Procedure TGTKhBrush.hDCSelect(DC : hDC);
var
  theColor : TColorRef;
  Obj : TGTKhDC;
begin
  Inherited hDCSelect(DC);
  If DC = 0 then
    exit;
  Obj := TGTKhDC(DC);
  With Obj do
    If Handle <> nil then begin
      GDKSetBrushBitmap(GC,BS_Solid,0);
      Case RopMode of
        R2_BLACK : theColor := $0;
	R2_WHITE : theColor := $FFFFFF;
	else begin
	  theColor := Self.Color;
          GDKSetBrushBitmap(GC, Self.Style,
            Self.Hatch);
	end;
      end;
      GDKSetForegroundColor(GC, theColor);
      gdk_gc_set_subwindow(GC, GDK_CLIP_BY_CHILDREN);
      gdk_gc_get_values(GC, GCVals);
    end;
end;

Class Function TGTKhFont.CreateFontIndirect(const LogFont: PLogFont): HFont;
var
  Obj : TGTKhFont;
  Tmp1,Tmp2,
  LPY : Longint;
begin
  Result := Inherited CreateFontIndirect(LogFont);
  Obj := TGTKhFont(Result);
  With Obj, LogFont^ do begin
    Tmp1 := GDK_Screen_Height;
    Tmp2 := GDK_Screen_Height_MM;
    LPY := MulDiv(Tmp1, 254, Tmp2 * 10 );
    Size      := -MulDiv(lfHeight, 98, LPY);
    Handle    := GDKGetFont(AsLogFont(hFont(Obj)));
  end;
end;

Procedure TGTKhFont.hDCSelect(DC : hDC);
var
  Obj : TGTKhDC;
begin
  Inherited hDCSelect(DC);
  If DC = 0 then
    exit;
  Obj := TGTKhDC(DC);
  With Obj do
    If Handle <> nil then begin
      GDKSetBrushBitmap(GC,BS_Solid,0);
      If Self.Handle <> nil then
        gdk_gc_set_font(GC, Self.Handle);
      If Self.Color <> -1 then
        GDKSetForegroundColor(GC, Self.Color)
      else
        GDKSetForegroundColor(GC,0);
      gdk_gc_set_line_attributes(GC,1,
        PS_Solid,GDK_CAP_BUTT, GDK_JOIN_BEVEL);
      gdk_gc_get_values(GC, GCVals);
    end;
end;

end.
 