{

  This program is distributed in the hope that it will be useful,

  but WITHOUT ANY WARRANTY; without even the implied warranty of

  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

  GNU General Public License (http://www.gnu.org/copyleft/gpl.html)

  for more details

  

************************************************************************

  WOL - Window Objects Library - version 0.00.6



  This library is a framework for Windows/GTK programming with the Free

  Pascal Compiler. The main objective is to make porting programs

  between Windows and Linux easier by using the same framework.



  If you find bugs or want colaborate with us to improve, please report

  to authors. It was originally writen by Carlos Alberto de Oliveira

  (carlosnatal@bol.com.br) and is maintened by the following authors:



  Carlos Alberto de Oliveira - carlosnatal@bol.com.br

  Andrew Johnson             - aj_genius@hotmail.com



  Missing you... Visit http://www.olhodetigre.hpg.com.br

                       http://www.freepascal.org

**************************************************************************

FileName : WOLBrush.pp

Current File Version* : .6



Author(s) : Carlos Alberto de Oliveira(carlosnatal@bol.com.br)



Comments  : This File attempts to provide a Brush component, similar to

            Delphi/Kylix TCanvas, though not entirely compatible.



*See end of file for File history

**************************************************************************

}



{$IFDEF FPC}

  {$MODE DELPHI}

{$ENDIF}



unit wolBrush;



interface



{$Include WOLDefs.Inc}

Uses

  Classes,

  {$IFDef GTKWIN}

  CPWindows,GTKWindows,

  {$Else}

  Windows,

  {$EndIF}

  wolDefs,

  wolRMan,

  wolDBG;



const

   NADA = 0;



type



   TBrushEvt = TNotifyEventA;



   TwolBrushStyle = ( wol_bsSolid,

                      wol_bsClear,

                      wol_bsHorizontal,

                      wol_bsVertical,

                      wol_bsFDiagonal,

                      wol_bsBDiagonal,

                      wol_bsCross,

                      wol_bsDiagCross);



   TwolBrushes = class(TPersistent)

   private

     FResource :TResourceItem;

     FColor    :TCOLORREF;

     FLogBrush :TLOGBRUSH;

     {FBitMap   :TBitMap;}

     FwolHandle:HBRUSH;

     FStyle    :TwolBrushStyle;

     FOnChange :TBrushEvt;



     function  WOL_GetStyle:TwolBrushStyle;

     procedure WOL_SetStyle(Value:TwolBrushStyle);

     function  WOL_GetHandle:HBRUSH;

     procedure WOL_SetHandle(Value:HBRUSH);

     function  WOL_GetColor:TCOLORREF;

     procedure WOL_SetColor(Value:TCOLORREF);

   protected

     procedure WOL_MountResource;

     procedure WOL_CreateBrush;

   public

     procedure Assign(Source: TPersistent); Override;

     constructor Create;

     destructor  Destroy; override;



     property Style   :TwolBrushStyle read WOL_GetStyle   Write WOL_SetStyle;

     property Handle  :HBRUSH         read WOL_GetHandle  Write WOL_SetHandle;

     property Color   :TCOLORREF      read WOL_GetColor   Write WOL_SetColor;

     property OnChange:TBrushEvt      read FOnChange      Write FOnChange;

   end;





var BrushManager:TResourceManager;



implementation



//******************************* TWOLBRUSHES ***************************//

//                                                                       //

//                                                                       //

//***********************************************************************//



function  TwolBrushes.WOL_GetStyle:TwolBrushStyle;

begin

  Result:=FStyle;

end;



procedure TwolBrushes.WOL_SetStyle(Value:TwolBrushStyle);

begin

  FStyle:=Value;

  WOL_CreateBrush;

end;



function  TwolBrushes.WOL_GetHandle:HBRUSH;

begin

  Result:=FwolHandle;

end;



procedure TwolBrushes.WOL_SetHandle(Value:HBRUSH);

begin

  if FwolHandle <> 0 then DeleteObject(FwolHandle);

  FwolHandle:=Value;

  if Assigned(FOnChange) then FOnChange(Self);

end;



function  TwolBrushes.WOL_GetColor:TCOLORREF;

begin

  Result:=FColor;

end;



procedure TwolBrushes.WOL_SetColor(Value:TCOLORREF);

begin

  FColor:=Value; {WOL_clColor(Value);}

  WOL_CreateBrush;

end;



procedure TwolBrushes.WOL_MountResource;

begin

  FillChar(FResource, SizeOf(TResourceItem), 0);

  FResource.ID1[1]:=Ord(FStyle);

  FResource.ID1[2]:=FColor;

end;



procedure TwolBrushes.WOL_CreateBrush;

begin

  FLogBrush.lbColor:=FColor;

  FLogBrush.lbHatch:=0;

  {if FBitmap <> nil then begin

      FLogBrush.lbStyle:=BS_PATTERN;

      FLogBrush.lbHatch:=Bitmap.Handle;

   end;}

  case FStyle of

       wol_bsSolid       : FLogBrush.lbStyle:=BS_SOLID;

       wol_bsClear       : FLogBrush.lbStyle:=BS_HOLLOW;

       wol_bsHorizontal  : begin

                             FLogBrush.lbStyle:=BS_HATCHED;

                             FLogBrush.lbHatch:=HS_HORIZONTAL;

                           end;

       wol_bsVertical    : begin

                             FLogBrush.lbStyle:=BS_HATCHED;

                             FLogBrush.lbHatch:=HS_VERTICAL;

                           end;

       wol_bsFDiagonal   : begin

                             FLogBrush.lbStyle:=BS_HATCHED;

                             FLogBrush.lbHatch:=HS_FDIAGONAL;

                           end;

       wol_bsBDiagonal   : begin

                             FLogBrush.lbStyle:=BS_HATCHED;

                             FLogBrush.lbHatch:=HS_BDIAGONAL;

                           end;

       wol_bsCross       : begin

                             FLogBrush.lbStyle:=BS_HATCHED;

                             FLogBrush.lbHatch:=HS_CROSS;

                           end;

       wol_bsDiagCross   : begin

                             FLogBrush.lbStyle:=BS_HATCHED;

                             FLogBrush.lbHatch:=HS_DIAGCROSS;

                           end;

  end;

  

  WOL_MountResource;

  BrushManager.Lock;

  FwolHandle:=BrushManager.GetHandle(FResource);

  if FwolHandle <> 0 then begin

     BrushManager.Unlock;

     if Assigned(FOnChange) then FOnChange(Self);

     Exit;

  end;

  if FwolHandle <> 0 then DeleteObject(FwolHandle);

  Self.FwolHandle:=CreateBrushIndirect(Self.FLogBrush);

  FResource.id1[0]:=FwolHandle;

  BrushManager.InsertTop(FResource);

  BrushManager.Unlock;

  if Assigned(FOnChange) then FOnChange(Self);

end;



procedure TWOLBrushes.Assign(Source: TPersistent);

begin

  if Source is TWOLBrushes then

     With TWOLBrushes(Source) do begin

       FWOLHandle:= Handle;

       FStyle    := Style;

       FLogBrush := TWOLBrushes(Source).FLogBrush;

       FOnChange := OnChange;

       Color     := Color;

       Exit;

     end;

  inherited Assign(Source);

end;



constructor TwolBrushes.Create;

begin

  inherited Create;

  FColor            :=GetSysColor(COLOR_WINDOW);

  FStyle            :=wol_bsSolid;

  FLogBrush.lbColor :=FColor;

  FLogBrush.lbHatch :=0;

  FLogBrush.lbStyle :=BS_Solid;

  FwolHandle        :=GetStockObject(White_BRUSH);

  WOL_MountResource;

  BrushManager.Lock;

  if BrushManager.GetHandle(FResource) = 0 then begin

     FResource.id1[0]:=FwolHandle;

     BrushManager.InsertTop(FResource);

  end;

  BrushManager.Unlock;

end;



destructor  TwolBrushes.Destroy;

begin

  if FwolHandle <> 0 then DeleteObject(FwolHandle);

  inherited Destroy;

end;



initialization

  BrushManager:=TResourceManager.Create;



finalization

  BrushManager.Destroy;



end.



{***********************************************************************}

{                            FILE HISTORY                               }

{***********************************************************************}

{Version .2 (Carlos Alberto de Oliveira, jun 2-6 2001)                  }

{                                                                       }

{  1.) Created wolBrushes component (06/04/2001)                        }

{***********************************************************************}

{Version .5 (Carlos Alberto de Oliveira, jun 11-12 2001)                }

{                                                                       }

{  1.) Changes on color assignment                                      }

{  2.) Added a brush manager to prevent lack memory for resources.      }

{      Changes in TwolBrushes.Create, WOL_CreateBrush and Destroy       }

{      Need improvements on GetResource for velocity                    }

{  4.) Added WOL_MountResource method for improve velocity of access to }

{      resources using TResourceManager                                 }

{  5.) Added BrushManager for improvement of resources access           }

{***********************************************************************}

{Version .6 (Andrew Johnson, Aug 22 2001)                               }

{                                                                       }

{  1.) Added compiler directives for GTK.                               }

{***********************************************************************}



