{

  This program is distributed in the hope that it will be useful,

  but WITHOUT ANY WARRANTY; without even the implied warranty of

  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

  GNU General Public License (http://www.gnu.org/copyleft/gpl.html)

  for more details

  

************************************************************************

  WOL - Window Objects Library - version 0.00.6



  This library is a framework for Windows/GTK programming with the Free

  Pascal Compiler. The main objective is to make porting programs

  between Windows and Linux easier by using the same framework.



  If you find bugs or want colaborate with us to improve, please report

  to authors. It was originally writen by Carlos Alberto de Oliveira

  (carlosnatal@bol.com.br) and is maintened by the following authors:



  Carlos Alberto de Oliveira - carlosnatal@bol.com.br

  Andrew Johnson             - aj_genius@hotmail.com



  Missing you... Visit http://www.olhodetigre.hpg.com.br

                       http://www.freepascal.org

**************************************************************************

FileName : WOLCanva.pp

Current File Version* : .6



Author(s) : Carlos Alberto de Oliveira(carlosnatal@bol.com.br)

            Andrew Johnson(aj_genius@hotmail.com)



Comments  : This File attempts to provide a Canvas component, similar to

            Delphi/Kylix TCanvas, though not entirely compatible, for the

            purpose of custom drawing; and later custom components.



*See end of file for File history

**************************************************************************

}





{$IFDEF FPC}

  {$MODE DELPHI}

{$ENDIF}





unit wolCanva;



interface



{$Include WOLDefs.Inc}

Uses

  Classes,

  {$IFDef GTKWIN}

  CPWindows,GTKWindows,

  {$Else}

  Windows,

  {$EndIF}

  wolBrush,

  wolCaret,

  wolPen,

  wolFont,

  wolDefs,

  wolDBG;



type



   TWOLCanvas = Class(TPersistent)

   private

     FBrush    : TWOLBrushes;

     FCaret    : TWOLCaret;

     FPen      : TWOLPen;

     FFont     : TWOLFont;

     FHandle   : HDC;

     FWOLLock  : {$IFDEF FPC}TCriticalSection{$ELSE}TRTLCriticalSection{$ENDIF};

     FCountLock: Integer;

     procedure OnChangePen(Sender:TObject);

     procedure OnChangeBrush(Sender:TObject);

     procedure OnChangeFont(Sender:TObject);

     Procedure SetHandle(Value:HDC);

     Procedure SelectBrush;

     Procedure SelectPen;

     Procedure SelectFont;

     procedure SetFont(Value: TWOLFont);

     procedure SetBrush(Value: TWOLBrushes);

     procedure SetPen(Value:TWOLPen);

     function  GetPixel(x,y:Integer):TColor;

     procedure SetPixel(x,y:Integer; Value:TColor);

     function  GetClipRect:TRect;

   public

     constructor Create;

     destructor  Destroy; override;



     function  TextExtent(const Text: string): TSize;

     function  TextWidth(const Text: string): Integer;

     function  TextHeight(const Text: string): Integer;

     Procedure FillRect(Rect : TRECT);

     procedure DrawFocusRect(Rect: TRECT);

     procedure MoveTo(X, Y: Integer);

     procedure LineTo(X, Y: Integer);

     Procedure DrawLine(Color:TColor; X1,Y1,X2,Y2:Integer);

     procedure Rectangle(X1, Y1, X2, Y2: Integer);

     procedure Ellipse(X1, Y1, X2, Y2: Integer);

     procedure TextRect(Rect:TRECT; X, Y:Integer; const Text:string; WordWrap, Center:Boolean);

     procedure TextOut(X, Y:Integer; const Text:string);



     procedure Arc(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);

     

     {procedure Draw(X, Y: Integer; Graphic: TWOLGraphic);

     procedure StretchDraw(const Rect: TRect; Graphic: TWOLGraphic);}

     

     procedure Chord(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);

     procedure FloodFill(X, Y: Integer; Color: TColor; FillStyle: TFillStyle);

     procedure FrameRect(const Rect: TRECT);

     procedure Pie(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Longint); {limites}

     procedure Polygon(Points: array of T_POINT);

     procedure Polyline(Points: array of T_POINT);

     procedure Refresh; {Estado padrao}

     procedure RoundRect(X1, Y1, X2, Y2: Integer);

     procedure CopyRect(const Dest, Source:TRect; Canvas:TWOLCanvas; CopyMode:Integer);

     procedure Lock;

     procedure Unlock;

     procedure Update;



     Property Font     :TWOLFont    read FFont   Write SetFont;

     Property Pen      :TWOLPen     read FPen    Write SetPen;

     Property Brush    :TWOLBrushes read FBrush  Write SetBrush;

     Property Handle   :HDC         read FHandle write SetHandle;

     property LockCount:Integer     read FCountLock;

     property ClipRect :TRect       read GetClipRect;

     

     property Pixel[x,y:integer]:TColor read GetPixel write SetPixel;

   end;





var OriginalFont :HFONT;

    OriginalBrush:HBRUSH;

    OriginalPen  :HPEN;



Function GetRect(ALeft,ATop,ARight,ABottom : Integer): TRect;

function ColorToRGB(Color:TColor):Longint;



implementation

uses sysutils;



const WOLModes:array[TWOLPenMode] of Word =

                    (R2_BLACK,       R2_WHITE,      R2_NOP,

                     R2_NOT,         R2_COPYPEN,    R2_NOTCOPYPEN,

                     R2_MERGEPENNOT, R2_MASKPENNOT, R2_MERGENOTPEN,

                     R2_MASKNOTPEN,  R2_MERGEPEN,   R2_NOTMERGEPEN,

                     R2_MASKPEN,     R2_NOTMASKPEN, R2_XORPEN,

                     R2_NOTXORPEN);



Function GetRect(ALeft,ATop,ARight,ABottom : Integer): TRect;

begin

  With Result do begin

    Left  := Aleft;

    Top   := ATop;

    Right := ARight;

    Bottom:= ABottom;

  end;

end;



function ColorToRGB(Color:TColor):Longint;

begin

  if Color < 0 then

     Result := GetSysColor(Color and $000000FF)

  else

     Result := Color;

end;



procedure TWOLCanvas.SetFont(Value: TWOLFont);

begin

  FFont.Assign(Value);

end;



procedure TWOLCanvas.SetBrush(Value: TWOLBrushes);

begin

  FBrush.Assign(Value);

end;



procedure TWOLCanvas.SetPen(Value: TWOLPen);

begin

  FPen.Assign(Value);

end;



function  TWOLCanvas.GetPixel(x,y:Integer):TColor;

begin

  if FHandle <> 0 then

     Result:={$Include WINPROC.INC}.GetPixel(FHandle, x, y)

  else

     Result:=0;

end;



procedure TWOLCanvas.SetPixel(x,y:Integer; Value:TColor);

begin

  if FHandle <> 0 then

     {$Include WINPROC.INC}.SetPixel(FHandle, x, y, Value);

end;



function TWOLCanvas.GetClipRect:TRect;

begin

  {$Include WINPROC.INC}.GetClipBox(FHandle, Result);

end;



procedure TWOLCanvas.OnChangePen(Sender:TObject);

begin

  SelectPen;

end;



procedure TWOLCanvas.OnChangeBrush(Sender:TObject);

begin

  SelectBrush;

end;



procedure TWOLCanvas.OnChangeFont(Sender:TObject);

begin

  SelectFont;

end;



constructor TWOLCanvas.Create;

begin

  inherited Create;

  FBrush         := TWolBrushes.Create;

  FPen           := TWOLPen.Create;

  FFont          := TWolFont.Create;

  FHandle        := 0;

  FPen.Color     := 0;

  FPen.Width     := 1;

  FPen.Style     := wol_psSolid;

  FPen.Mode      := wol_pmCopy;

  FBrush.Color   := $FFFFFF;

  FBrush.Style   := wol_bsSolid;



  OriginalFont   := FFont.Handle;

  OriginalBrush  := FBrush.Handle;

  OriginalPen    := FPen.Handle;



  FCountLock     := 0;



  FPen.OnChange  :=OnChangePen;

  FBrush.OnChange:=OnChangeBrush;

  FFont.OnChange :=OnChangeFont;

  InitializeCriticalSection(FWOLLock);

end;



destructor  TWOLCanvas.Destroy;

begin

  FBrush.Destroy;

  FPen.Destroy;

  FFont.Destroy;

  FHandle:=0;

  DeleteObject(FHandle);

  if FCountLock <> 0 then

     DeleteCriticalSection(FWOLLock);

  inherited Destroy;

end;



Procedure TWOLCanvas.SelectPen;

begin

  If (FHandle <> 0) and (FPen.Handle <> 0) then begin

     SelectObject(FHandle, FPen.Handle);

     SetROP2(FHandle, WOLModes[FPen.Mode]);

  end;

end;



Procedure TWOLCanvas.SelectBrush;

begin

  If (FHandle <> 0) and (FBrush.Handle <> 0) then begin

     SelectObject(FHandle, FBrush.Handle);

     if FBrush.Style = wol_bsSolid then begin

        {$Include WINPROC.INC}.SetBkColor(FHandle, ColorToRGB(FBrush.Color));

        {$Include WINPROC.INC}.SetBkMode(FHandle, OPAQUE);

        end

     else begin

        {$Include WINPROC.INC}.SetBkColor(FHandle, not ColorToRGB(FBrush.Color));

        {$Include WINPROC.INC}.SetBkMode(FHandle, TRANSPARENT);

     end;

  end;

end;



Procedure TWOLCanvas.SelectFont;

begin

  If (FHandle <> 0) and (FFont.Handle <> 0) then begin

     SelectObject(FHandle, FFont.Handle);

     SetTextColor(FHandle, FFont.Color);

  end;

end;



Procedure TWOLCanvas.SetHandle(Value : HDC);

begin

  if FHandle = Value then Exit;

  FHandle:=Value;

  SelectPen;

  SelectBrush;

  SelectFont;

end;



function TWOLCanvas.TextExtent(const Text: string): TSize;

begin

  Result.cX := 0;

  Result.cY := 0;

  if FHandle <> 0 then

     {$Include WINPROC.INC}.GetTextExtentPoint(FHandle, PChar(Text), Length(Text), Result);

end;



function TWOLCanvas.TextWidth(const Text: string): Integer;

begin

  Result:=TextExtent(Text).cX;

end;



function TWOLCanvas.TextHeight(const Text: string): Integer;

begin

  Result:=TextExtent(Text).cY;

end;



Procedure TWOLCanvas.FillRect(Rect : TRect);

begin

  if FHandle <> 0 then

     {$Include WINPROC.INC}.FillRect(FHandle, Rect, FBrush.Handle);

end;



procedure TWOLCanvas.MoveTo(X, Y: Integer);

begin

  if FHandle <> 0 then

     {$Include WINPROC.INC}.MoveToEx(FHandle, X, Y, nil);

end;



procedure TWOLCanvas.LineTo(X, Y: Integer);

begin

  if FHandle <> 0 then

     {$Include WINPROC.INC}.LineTo(FHandle, X, Y);

end;



Procedure TWOLCanvas.Drawline(Color:TColor; X1,Y1,X2,Y2:integer);

var CorAntes:TColor;

begin

  CorAntes:=FPen.Color;

  FPen.Color:=Color;

  MoveTo(X1,Y1);

  LineTo(X2,Y2);

  FPen.Color:=CorAntes;

  SelectPen;

  SelectBrush;

end;



procedure TWOLCanvas.Rectangle(X1, Y1, X2, Y2: Integer);

begin

  if FHandle <> 0 then

     {$Include WINPROC.INC}.Rectangle(FHandle, X1, Y1, X2, Y2);

end;



procedure TWOLCanvas.Ellipse(X1, Y1, X2, Y2: Integer);

begin

  if FHandle <> 0 then

     {$Include WINPROC.INC}.Ellipse(FHandle, X1, Y1, X2, Y2);

end;



procedure TWOLCanvas.DrawFocusRect(Rect: TRect);

begin

  if FHandle <> 0 then

     {$Include WINPROC.INC}.DrawFocusRect(FHandle, Rect);

end;



procedure TWOLCanvas.TextRect(Rect:TRect; X, Y:Integer; const Text:string; WordWrap, Center:Boolean);

const WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);

var theRect     : TRect;

    DrawStyle,

    Flags       : Longint;

    VT : Integer;

begin

  theRect     := Rect;

  theRect.Left:= Rect.Left + X;

  theRect.Top := Rect.Top + Y;

  DrawStyle   := DT_EXPANDTABS or WordWraps[WordWrap];

  If Center then

    DrawStyle := DrawStyle or DT_Center;

  VT := ((theRect.Bottom - TheRect.Top) div 2);

  Flags := DrawStyle or DT_CALCRect;

  DrawText(Handle, PChar(Text), Length(Text), theRect, Flags);

  Rect.Top := VT - ((theRect.Bottom - TheRect.Top) div 2);

  theRect     := Rect;

  theRect.Left:= Rect.Left + X;

  theRect.Top := Rect.Top + Y;

  Flags := DrawStyle;

  DrawText(FHandle, PChar(Text), Length(Text), theRect, Flags);

end;



procedure TWOLCanvas.TextOut(X, Y:Integer; const Text:string);

begin

  if FHandle <> 0 then

     {$Include WINPROC.INC}.TextOut(FHandle, X, Y, PChar(Text), Length(Text));

end;



procedure TWOLCanvas.Arc(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);

begin

  if Self.FHandle <> 0 then

     {$Include WINPROC.INC}.Arc(FHandle, X1, Y1, X2, Y2, X3, Y3, X4, Y4);

end;



{procedure TWOLCanvas.Draw(X, Y: Integer; Graphic: TWOLGraphic);

begin



end;



procedure TWOLCanvas.StretchDraw(const Rect: TRect; Graphic: TWOLGraphic );

begin



end;}



procedure TWOLCanvas.Chord(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);

begin

  if Self.FHandle <> 0 then

     {$Include WINPROC.INC}.Chord(FHandle, X1, Y1, X2, Y2, X3, Y3, X4, Y4);

end;



procedure TWOLCanvas.FloodFill(X, Y:Integer; Color:TColor; FillStyle:TFillStyle);

const fuFillType:array[TFillStyle] of UINT = (FLOODFILLBORDER, FLOODFILLSURFACE);

begin

  if Self.FHandle <> 0 then

     ExtFloodFill(FHandle, X, Y, Color, fuFillType[FillStyle]);

end;



procedure TWOLCanvas.FrameRect(const Rect: TRect);

var lRect:TRect;

begin

  if Self.FHandle <> 0 then begin

     lRect:=Rect;

     {$Include WINPROC.INC}.FrameRect(FHandle, lRect, FBrush.Handle);

  end;

end;



procedure TWOLCanvas.Pie(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Longint); {limites}

begin

  if Self.FHandle <> 0 then

     {$Include WINPROC.INC}.Pie(FHandle, X1, Y1, X2, Y2, X3, Y3, X4, Y4);

end;



procedure TWOLCanvas.Polygon(Points: array of T_POINT);

begin

  if Self.FHandle <> 0 then

     {$Include WINPROC.INC}.Polygon(FHandle, Points[0], High(Points)+1);

end;



procedure TWOLCanvas.Polyline(Points: array of T_POINT);

begin

  if Self.FHandle <> 0 then

     {$Include WINPROC.INC}.Polyline(FHandle, Points[0], high(Points));

end;



procedure TWOLCanvas.Refresh; {Estado default}

begin

  if FHandle <> 0 then begin

     SelectObject(FHandle, OriginalFont);

     SelectObject(FHandle, OriginalBrush);

     SelectObject(FHandle, OriginalPen);

  end;

end;



procedure TWOLCanvas.RoundRect(X1, Y1, X2, Y2: Integer);

var vR:Integer;

begin

  if Self.FHandle <> 0 then begin

     vR:=(X2-X1) div 3 + (Y2-Y1) div 3;

     vR:=vR div 2;

     {$Include WINPROC.INC}.RoundRect(FHandle, X1, Y1, X2, Y2, vR, vR);

  end;

end;



procedure TWOLCanvas.CopyRect(const Dest, Source:TRect; Canvas:TWOLCanvas; CopyMode:Integer);

var DWidth,  SWidth,

    DHeight, SHeight:Integer;

begin

  DWidth  := Dest.Right - Dest.Left;

  SWidth  := Source.Right - Source.Left;

  DHeight := Dest.Bottom - Dest.Top;

  SHeight := Source.Bottom - Source.Top;

  StretchBlt(FHandle,       Dest.Left,   Dest.Top,   DWidth, DHeight,

             Canvas.Handle, Source.Left, Source.Top, SWidth, SHeight,

             CopyMode);

end;



procedure TWOLCanvas.Lock;

begin

  EnterCriticalSection(FWOLLock);

  Inc(FCountLock);

end;



procedure TWOLCanvas.Unlock;

begin

  LeaveCriticalSection(FWOLLock);

  Dec(FCountLock);

end;



procedure TWOLCanvas.Update;

begin

  SelectPen;

  SelectBrush;

  SelectFont;

end;



{$Warnings Off}

end.



{***********************************************************************}

{                            FILE HISTORY                               }

{***********************************************************************}

{Version .1 (Carlos Alberto de Oliveira,? ? 2001)                       }

{                                                                       }

{  1.) Wrote initial TWOLcanvas code, really just a place holder.       }

{***********************************************************************}

{Version .2 (Andrew Johnson, Jul 5-6 2001)                              }

{                                                                       }

{  1.) Wrote TCnvas                                                     }

{  2.) Wrote TextExtent, TextWidth, TextHeight, FillRect, DrawFocusRect,}

{      MoveTo, LineTo, Rectangle, Ellipse, TextRect, TextOut            }

{  3.) Wrote initial code for selecting Pen & Brush                     }

{  4.) Merged into TWOLCanvas, removing most Pen/Brush Code             }

{  5.) Added DrawLine                                                   }

{  6.) Modified TextRect/TextOut Routines to allow centering the text.  }

{  6.) Cleaned up code, removed pointless routines, etc.                }

{***********************************************************************}

{Version .3 (Carlos Alberto de Oliveira,? ? 2001)                       }

{                                                                       }

{  1.) Finished basic Pen/Brush code, added basic Font code             }

{  2.) Updated Drawline routine to call WOL_clColor to determine color  }

{  3.) Rewrote TextOut to use Windows.TextOut                           }

{  4.) Cleaned up several routines                                      }

{***********************************************************************}

{Version .4 (Andrew Johnson, Jul 9-10 2001)                             }

{                                                                       }

{  1.) Fixed Font Select to set the Font Color as well                  }

{  2.) Added CopyRect Routine                                           }

{  3.) Added File Info Header and initial File History                  }

{  4.) Added Write routine for Font Property                            }

{  5.) Added Write routine for Brush Property                           }

{  6.) Added Arc Routine                                                }

{  7.) Added Write routine for Pen Property                             }

{  8.) Added FrameRect Routine                                          }

{  9.) Added Chord Routine                                              }

{  10.) Added Pie Routine                                               }

{  11.) Added FloodFill Routine                                         }

{  12.) Added FrameRect Routine                                         }

{  13.) Added RoundRect Routine                                         }

{***********************************************************************}

{Version .5 (Carlos Alberto de Oliveira, Jul 11-12 2001)                }

{                                                                       }

{  1.) Changed color assignment and TDrawLine                           }

{  2.) Added SetROP2 to SelecPen that handle Pen.Mode                   }

{  3.) Added property Pixel and Get/SetPixel routines                   }

{  4.) Added Lock and Unlock methods                                    }

{  5.) Fixed Lock and Unlock methods                                    }

{  6.) Added Update method                                              }

{  7.) Removed Parent from Create constructor                           }

{  8.) Added LockCount property                                         }

{  9.) Added ColorToRGB                                                 }

{***********************************************************************}

{Version .6 (Andrew Johnson, Aug 22,27 2001)                            }

{                                                                       }

{ Aug 22                                                                }

{  1.) Added compiler directives for GTK.                               }

{                                                                       }

{ Aug 27                                                                }

{  1.) Fixed textrect center bug, where text always centers on canvas,  }

{      instead of in the rect.                                          }

{***********************************************************************}



