{

  This program is distributed in the hope that it will be useful,

  but WITHOUT ANY WARRANTY; without even the implied warranty of

  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

  GNU General Public License (http://www.gnu.org/copyleft/gpl.html)

  for more details

  

************************************************************************

  WOL - Window Objects Library - version 0.00.6



  This library is a framework for Windows/GTK programming with the Free

  Pascal Compiler. The main objective is to make porting programs

  between Windows and Linux easier by using the same framework.



  If you find bugs or want colaborate with us to improve, please report

  to authors. It was originally writen by Carlos Alberto de Oliveira

  (carlosnatal@bol.com.br) and is maintened by the following authors:



  Carlos Alberto de Oliveira - carlosnatal@bol.com.br

  Andrew Johnson             - aj_genius@hotmail.com



  Missing you... Visit http://www.olhodetigre.hpg.com.br

                       http://www.freepascal.org

**************************************************************************

FileName : WOLCaret.pp

Current File Version* : .6



Author(s) : Carlos Alberto de Oliveira(carlosnatal@bol.com.br)



Comments  : This File attempts to provide a Caret component.



*See end of file for File history

**************************************************************************

}



{$IFDEF FPC}

  {$MODE DELPHI}

{$ENDIF}



unit wolCaret;



interface



{$Include WOLDefs.Inc}

Uses

  Classes,

  {$IFDef GTKWIN}

  CPWindows,GTKWindows;

  {$Else}

  Windows;

  {$EndIF}



type

    TWOLCaret = Class(TObject)

    private

      FWinHandle : HWND;

      FWidth     : Integer;

      FHeight    : Integer;

      FLeft      : Integer;

      FTop       : Integer;

      FBlink     : Integer;

      //FBitMap    : HBITMAP;

    protected

      procedure WOLCreateCaret;

      procedure WOLPosCaret(var X,Y:Integer; Setting:Boolean);



      function  WOL_GetParent:HWND;

      procedure WOL_SetParent(Value:HWND);

      function  WOL_GetWidht:Integer;

      procedure WOL_SetWidth(Value:Integer);

      function  WOL_GetHeight:Integer;

      procedure WOL_SetHeight(Value:Integer);

      function  WOL_GetLeft:Integer;

      procedure WOL_SetLeft(Value:Integer);

      function  WOL_GetTop:Integer;

      procedure WOL_SetTop(Value:Integer);

      function  WOL_GetBlink:Integer;

      procedure WOL_SetBlink(Value:Integer);

    public

      constructor Create(Parent:HWND);

      destructor  Destroy; override;



      procedure Show;

      procedure Hide;



      property Parent:HWND    read WOL_GetParent Write WOL_SetParent;

      property Width:Integer  read WOL_GetWidht  Write WOL_SetWidth;

      property Height:Integer read WOL_GetHeight Write WOL_SetHeight;

      property X:Integer      read WOL_GetLeft   Write WOL_SetLeft;

      property Y:Integer      read WOL_GetTop    Write WOL_SetTop;

      property Blink:Integer  read WOL_GetBlink  Write WOL_SetBlink;

    end;



implementation



procedure TWOLCaret.WOLCreateCaret;

var hCaret:HBITMAP;

begin

  if Self.FWinHandle = 0 then Exit;

  hCaret:=HBITMAP(1);

  CreateCaret(Self.FwinHandle, hCaret, 10, 40);

  SetCaretPos(Self.FLeft, Self.FTop);

  ShowCaret(Self.FWinHandle);

end;



procedure TWOLCaret.WOLPosCaret(var X,Y:Integer; Setting:Boolean);

var pt:TPoint;

begin

  if Self.FWinHandle = 0 then Exit;

  if Setting then

     SetCaretPos(X, Y)

  else begin

     {$IFDEF FPC}

     GetCaretPos(@pt);

     {$ELSE}

     GetCaretPos(pt);

     {$ENDIF}

     X:=pt.x;

     Y:=pt.y;

  end;

end;



function  TWOLCaret.WOL_GetParent:HWND;

begin

  Result:=Self.FWinHandle

end;



procedure TWOLCaret.WOL_SetParent(Value:HWND);

begin

  Self.FWinHandle:=Value;

  WOLCreateCaret;

end;



function  TWOLCaret.WOL_GetWidht:Integer;

begin

  Result:=Self.FWidth;

end;



procedure TWOLCaret.WOL_SetWidth(Value:Integer);

begin

  Self.FWidth:=Value;

  WOLCreateCaret;

end;



function  TWOLCaret.WOL_GetHeight:Integer;

begin

  Result:=Self.FHeight;

end;



procedure TWOLCaret.WOL_SetHeight(Value:Integer);

begin

  Self.Height:=Value;

  WOLCreateCaret;

end;



function  TWOLCaret.WOL_GetLeft:Integer;

var x,y:Integer;

begin

  if Self.FWinHandle = 0 then

     Result:=Self.FLeft

  else begin

     WOLPosCaret(x, y, False);

     Result:=x;

     Self.FLeft:=x;

  end;

end;



procedure TWOLCaret.WOL_SetLeft(Value:Integer);

var x,y:integer;

begin

  if Self.FWinHandle = 0 then

     Self.FLeft:=Value

  else begin

     x:=Self.FLeft;

     y:=Self.FTop;

     WOLPosCaret(x, y, True);

  end;

end;



function  TWOLCaret.WOL_GetTop:Integer;

var x,y:integer;

begin

  if Self.FWinHandle = 0 then

     Result:=Self.FTop

  else begin

     WOLPosCaret(x, y, False);

     Result:=y;

     Self.FTop:=y;

  end;

end;



procedure TWOLCaret.WOL_SetTop(Value:Integer);

var x,y:integer;

begin

  if Self.FWinHandle = 0 then

     Self.FTop:=Value

  else begin

     x:=Self.FLeft;

     y:=Self.FTop;

     WOLPosCaret(x, y, True);

  end;

end;



function  TWOLCaret.WOL_GetBlink:Integer;

begin

  Result:=Self.FBlink;

  if Self.FWinHandle = 0 then

     Result:=GetCaretBlinkTime;

end;



procedure TWOLCaret.WOL_SetBlink(Value:Integer);

begin

  Self.FBlink:=Value;

  if Self.FWinHandle = 0 then

     SetCaretBlinkTime(Self.FBlink);

end;



procedure TWOLCaret.Show;

begin

  if Self.FWinHandle = 0 then Exit;

  ShowCaret(Self.FWinHandle);

end;



procedure TWOLCaret.Hide;

begin

  if Self.FWinHandle = 0 then Exit;

  HideCaret(Self.FWinHandle);

end;



constructor TWOLCaret.Create(Parent:HWND);

begin

  inherited Create;

  FWinHandle:=Parent;

  FWidth    :=0;

  FHeight   :=0;

  FLeft     :=0;

  FTop      :=0;

  FBlink    :=GetCaretBlinkTime;

end;



destructor  TWOLCaret.Destroy;

begin

  DestroyCaret;

  inherited Destroy;

end;





end.



{***********************************************************************}

{                            FILE HISTORY                               }

{***********************************************************************}

{                                                                       }

{Version .2 (Carlos Alberto de Oliveira, jun 2-6 2001)                  }

{  1.) Created wolCaret unit                                            }

{***********************************************************************}

{Version .6 (Andrew Johnson, Aug 22 2001)                               }

{                                                                       }

{  1.) Added compiler directives for GTK.                               }

{***********************************************************************}



