{

  This program is distributed in the hope that it will be useful,

  but WITHOUT ANY WARRANTY; without even the implied warranty of

  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

  GNU General Public License (http://www.gnu.org/copyleft/gpl.html)

  for more details

  

************************************************************************

  WOL - Window Objects Library - version 0.00.3



  This library is a framework for Windows/GTK programming with the Free

  Pascal Compiler. The main objective is to make porting programs

  between Windows and Linux easier by using the same framework.



  If you find bugs or want colaborate with us to improve, please report

  to authors. It was originally writen by Carlos Alberto de Oliveira

  (carlosnatal@bol.com.br) and is maintened by the following authors:



  Carlos Alberto de Oliveira - carlosnatal@bol.com.br

  Andrew Johnson             - aj_genius@hotmail.com



  Missing you... Visit http://www.olhodetigre.hpg.com.br

                       http://www.freepascal.org

**************************************************************************

FileName : WOLCtrls.pp

Current File Version* : .1



Author(s) : Andrew Johnson(AJ_Genius@Hotmail.com)



Comments  : This File attempts to provide basic TControl support

            similar to what is found in Delphi/Kylix.



*See end of file for File history

**************************************************************************

}

unit WOLCtrls;



{$IfDef FPC}

  {$Mode Delphi}

  {$Warnings Off}

  {$Hints Off}  

{$EndIf}



interface



{$Include WOLDefs.Inc}

uses

  Classes, 

  {$IFDef GTKWIN}

  CPWindows,GTKWindows,

  {$Else}

  Windows,

  {$EndIF}

  {$IFNDEF FPC}Messages,{$ENDIF}

  wolFont,  wolCanva, wolBrush,

  SysUtils, wolDefs, wolDBG,WOLBeta;



Type

  TWOLAlign = (waLNone, walTop, walBottom, walLeft, walRight, walClient);



  TControl = Class(TWOLBetaObject)

  Private

    FAlign: TWOLAlign;

    FParent : TWOLBetaObject;

    FLeft,

    FTop,

    FWidth,

    FHeight : Integer;

    FVisible : Boolean;

    procedure SetAlign(Value: TWOLAlign);

    Procedure SetLeft(Value : Integer);

    Procedure SetTop(Value : Integer);

    Procedure SetWidth(Value : Integer);

    Procedure SetHeight(Value : Integer);

    Procedure SetVisible(Value : Boolean);

    Procedure SetParent(Value : TWOLBetaObject);

  Protected

    procedure RequestAlign;

    procedure WndProc(var Message: TMessage); Override;

  Public

    Constructor Create(AOwner : TComponent); virtual;

    Procedure Invalidate;

    Procedure Repaint;

    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer);

    

    property Align: TWOLAlign read FAlign write SetAlign default walNone;

    Property Parent : TWOLBetaObject read FParent write SetParent;

    property ShowHint;

    Property Visible : Boolean read FVisible write SetVisible;

    

  Published

    property Left: Integer read FLeft write SetLeft;

    property Top: Integer read FTop write SetTop;

    property Width: Integer read FWidth write SetWidth;

    property Height: Integer read FHeight write SetHeight;

    property Cursor;

    property Hint;

  end;



  TGraphicControl = class(TControl)

  private

    FCanvas: TWOLCanvas;

  protected

    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;

    procedure Paint; virtual;

    property Canvas: TWOLCanvas read FCanvas;

  public

    constructor Create(AOwner: TComponent); override;

    destructor Destroy; override;

  end;



implementation



procedure TControl.WndProc(var Message: TMessage);

var

  MP : TWMPaint;

begin

  Message.Result := 0;

  If Message.Msg = WM_Paint then begin

     MP.Msg    := WM_Paint;

     MP.DC     := Message.WParam;

     MP.Result := Message.Result;

     Dispatch(MP);

     end

  else

     inherited WndProc(Message);

end;



procedure TControl.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);

begin

  FLeft   := ALeft;

  FTop    := ATop;

  FWidth  := AWidth;

  FHeight := AHeight;

  Invalidate;

  Repaint;

end;



procedure TControl.RequestAlign;

begin

  case fAlign of

    walClient: begin

                 If Parent <> nil then

                    SetBounds(0, 0, Parent.ClientWidth, Parent.ClientHeight);

               end;

    walTop   : begin

                 If Parent <> nil then

                    SetBounds(0, 0, Parent.ClientWidth, Height);

               end;

    walBottom: begin

                 If Parent <> nil then

                    SetBounds(0, Parent.ClientHeight - Height, Parent.ClientWidth, Height);

               end;

    walLeft  : begin

                 If Parent <> nil then

                    SetBounds(0,  0, Width, Parent.ClientHeight);

               end;

    walRight : begin

                 If Parent <> nil then

                    SetBounds(Parent.ClientWidth - Width,0, Width, Parent.ClientHeight);

               end;

  end;

end;



procedure TControl.SetAlign(Value: TWOLAlign);

begin

  fAlign := Value;

  RequestAlign;

end;



Procedure TControl.SetLeft(Value : Integer);

begin

  SetBounds(Value, FTop, FWidth, FHeight);

end;



Procedure TControl.SetTop(Value : Integer);

begin

  SetBounds(FLeft, Value, FWidth, FHeight);

end;



Procedure TControl.SetWidth(Value : Integer);

begin

  SetBounds(FLeft, FTop, Value, FHeight);

end;



Procedure TControl.SetHeight(Value : Integer);

begin

  SetBounds(FLeft, FTop, FWidth, Value);

end;



Procedure TControl.SetVisible(Value : Boolean);

begin

  If FVisible = Value then

     Exit;

  Invalidate;

  FVisible := Value;

  If Value then

     Repaint;

end;



Procedure TControl.SetParent(Value : TWOLBetaObject);

begin

  Inherited SetParent(Value);

  Invalidate;

  FParent := Value;

  Invalidate;

  Repaint;

end;



Constructor TControl.Create(AOwner : TComponent);

begin

  WindowProc := WndPRoc;

  FLeft      := 0;

  FTop       := 0;

  FWidth     := 10;

  FHeight    := 10;

  FVisible   := True;

  FParent    := nil;

end;



procedure TControl.Invalidate;

var ARect : TRect;

begin

  If (FParent = nil) then

     Exit;

  If FParent.Handle = 0 then

     Exit;

  ARect.Left   := Left;

  ARect.Right  := Left + Width;

  ARect.Top    := Top;

  ARect.Bottom := Top + Height;

//  InvalidateRect(FParent.Handle, {$IfNDef fpc}@{$EndIF}ARect, True);

  UpdateWindow(FParent.Handle);

end;



procedure TControl.Repaint;

var Index  : integer;

    aHandle: HDC;

    P      : TPoint;

begin

  If FParent <> nil then

  if (Visible) and (FParent.Handle <> 0) then begin

     aHandle := GetDC(FParent.Handle);

     try

       IntersectClipRect(aHandle, Left, Top, Left + Width, Top + Height);

    (*   Index := SaveDC(aHandle);

       GetWindowOrgEx(aHandle, {$IfDef fpc}@{$EndIF}P);

       SetWindowOrgEx(aHandle, P.X - Left, P.Y - Top, nil);*)

       IntersectClipRect(aHandle, 1, 1, Width - 2, Height - 1);

       Perform(WM_PAINT, aHandle,0);

       (*RestoreDC(aHandle, Index);*)

     finally

       ReleaseDC(FParent.Handle, aHandle);

     end;

  end;

end;





constructor TGraphicControl.Create(AOwner: TComponent);

begin

  inherited Create(AOwner);

  FCanvas := TWOLCanvas.Create;

end;



destructor TGraphicControl.Destroy;

begin

  FCanvas.Free;

  inherited Destroy;

end;



procedure TGraphicControl.WMPaint(var Message: TWMPaint);

begin

  with Message do

    if DC <> 0 then

       With Canvas do begin

         Try

           Lock;

           Handle := DC;

           Self.Paint;

         Finally

            Handle := 0;

            Unlock;

            Result := 1;

         end;

       end;

end;



procedure TGraphicControl.Paint;

begin

end;



end.

{***********************************************************************}

{                            FILE HISTORY                               }

{***********************************************************************}

{Version .1 (Andrew Johnson, Aug 6-7 2001)                              }

{                                                                       }

{ Aug 6                                                                 }

{  1.) Created TControl based on TCtrl code                             }

{  2.) Updated TControl.Repaint Routine to PosMessage                   }

{  3.) Added WndProc to pass on TWMPaint Properly                       }

{  4.) Added TGraphicControl                                            }

{ Aug 7                                                                 }

{  1.) Fixed TGraphicControl.WMPaint to Lock/Unlock canvas              }

{  2.) Added TControl.Align property and partial support                }

{***********************************************************************}



