{

  This program is distributed in the hope that it will be useful,

  but WITHOUT ANY WARRANTY; without even the implied warranty of

  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

  GNU General Public License (http://www.gnu.org/copyleft/gpl.html)

  for more details

  

************************************************************************

  WOL - Window Objects Library - version 0.00.6



  This library is a framework for Windows/GTK programming with the Free

  Pascal Compiler. The main objective is to make porting programs

  between Windows and Linux easier by using the same framework.



  If you find bugs or want colaborate with us to improve, please report

  to authors. It was originally writen by Carlos Alberto de Oliveira

  (carlosnatal@bol.com.br) and is maintened by the following authors:



  Carlos Alberto de Oliveira - carlosnatal@bol.com.br



  Missing you... Visit http://www.olhodetigre.hpg.com.br

                       http://www.freepascal.org

**************************************************************************

FileName : WOLFont.pp

Current File Version* : .6



Author(s) : Carlos Alberto de Oliveira(carlosnatal@bol.com.br)

            Andrew Johnson(aj_genius@hotmail.com)



Comments  : This File attempts to provide a Font component, similar to

            Delphi/Kylix TCanvas, though not entirely compatible.



*See end of file for File history

**************************************************************************

}



{$IFDEF FPC}

  {$MODE DELPHI}

{$ENDIF}



unit wolFont;



interface



{$Include WOLDefs.Inc}

Uses

  Classes,

  SysUtils,

  {$IFDef GTKWIN}

  CPWindows,GTKWindows,

  {$Else}

  Windows,

  {$EndIF}

  {$IFNDEF FPC}Messages,{$ENDIF}

  wolDefs,

  wolRMan,

  wolDBG;



type

   TFontEvt = TNotifyEventA;



   TCharSet = 0..255;



   TWOL_FontStyle  = (wol_fsItalic,

                      wol_fsUnderline,

                      wol_fsStrikeOut,

                      wol_fsBold);

   TWOL_SetFStyle  = set of TWOL_FontStyle;



   TWOL_Pitch      = (wol_fpDefault,

                      wol_fpVariable,

                      wol_fpFixed);

   

   TWOL_FontWeight = (wol_hbDefault,

                      wol_hbThin,

                      wol_hbExtraLight,

                      wol_hbUltraLight,

                      wol_hbLight,

                      wol_hbNormal,

                      wol_hbRegular,

                      wol_hbMedium,

                      wol_hbSemiBold,

                      wol_hbDemiBold,

                      wol_hbBold,

                      wol_hbExtraBold,

                      wol_hbUltraBold,

                      wol_hbHeavy,

                      wol_hbBlack);



   TwolFont = class(TPersistent)

   private

     FResource  : TResourceItem;

     FWinFont   : HFONT;

     FLogFont   : TLOGFONT;

     FFontColor : TCOLORREF;

     FFontSize  : Integer;

     FPPI       : Integer;



     FFontStyle : TWOL_SetFStyle;

     FFontWeight: TWOL_FontWeight;



     FOnChange  : TFontEvt;



     procedure WOL_SetHandle(Value:HFONT);

     function  WOL_GetHandle:HFONT;

     function  WOL_GetStyle:TWOL_SetFStyle;

     procedure WOL_SetStyle(Value:TWOL_SetFStyle);

     function  WOL_GetBold:TWOL_FontWeight;

     procedure WOL_SetBold(Value:TWOL_FontWeight);

     function  WOL_GetFName:string;

     procedure WOL_SetFName(Value:string);

     function  WOL_GetAngle:Integer;

     procedure WOL_SetAngle(Value:Integer);

     function  WOL_GetColor:TCOLORREF;

     procedure WOL_SetColor(Value:TCOLORREF);

     function  WOL_GetSize:Integer;

     procedure WOL_SetSize(Value:Integer);

     function  WOL_GetHeight:Integer;

     procedure WOL_SetHeight(Value:Integer);

     function  WOL_GetChSet:TCharSet;

     procedure WOL_SetChSet(Value:TCharSet);

     function  WOL_GetPitch:TWOL_Pitch;

     procedure WOL_SetPitch(Value:TWOL_Pitch);

     function  WOL_GetPPI:Integer;

     procedure WOL_SetPPI(Value:Integer);

   protected

     procedure WOL_MountResource;

     procedure CreateLogFont;

   public

     constructor Create;

     destructor  Destroy; override;



     procedure Assign(Source: TPersistent); Override;



     property Handle  :HFONT           read WOL_GetHandle write WOL_SetHandle;

     property Style   :TWOL_SetFStyle  read WOL_GetStyle  write WOL_SetStyle;

     property HowBold :TWOL_FontWeight read WOL_GetBold   write WOL_SetBold;

     property FontName:String          read WOL_GetFName  write WOL_SetFName;

     property Angle   :Integer         read WOL_GetAngle  write WOL_SetAngle;

     property Color   :TCOLORREF       read WOL_GetColor  write WOL_SetColor;

     property Size    :Integer         read WOL_GetSize   write WOL_SetSize;

     property Height  :Integer         read WOL_GetHeight write WOL_SetHeight;

     property CharSet :TCharSet        read WOL_GetChSet  write WOL_SetChSet;

     property Pitch   :TWOL_Pitch      read WOL_GetPitch  write WOL_SetPitch;

     property PPI     :Integer         read WOL_GetPPI    write WOL_SetPPI;

     property OnChange:TFontEvt        read FOnChange     Write FOnChange;

   end;





var PixeisPorPolegada:Integer;

    _DeviceContext   :HDC;

    FontManager      :TResourceManager;



implementation



const

    FW_DONTCARE   =   0;

    FW_THIN       = 100;

    FW_EXTRALIGHT = 200;

    FW_ULTRALIGHT = 250;

    FW_LIGHT      = 300;

    FW_NORMAL     = 400;

    FW_REGULAR    = 450;

    FW_MEDIUM     = 500;

    FW_SEMIBOLD   = 600;

    FW_DEMIBOLD   = 650;

    FW_BOLD       = 700;

    FW_EXTRABOLD  = 800;

    FW_ULTRABOLD  = 850;

    FW_HEAVY      = 900;

    FW_BLACK      = 950;



procedure TwolFont.WOL_SetHandle(Value:HFONT);

begin

  if FWinFont <> 0 then DeleteObject(FWinFont);

  Self.FWinFont:=Value;

  if Assigned(FOnChange) then FOnChange(Self);

end;



function TwolFont.WOL_GetHandle:HFONT;

begin

  Result:=FWinFont;

end;



function  TwolFont.WOL_GetStyle:TWOL_SetFStyle;

begin

  FFontStyle:=[];

  with FLogFont do begin

    if lfWeight > FW_REGULAR then Include(FFontStyle, wol_fsBold);

    if Byte(lfItalic)   <> 0 then Include(FFontStyle, wol_fsItalic);

    if Byte(lfUnderline)<> 0 then Include(FFontStyle, wol_fsUnderline);

    if Byte(lfStrikeOut)<> 0 then Include(FFontStyle, wol_fsStrikeOut);

  end;

  Result:=FFontStyle;

end;



procedure TwolFont.WOL_SetStyle(Value:TWOL_SetFStyle);

begin

  FFontStyle:=Value;

  FLogFont.lfItalic   :=Byte(FALSE);

  FLogFont.lfUnderline:=Byte(FALSE);

  FLogFont.lfStrikeOut:=Byte(FALSE);

  FLogFont.lfWeight   :=FW_NORMAL;

  if wol_fsItalic    in FFontStyle then FLogFont.lfItalic   :=Byte(TRUE);

  if wol_fsUnderline in FFontStyle then FLogFont.lfUnderline:=Byte(TRUE);

  if wol_fsStrikeOut in FFontStyle then FLogFont.lfStrikeOut:=Byte(TRUE);

  if wol_fsBold      in FFontStyle then FLogFont.lfWeight   :=FW_BOLD;

  CreateLogFont;

end;



function  TwolFont.WOL_GetBold:TWOL_FontWeight;

begin

  Result:=FFontWeight;

end;



procedure TwolFont.WOL_SetBold(Value:TWOL_FontWeight);

begin

  FFontWeight:=Value;

  case FFontWeight of

       wol_hbDefault        : FLogFont.lfWeight:=FW_DONTCARE;

       wol_hbThin           : FLogFont.lfWeight:=FW_THIN;

       wol_hbExtraLight     : FLogFont.lfWeight:=FW_EXTRALIGHT;

       wol_hbUltraLight     : FLogFont.lfWeight:=FW_ULTRALIGHT;

       wol_hbLight          : FLogFont.lfWeight:=FW_LIGHT;

       wol_hbNormal         : FLogFont.lfWeight:=FW_NORMAL;

       wol_hbRegular        : FLogFont.lfWeight:=FW_REGULAR;

       wol_hbMedium         : FLogFont.lfWeight:=FW_MEDIUM;

       wol_hbSemiBold       : FLogFont.lfWeight:=FW_SEMIBOLD;

       wol_hbDemiBold       : FLogFont.lfWeight:=FW_DEMIBOLD;

       wol_hbBold           : FLogFont.lfWeight:=FW_BOLD;

       wol_hbExtraBold      : FLogFont.lfWeight:=FW_EXTRABOLD;

       wol_hbUltraBold      : FLogFont.lfWeight:=FW_ULTRABOLD;

       wol_hbHeavy          : FLogFont.lfWeight:=FW_HEAVY;

       wol_hbBlack          : FLogFont.lfWeight:=FW_BLACK;

  end;

  CreateLogFont;

end;



function  TwolFont.WOL_GetFName:string;

begin

  {$IFDEF FPC}

  Result:=strPas(FLogFont.lfFaceName);

  {$ELSE}

  Result:=FLogFont.lfFaceName;

  {$ENDIF}

end;



procedure TwolFont.WOL_SetFName(Value:string);

begin

  FillChar(FLogFont.lfFaceName, 32, 0);

  {$IFDEF FPC}

  FLogFont.lfFaceName:=PChar(Value);

  {$ELSE}

  strPCopy(FLogFont.lfFaceName, Value);

  {$ENDIF}

  CreateLogFont;

end;



function  TwolFont.WOL_GetAngle:Integer;

begin

  Result:=FLogFont.lfEscapement;

end;



procedure TwolFont.WOL_SetAngle(Value:Integer);

begin

  FLogFont.lfEscapement :=Value;

  FLogFont.lfOrientation:=Value;

  CreateLogFont;

end;



function  TwolFont.WOL_GetColor:TCOLORREF;

begin

  Result:=FFontColor;

end;



procedure TwolFont.WOL_SetColor(Value:TCOLORREF);

begin

  if Value = FFontColor then Exit;

  FFontColor:=Value;

  CreateLogFont;

end;



function  TwolFont.WOL_GetSize:Integer;

begin

  Result:=-MulDiv(FLogFont.lfHeight, 72, FPPI);

end;



procedure TwolFont.WOL_SetSize(Value:Integer);

begin

  FLogFont.lfHeight:=-MulDiv(Value, FPPI, 72);

  FFontSize:=Value;

  CreateLogFont;

end;



function  TwolFont.WOL_GetHeight:Integer;

begin

  Result:=FLogFont.lfHeight;

end;



procedure TwolFont.WOL_SetHeight(Value:Integer);

begin

  FLogFont.lfHeight:=Value;

  CreateLogFont;

end;



function  TwolFont.WOL_GetChSet:TCharSet;

begin

  Result:=TCharSet(FLogFont.lfCharSet);

end;



procedure TwolFont.WOL_SetChSet(Value:TCharSet);

begin

  FLogFont.lfCharSet:=Byte(Value);

  CreateLogFont;

end;



function  TwolFont.WOL_GetPitch:TWOL_Pitch;

begin

  if FLogFont.lfPitchAndFamily = VARIABLE_PITCH then

     Result:=wol_fpVariable

  else if FLogFont.lfPitchAndFamily = FIXED_PITCH then

     Result:=wol_fpFixed

  else

     Result:=wol_fpDefault;

end;



procedure TwolFont.WOL_SetPitch(Value:TWOL_Pitch);

begin

  case Value of

       wol_fpVariable: FLogFont.lfPitchAndFamily:=VARIABLE_PITCH;

       wol_fpFixed   : FLogFont.lfPitchAndFamily:=FIXED_PITCH;

       wol_fpDefault : FLogFont.lfPitchAndFamily:=DEFAULT_PITCH;

  end;

end;



function  TwolFont.WOL_GetPPI:Integer;

begin

  Result:=FPPI;

end;



procedure TwolFont.WOL_SetPPI(Value:Integer);

begin

  FPPI:=Value;

end;



procedure TwolFont.WOL_MountResource;

begin

  FResource.ID1[0]:=0;

  FResource.ID1[1]:=FLogFont.lfHeight;

  FResource.ID1[2]:=FLogFont.lfPitchAndFamily;

  FResource.ID1[3]:=FLogFont.lfItalic    + FLogFont.lfUnderline +

                    FLogFont.lfStrikeOut + FLogFont.lfWeight    +

                    FLogFont.lfCharSet   + FLogFont.lfEscapement;

  FResource.ID1[4]:=FFontColor;

  FResource.ID2   :=strPas(FLogFont.lfFaceName);  

end;



procedure TwolFont.CreateLogFont;

begin

  WOL_MountResource;

  FontManager.Lock;

  FWinFont:=FontManager.GetHandle(FResource);

  if FWinFont <> 0 then begin

     FontManager.Unlock;

     if Assigned(FOnChange) then FOnChange(Self);

     Exit;

  end;

  FLogFont.lfWidth        :=0;

  FLogFont.lfQuality      :=DEFAULT_QUALITY;

  FLogFont.lfOutPrecision :=OUT_DEFAULT_PRECIS;

  FLogFont.lfClipPrecision:=CLIP_DEFAULT_PRECIS;

  if FWinFont <> 0 then DeleteObject(FWinFont);

  FWinFont:=CreateFontIndirect({$IFDEF FPC}@{$ENDIF}FLogFont);

  FResource.ID1[0]:=FWinFont;

  FontManager.InsertTop(FResource);

  FontManager.Unlock;

  if Assigned(FOnChange) then FOnChange(Self);

end;



constructor TwolFont.Create;

begin

  inherited Create;

  FillChar(FLogFont, SizeOf(TLOGFONT), 0);

  GetObject(GetStockObject(SYSTEM_FONT), SizeOf(TLOGFONT), @FLogFont);

  FWinFont   :=GetStockObject(SYSTEM_FONT);

  FFontColor :=0;

  FFontStyle :=[];

  FFontWeight:=wol_hbNormal;

  FPPI       :=PixeisPorPolegada;

  

  WOL_MountResource;

  FontManager.Lock;

  if FontManager.GetHandle(FResource) = 0 then begin

     FResource.ID1[0]:=FWinFont;

     FontManager.InsertTop(FResource);

  end;

  FontManager.Unlock;

end;



destructor TwolFont.Destroy;

begin

  if FWinFont <> 0 then DeleteObject(FWinFont);

  inherited Destroy;

end;



procedure TwolFont.Assign(Source:TPersistent);

begin

  if Source is TWOLFont then

    With TWOLFont(Source) do begin

      Self.FWinFont   := Handle;

      Self.FFontStyle := Style;

      Self.FFontWeight:= HowBold;

      Self.FLogFont   := TWOLFont(Source).FLogFont;

      FOnChange       := OnChange;

      Self.Color      := Color;

      Exit;

    end;

  inherited Assign(Source);

end;



initialization

  _DeviceContext:=GetDC(0);

  PixeisPorPolegada:=GetDeviceCaps(_DeviceContext, LOGPIXELSY);

  ReleaseDC(0, _DeviceContext);

  FontManager:=TResourceManager.Create;



finalization

  FontManager.Destroy;



end.



{***********************************************************************}

{                            FILE HISTORY                               }

{***********************************************************************}

{Version .2 (Carlos Alberto de Oliveira, jun 2-6 2001)                  }

{                                                                       }

{  1.) Created wolFont and unit wolColor (06/02/2001)                   }

{***********************************************************************}

{Version .5 (Carlos Alberto de Oliveira, jul 11-14 2001)                }

{                                                                       }

{  1.) Changes on color assignment                                      }

{  2.) Added Height and CharSet properties and methods to manipulate it }

{  3.) Added PPI property (PixelsPerInch) and methods                   }

{  4.) Added Pitch property and methods                                 }

{  5.) Added Height property and methods                                }

{  6.) Added CharSet property and methods                               }

{  7.) Added WOL_MountResource to improve velocity in resources access  }

{  8.) Added FontManager for improvement of resources access            }

{  9.) Integrated Font with Printer                                     }

{***********************************************************************}

{Version .6 (Andrew Johnson, Aug 22 2001)                               }

{                                                                       }

{  1.) Added compiler directives for GTK.                               }

{***********************************************************************}



