{

  This program is distributed in the hope that it will be useful,

  but WITHOUT ANY WARRANTY; without even the implied warranty of

  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

  GNU General Public License (http://www.gnu.org/copyleft/gpl.html)

  for more details



************************************************************************

  WOL - Window Objects Library - version 0.00.6



  This library is a framework for Windows/GTK programming with the Free

  Pascal Compiler. The main objective is to make porting programs

  between Windows and Linux easier by using the same framework.



  If you find bugs or want colaborate with us to improve, please report

  to authors. It was originally writen by Carlos Alberto de Oliveira

  (carlosnatal@bol.com.br) and is maintened by the following authors:



  Carlos Alberto de Oliveira - carlosnatal@bol.com.br

  Andrew Johnson - AJ_Genius@Hotmail.com

  Missing you... Visit http://www.olhodetigre.hpg.com.br

                       http://www.freepascal.org

**************************************************************************

FileName : WOLPen.pp

Current File Version* : .6



Author(s) : Carlos Alberto de Oliveira(carlosnatal@bol.com.br)



Comments  : This File attempts to provide a Pen component, similar to

            Delphi/Kylix TCanvas, though not entirely compatible.



*See end of file for File history

**************************************************************************

}



{$IFDEF FPC}

  {$MODE DELPHI}

{$ENDIF}

unit wolPen;



interface



{$Include WOLDefs.Inc}

Uses

  Classes,

  {$IFDef GTKWIN}

  CPWindows,GTKWindows,

  {$Else}

  Windows,

  {$EndIF}

  wolDefs,

  wolRMan,

  wolDBG;



type

    TWOLPenStyle = ( wol_psSolid,

                     wol_psDash,

                     wol_psDot,

                     wol_psDashDot,

                     wol_psDashDotDot,

                     wol_psNull,

                     wol_psInsideFrame );



    TWOLPenMode  = ( wol_pmBlack,

                     wol_pmWhite,

                     wol_pmNop,

                     wol_pmNot,

                     wol_pmCopy,

                     wol_pmNotCopy,

                     wol_pmMergePenNot,

                     wol_pmMaskPenNot,

                     wol_pmMergeNotPen,

                     wol_pmMaskNotPen,

                     wol_pmMerge,

                     wol_pmNotMerge,

                     wol_pmMask,

                     wol_pmNotMask,

                     wol_pmXor,

                     wol_pmNotXor );





    TPenEvent = TNotifyEventA;



    TWOLPen = Class(TPersistent)

    private

      FResource   : TResourceItem;

      FWolHandle  : HPEN;

      FStyle      : TWOLPenStyle;

      FMode       : TWOLPenMode;

      FColor      : TCOLORREF;

      FWidth      : Integer;

      FLogPen     : TLogPen;

      FOnChange   : TPenEvent;

    protected

      procedure WOL_MountResource;

      procedure WOLCreatePen;



      function  WOL_GetMode:TWOLPenMode;

      procedure WOL_SetMode(Value:TWOLPenMode);

      function  WOL_GetWidth:Integer;

      procedure WOL_SetWidth(Value:Integer);

      function  WOL_GetColor:TCOLORREF;

      procedure WOL_SetColor(Value:TCOLORREF);

      function  WOL_GetStyle:TWOLPenStyle;

      procedure WOL_SetStyle(Value:TWOLPenStyle);

      function  WOL_GetHandle:HPEN;

      procedure WOL_SetHandle(Value:HPEN);

    public

      procedure Assign(Source: TPersistent); Override;

      constructor Create;

      destructor  Destroy; override;



      property Width   :Integer      read WOL_GetWidth  Write WOL_SetWidth;

      property Color   :TCOLORREF    read WOL_GetColor  Write WOL_SetColor;

      property Style   :TWOLPenStyle read WOL_GetStyle  Write WOL_SetStyle;

      property Handle  :HPEN         read WOL_GetHandle Write WOL_SetHandle;

      property Mode    :TWOLPenMode  read WOL_GetMode   Write WOL_SetMode;



      property OnChange:TPenEvent    read FOnChange     Write FOnChange;

    end;





var PenManager:TResourceManager;



implementation





//******************************* TWOLPen ********************************//

//                                                                        //

//                                                                        //

//************************************************************************//



procedure TWOLPen.WOL_MountResource;

begin

  FillChar(FResource, SizeOf(TResourceItem), 0);

  FResource.ID1[1]:=FColor;

  FResource.ID1[2]:=FWidth;

  FResource.ID1[3]:=Ord(FStyle);

end;



procedure TWOLPen.WOLCreatePen;

begin

  Self.FLogPen.lopnColor  :=FColor;

  Self.FLogPen.lopnWidth.x:=Self.FWidth;

  Self.FLogPen.lopnWidth.y:=0;

  case Self.FStyle of

       wol_psSolid      : Self.FLogPen.lopnStyle:=PS_SOLID;

       wol_psDash       : Self.FLogPen.lopnStyle:=PS_DASH;

       wol_psDot        : Self.FLogPen.lopnStyle:=PS_DOT;

       wol_psDashDot    : Self.FLogPen.lopnStyle:=PS_DASHDOT;

       wol_psDashDotDot : Self.FLogPen.lopnStyle:=PS_DASHDOTDOT;

       wol_psNull       : Self.FLogPen.lopnStyle:=PS_NULL;

       wol_psInsideFrame: Self.FLogPen.lopnStyle:=PS_INSIDEFRAME;

  end;

  if (FWidth > 1) and (not (FStyle in [wol_psSolid, wol_psNull, wol_psInsideFrame])) then begin

     FStyle:=wol_psSolid;

     FLogPen.lopnStyle:=PS_SOLID;

  end;

  WOL_MountResource;

  PenManager.Lock;

  FwolHandle:=PenManager.GetHandle(FResource);

  if FwolHandle <> 0 then begin

     PenManager.Unlock;

     if Assigned(FOnChange) then FOnChange(Self);

     Exit;

  end;

  if FwolHandle <> 0 then DeleteObject(FwolHandle);

  Self.FwolHandle:=CreatePenIndirect(Self.FLogPen);

  FResource.ID1[0]:=FwolHandle;

  PenManager.InsertTop(FResource);

  PenManager.Unlock;

  if Assigned(FOnChange) then FOnChange(Self);

end;



function  TWOLPen.WOL_GetMode:TWOLPenMode;

begin

  Result:=FMode;

end;



procedure TWOLPen.WOL_SetMode(Value:TWOLPenMode);

begin

  if FMode <> Value then

     FMode:=Value;

end;



function  TWOLPen.WOL_GetWidth:Integer;

begin

  Result:=Self.FWidth;

end;



procedure TWOLPen.WOL_SetWidth(Value:Integer);

begin

  Self.FWidth:=Value;

  WOLCreatePen;

end;



function  TWOLPen.WOL_GetColor:TCOLORREF;

begin

  Result:=Self.FColor;

end;



procedure TWOLPen.WOL_SetColor(Value:TCOLORREF);

begin

  Self.FColor:=Value;

  WOLCreatePen;

end;



function  TWOLPen.WOL_GetStyle:TWOLPenStyle;

begin

  Result:=Self.FStyle;

end;



procedure TWOLPen.WOL_SetStyle(Value:TWOLPenStyle);

begin

  Self.FStyle:=Value;

  WOLCreatePen;

end;



procedure TWOLPen.Assign(Source: TPersistent);

begin

  if Source is TWOLPen then

     With TWOLPen(Source) do begin

       Self.FWOLHandle:= Handle;

       Self.FStyle    := Style;

       Self.FWidth    := Width;

       Self.FLogPen   := TWOLPen(Source).FLogPen;

       FOnChange      := OnChange;

       Self.Color     := Color;

       Exit;

     end;

  inherited Assign(Source);

end;



constructor TWOLPen.Create;

begin

  inherited Create;

  FwolHandle:=GetStockObject(NULL_PEN);

  FColor    :=0;

  FWidth    :=0;

  FStyle    :=wol_psSolid;

  FMode     :=wol_pmCopy;

  FillChar(FLogPen, SizeOf(TLOGPEN), 0);

  WOL_MountResource;

  PenManager.Lock;

  if PenManager.GetHandle(FResource) = 0 then begin

     FResource.ID1[0]:=FwolHandle;

     PenManager.InsertTop(FResource);

  end;

  PenManager.Unlock;

end;



destructor  TWOLPen.Destroy;

begin

  if FwolHandle <> 0 then DeleteObject(FwolHandle);

  inherited Destroy;

end;



function  TWOLPen.WOL_GetHandle:HPEN;

begin

  Result:=FwolHandle;

end;



procedure TWOLPen.WOL_SetHandle(Value:HPEN);

begin

  if FwolHandle <> 0 then DeleteObject(FwolHandle);

  FWolHandle:=Value;

  if Assigned(FOnChange) then FOnChange(Self);

end;



initialization

   PenManager:=TResourceManager.Create;



finalization

   PenManager.Destroy;



end.



{***********************************************************************}

{                            FILE HISTORY                               }

{***********************************************************************}

{Version .2 (Carlos Alberto de Oliveira, jun 2-6 2001)                  }

{                                                                       }

{  1.) Created wolPen component                                         }

{***********************************************************************}

{Version .5 (Carlos Alberto de Oliveira, jul 11 2001)                   }

{                                                                       }

{  1.) Added PenMode property and routines to handle it                 }

{  2.) Changed color assignment                                         }

{  3.) Added WOL_MountResource for improvement in resources access      }

{  4.) Added PenManager for improvement in resources access             }

{***********************************************************************}

{Version .6 (Andrew Johnson, Aug 22 2001)                               }

{                                                                       }

{  1.) Added compiler directives for GTK.                               }

{***********************************************************************}



