{

  This program is distributed in the hope that it will be useful,

  but WITHOUT ANY WARRANTY; without even the implied warranty of

  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

  GNU General Public License (http://www.gnu.org/copyleft/gpl.html)

  for more details



************************************************************************

  WOL - Window Objects Library - version 0.00.3



  This library is a framework for Windows/GTK programming with the Free

  Pascal Compiler. The main objective is to make porting programs

  between Windows and Linux easier by using the same framework.



  If you find bugs or want colaborate with us to improve, please report

  to authors. It was originally writen by Carlos Alberto de Oliveira

  (carlosnatal@bol.com.br) and is maintened by the following authors:



  Carlos Alberto de Oliveira - carlosnatal@bol.com.br



  Missing you... Visit http://www.olhodetigre.hpg.com.br

                       http://www.freepascal.org

**************************************************************************

FileName : WOLFont.pp

Current File Version* : .4



Author(s) : Carlos Alberto de Oliveira(carlosnatal@bol.com.br)



Comments  : This File attempts to provide a Memory Manager for WOL objects

            and is part of kernel of WOL library.



*See end of file for File history

**************************************************************************

}





{$IFDEF FPC}

  {$MODE DELPHI}

{$ENDIF}



unit wolRMan;



interface



{$Include WOLDefs.Inc}

Uses

  Classes,

  {$IFDef GTKWIN}

  CPWindows,GTKWindows,

  {$Else}

  Windows,

  {$EndIF}

  wolDBG;



type



  TResourceItem = packed record

                    ID1   :array[0..4] of integer;

                    ID2   :string[32];

                  end;



  PResourceData = ^TResourceData;

  TResourceData = record

    Handle: THandle;

    data  : TResourceItem;

    next  : PResourceData;

    prev  : PResourceData;

  end;



  TResourceManager = class(TObject)

  private

    top    : PResourceData;

    bottom : PResourceData;

    current: PResourceData;

    size   : Longint;

    FLock  : {$IFDEF FPC}TCriticalSection{$ELSE}TRTLCriticalSection{$ENDIF};

  public

    constructor Create;

    destructor  Destroy; override;

    function  IsEmpty:Boolean;

    function  GetSize:Longint;

    procedure InsertBottom(item:TResourceItem);

    procedure InsertTop(item:TResourceItem);

    function  InsertCurrent(item:TResourceItem):Boolean;

    function  FindFirst(item:TResourceItem; var absLoc:longint):Boolean;

    function  Delete:Boolean;

    function  MoveFirst:Boolean;

    function  MoveLast:Boolean;

    function  MoveNext:Boolean;

    function  MovePrevious:Boolean;

    function  Seek(absLoc:longint):Boolean;

    function  GetData(var item:TResourceItem): Boolean;



    function  CalcID(item:TResourceItem):Longint;

    function  GetHandle(item:TResourceItem):THandle;

    procedure Lock;

    procedure Unlock;

  end;



implementation



{ set up the TResourceManager object with default values }

constructor TResourceManager.create;

begin

  inherited create;

  InitializeCriticalSection(FLock);

  top    := nil;

  bottom := nil;

  current:= nil;

  size   := 0;

end;



{ destroy the entire list, TResourceData by TResourceData }

destructor TResourceManager.Destroy;

var curRes : PResourceData;

    nextRes: PResourceData;

begin

  Lock;

  curRes:= top;

  while not (curRes = nil) do begin

    nextRes := curRes^.next;

    if curRes^.handle <> 0 then DeleteObject(curRes^.handle);

    freemem(curRes, SizeOf(TResourceData));

    curRes := nextRes;

  end;

  top    := nil;

  bottom := nil;

  current:= nil;

  Unlock;

  DeleteCriticalSection(FLock);

  inherited destroy;

end;



{ returns true if the list has no Ress }

function TResourceManager.isEmpty: Boolean;

begin

  result := (size = 0);

end;



{ returns number of Ress in list }

function TResourceManager.getSize: Longint;

begin

  result := size;

end;



{ insert TResourceData at bottom of list }

procedure TResourceManager.InsertBottom(item: TResourceItem);

var newRes: PResourceData;

begin

  GetMem(newRes, Sizeof(TResourceData));

  newRes^.handle:= item.id1[0];

  item.id1[0]   := 0;

  newRes^.data  := item;

  newRes^.prev  := bottom;

  newRes^.next  := nil;

  { special case: this is first TResourceData added }

  if bottom = nil then

     top := newRes

  else

     bottom^.next := newRes;

  bottom := newRes;

  size := size + 1;

end;



{ insert TResourceData at top of list }

procedure TResourceManager.InsertTop(item: TResourceItem);

var newRes: PResourceData;

begin

  GetMem(newRes, Sizeof(TResourceData));

  newRes^.handle:= item.id1[0];

  item.id1[0]   := CalcID(item);

  {newRes^.data  := item;}

  move(item, newRes^.data, SizeOf(TResourceItem));

  newRes^.prev  := nil;

  newRes^.next  := top;

  { special case: this is first TResourceData added }

  if top = nil then

     bottom := newRes

  else

     top^.prev := newRes;

  top := newRes;

  size := size + 1;

end;



{ insert TResourceData after current item }

function TResourceManager.InsertCurrent(item: TResourceItem): Boolean;

var newRes: PResourceData;

begin

  if (current = nil) then

     result := False

  else begin

     GetMem(newRes, Sizeof(TResourceData));

     newRes^.handle:= item.id1[0];

     item.id1[0]   := 0;

     newRes^.data  := item;

     newRes^.prev  := current;

     newRes^.next  := current^.next;

    { special case: current TResourceData is last TResourceData }

    if current^.next = nil then

       bottom := newRes

    else

       current^.next^.prev := newRes;

    current^.next := newRes;

    size := size + 1;

    result := True;

  end;

end;



{ Look for item in data field. Starts at top of list }

{ and looks at every item until a match is found.    }

{ if found, makes matched TResourceData current, and returns  }

{ absolute location of match where 1 = top.          }

function TResourceManager.FindFirst(item: TResourceItem; var absLoc: longint): Boolean;

var curRes: PResourceData;

    cnt   : longInt;

    vID   : longint;

begin

  result := False;

  curRes := top;

  cnt    := 0;

  absLoc := 0;

  vID:=CalcID(item);

  while not (curRes = nil) do begin

    cnt := cnt + 1;

    if (curRes^.Data.id1[0] = vID) then begin

       absLoc := cnt;

       current:= curRes;

       result := True;

       break;

       end

    else

       curRes := curRes^.next;

  end;

end;



{ delete the current TResourceData }

function TResourceManager.Delete: Boolean;

var Del:Boolean;

begin

  { we can only delete the current record }

  Result:=False;

  Del:=False;

  if current = nil then

     result := False

  else begin

     { see if list has one item }

     if size = 1 then begin

        top := nil;

        bottom := nil;

        Del:=True;

        end

     { see if we're at the top of list }

     else if current^.prev = nil then begin

        top := current^.next;

        top^.prev := nil;

        Del:=True;

        end

     { see if we're at the bottom of list }

     else if current^.next = nil then begin

        bottom := current^.prev;

        bottom^.next := nil;

        Del:=True;

        end

     { we must be in middle of list of size > 1 }

     else begin

        current^.prev^.next := current^.next;

        current^.next^.prev := current^.prev;

        Del:=True;

     end;

  end;



  if Del then begin

     result := True;

     freemem(current, SizeOf(TResourceData));

     current := nil;

     size := size - 1;

     if size = 0 then begin

        top := nil;

        bottom := nil;

     end;

  end;

end;



{ make first value in list current }

function TResourceManager.MoveFirst: Boolean;

begin

  if top = nil then

     result := False

  else begin

     current := top;

     result := True;

  end;

end;



{ make last value in list current }

function TResourceManager.MoveLast: Boolean;

begin

  if bottom = nil then

     result := False

  else begin

     current := bottom;

     result := True;

  end;

end;



{ make next value in list current }

function TResourceManager.MoveNext: Boolean;

begin

  if (current = nil) or (current^.next = nil) then

     result := False

  else begin

     current := current^.next;

     result := True;

  end

end;



{ make previous value in list current }

function TResourceManager.MovePrevious: Boolean;

begin

  if (current = nil) or (current^.prev = nil) then

     result := False

  else begin

     current := current^.prev;

     result := True;

  end;

end;



{ return data item from current list position }

function TResourceManager.GetData(var item: TResourceItem): Boolean;

begin

  if (current = nil) then

     result := False

  else begin

     item := current^.data;

     result := True;

  end;

end;



{ make current the absolute TResourceData N in the list }

{ where top = 1 }

function TResourceManager.Seek(absloc: longint): Boolean;

var curRes: PResourceData;

    cnt: longint;

begin

  result := False;

  if absloc <= 0 then

     exit;

  cnt := 0;

  curRes := top;

  while not (curRes = nil) do begin

    cnt := cnt + 1;

    if cnt = absloc then begin

       current := curRes;

       result := True;

       exit;

    end;

    curRes := curRes^.next;

  end;

end;



function TResourceManager.CalcID(item:TResourceItem):Longint;

var i:integer;

begin

  Result:=$FFFFFFFF;

  for i:=1 to 4  do Result:=Result xor Item.ID1[i];

  for i:=1 to 32 do Result:=Result xor Ord(Item.ID2[i]);

end;



function TResourceManager.GetHandle(item:TResourceItem):THandle;

var absLoc:Longint;

begin

  Result:=0;

  item.id1[0]:=0;

  if FindFirst(item, absLoc) then

     Result:=current^.Handle;

end;



procedure TResourceManager.Lock;

begin

  EnterCriticalSection(FLock);

end;



procedure TResourceManager.Unlock;

begin

  LeaveCriticalSection(FLock);

end;





{***********************************************************************}

{                            FILE HISTORY                               }

{***********************************************************************}

{Version .5 (Carlos Alberto de Oliveira, jul 12 2001)                   }

{                                                                       }

{  1.) Created ResourceManager                                          }

{                                                                       }

{***********************************************************************}

{Version .6 (Carlos Alberto de Oliveira, jul 8-14 2001)                 }

{                                                                       }

{  1.) Fixed FLock to eliminate access violation                        }

{                                                                       }

{Version .6 (Andrew Johnson, Aug 22 2001)                               }

{                                                                       }

{  1.) Added compiler directives for GTK.                               }

{***********************************************************************}



end.

