Unit XRoutines;

{$IFDef FPC}
  {$Mode ObjFPC}
{$EndIF}

interface

Uses Classes, SysUtils, Math, {$IFNDef Win32}X, Xlib,{$EndIF} CPWindows;

Procedure CircCenter(X1,Y1,X2,Y2: Extended; var X,Y : Extended);
Function Quadrant(X,Y,CX,CY,R : Extended) : Integer;
Procedure CPCircle(X,Y,CX,CY,R : Extended; Var ResX,ResY : Extended);
Function RadialAngle(X,Y,CX,CY,R : Extended) : Extended;
Procedure GetAngles(X1, Y1, X2, Y2, X3, Y3, X4, Y4 : Integer;
  var Start_Angle,End_Angle : Extended);
Procedure GetArcVariables(X1,Y1,X2,Y2,X3,Y3,X4,Y4 : Integer;
  var Width, Height, Angle1, Angle2 : Integer);
{$IFNDef Win32}
Procedure CreateFontList(Display : PDisplay);
{$EndIF}
Function GetFontName(Font : TLogFont) : PChar;

Function XGetCursorPos(Display : PDisplay; var pos : TPoint) : Boolean;
Procedure XSetCursorPos(Display : PDisplay; X, Y : Integer);
Procedure InitDisplay(DisplayName : PChar);
Procedure CloseDisplay;

var
  FontList : TStrings;
  theDisplay : PDisplay;

implementation

Procedure CircCenter(X1,Y1,X2,Y2: Extended; var X,Y : Extended);
var
  X3, Y3 :  Extended;
begin
  If X2 < X1 then begin
    X3 := X2;
    X2 := X1;
    X1 := X3;
  end;
  If Y2 < Y1 then begin
    Y3 := Y2;
    Y2 := Y1;
    Y1 := Y3;
  end;
  X := X1 + (X2 - X1) / 2;
  Y := Y1 + (Y2 - Y1) / 2;
end;

Function Quadrant(X,Y,CX,CY,R : Extended) : Integer;
begin
  Result := -1;
  If (Y < CY) then begin
    If (X > CX) then begin
      Result := 1;
    end
    else
      If (X < CX) then begin
        Result := 2;
      end
    else begin
      Result := 5;
    end;
  end
  else
    If (Y > CY) then begin
      If (X < CX) then begin
        Result := 3;
      end
      else
        If (X > CX) then begin
          Result := 4;
        end
      else begin
        Result := 7;
      end;
    end
  else
    If (Y = CY) then begin
      If (X > CX) then begin
        Result := 6;
      end
      else
        If (X < CX) then begin
          Result := 8;
        end;
    end;
end;

Procedure CPCircle(X,Y,CX,CY,R : Extended; Var ResX,ResY : Extended);
var
  H, TH, a, b : Extended;
begin
  ResX := CX;
  ResY := CY;
  If (X = CX) and (Y = CY) then
    exit;
  H := Sqrt(Sqr(X-CX) + Sqr(Y-CY));
  If H = 0 then
    exit;
  R := ABS(R);
  TH := ArcSin(ABS(Y-CY)/H);
  b := ABS(R*Cos(TH));
  a := ABS(R*Sin(TH));
  If H = R then begin
    ResX := X;
    ResY := Y;
  end
  else
    Case Quadrant(X,Y,CX,CY,R) of
      -1:begin{0,0}
           exit;
         end;
       1:begin
           ResX := CX + b;
           ResY := CY - a;
         end;
       2:begin
           ResX := CX - b;
           ResY := CY - a;
         end;
       3:begin
           ResX := CX - b;
           ResY := CY + a;
         end;
       4:begin
           ResX := CX + b;
           ResY := CY + a;
         end;
       5:begin{0,Y}
           ResX := CX;
           ResY := CY - a;
         end;
       6:begin{X,0}
           ResX := CX + b;
           ResY := CY;
         end;
       7:begin{0,-Y}
           ResX := CX;
           ResY := CY + a;
         end;
       8:begin{-X,0}
           ResX := CX - b;
           ResY := CY;
         end;
    end;
end;

Function RadialAngle(X,Y,CX,CY,R : Extended) : Extended;
var
  AX,AY,
  Chord : Extended;
begin
  CPCircle(X,Y,CX,CY,R,AX,AY);
  Chord := 0;
  Case Quadrant(AX,AY,CX,CY,R) of
    -1:begin{0,0}
         Result := 0;
       end;
     1,4:
       begin
         Chord := Sqrt(Sqr(AX - CX) + Sqr(AY - CY + R));
         Result := 90 + (360 - 2*RadToDeg(ArcSin(Chord/(2*R))));
       end;
     2,3:
       begin
         Chord := Sqrt(Sqr(AX - CX) + Sqr(AY - CY + R));
         Result := 90 + (2*RadToDeg(ArcSin(Chord/(2*R))));
       end;
     5:begin{0,Y}
         Result := 0;
       end;
     6:begin{X,0}
         Result := (3*Pi)/4;
       end;
     7:begin{0,-Y}
         Result := Pi;
       end;
     8:begin{-X,0}
         Result := Pi/2;
       end;
  end;
end;

Procedure GetAngles(X1, Y1, X2, Y2, X3, Y3, X4, Y4 : Integer;
  var Start_Angle,End_Angle : Extended);
var
  CX,CY,R : Extended;
begin
  CircCenter(X1,Y1,X2,Y2,CX,CY);
  R := ABS(Y2 - Y1) / 2;
  If ABS(X2 - X1) / 2 > R then
    R := ABS(X2-X1) /2;
  Start_Angle := RadialAngle(X3,Y3,CX,CY,R);
  End_Angle := RadialAngle(X4,Y4,CX,CY,R);
end;

Procedure GetArcVariables(X1,Y1,X2,Y2,X3,Y3,X4,Y4 : Integer; 
  var Width, Height, Angle1, Angle2 : Integer);
Var
  iStart_Angle, iDiff_Angle,
  start_angle, end_angle : Extended;
begin
  GetAngles(X1,Y1,X2,Y2,X3,Y3,X4,Y4,Start_Angle,End_Angle);
  Width := X2 - X1;
  Height := Y2 - Y1;
  istart_angle := start_angle * 64;
  idiff_angle  := (end_angle - start_angle) * 64;
  if (idiff_angle <= 0) then
    idiff_angle := idiff_angle + 360 * 64;
  Angle1 := Round(istart_angle); 
  Angle2 := Round(idiff_angle);
end;

{$IFNDef Win32}

Type
  TFontObject = Class(TObject)
  Public
    Familys,
    Weights : TStrings;
    theName : String;
    Constructor Create(FontName : String);
  end;

Constructor TFontObject.Create(FontName : String);
begin
  Inherited Create;
  theName := FontName;
  Weights := TStringList.Create;
  Familys := TStringList.Create;
end;

Type
  TXFontInfo = Record
    Family  : String;
    theName : String;
    Weight  : String;
    Style   : String;
    Size    : Longint;
  end;

Function FindAChar(CH : Char; SearchString : AnsiString; StartPos : Integer) : Integer;
var
  Tmp : String;
begin
  Tmp := Copy(SearchString, StartPos + 1, Length(SearchString) - StartPos);
  Result := Pos(String(CH),Tmp) - 1;
  If Result > -1 then
    Result := Result + StartPos;
end;

Procedure XParseFont(FontName : PChar; var FontInfo : TXFontInfo);
var
  Temp : AnsiString;
  I,J,K,L : Integer;
begin
  FillChar(FontInfo,SizeOf(TXFontInfo),0);
  If AnsiString(FontName) = '' then
    exit;
  I := -1;
  K := 0;
  J := 500;
  Repeat
    J := I + 1;
    I := FindAChar('-',AnsiString(Fontname), J);
    If (J <> I) or (FontName[J] = '-') then begin;
      Temp := '';
      K := K + 1;
      Case K of
        1 : Begin
	      For L := J to I - 1 do
	        Temp := Temp + FontName[L];
	      FontInfo.Family := Temp;
	    end;
        2 : Begin
	      For L := J to I - 1 do
	        Temp := Temp + FontName[L];
	      FontInfo.theName := Temp;
	    end;
        3 : Begin
	      For L := J to I - 1 do
	        Temp := Temp + FontName[L];
              FontInfo.Weight := UpperCase(Temp);
	    end;
        4 : Begin
	      For L := J to I - 1 do
	        Temp := Temp + FontName[L];
              FontInfo.Style := Uppercase(Temp);
	    end;
        7 : Begin
	      For L := J to I - 1 do
	        Temp := Temp + FontName[L];
	      If (Temp = '') or (Temp = '*') then
	        Temp := '11';
	      FontInfo.Size := StrToInt(Temp);
	    end;
      end;	    
    end;  
  Until ((J = I) and (FontName[J] <> '-')) or (K >= Length(AnsiString(FontName)));
end;

Procedure UpdateFont(FontName : AnsiString);
var
  FontInfo : TXFontInfo;
  Font : TFontObject;
  I : Integer;
begin
  XParseFont(PChar(FontName), FontInfo);
  If FontInfo.theName = '' then
    exit;
  I := FontList.IndexOf(UpperCase(FontInfo.theName));
  If I <> -1 then
    Font := TFontObject(FontList.Objects[I])
  else begin
    Font := TFontObject.Create(UpperCase(FontInfo.theName));
    I := FontList.AddObject(Font.theName,Font);
    Font := TFontObject(FontList.Objects[I]);
    Font.Familys.Append('*');
    Font.Weights.AddObject('*', TStringList.Create);
    TStrings(Font.Weights.Objects[0]).Append('*');
  end;
  I := Font.Familys.IndexOf(UpperCase(FontInfo.Family));
  If I < 0 then
    Font.Familys.Append(UpperCase(FontInfo.Family));
  I := Font.Weights.IndexOf(UpperCase(FontInfo.Weight));
  If I < 0 then begin
    I := Font.Weights.AddObject(UpperCase(FontInfo.Weight), TStringList.Create);
  end;
  If TStrings(Font.Weights.Objects[I]).IndexOf(UpperCase(FontInfo.Style)) = -1
  then begin
    TStrings(Font.Weights.Objects[I]).Append(UpperCase(FontInfo.Style));
  end;
end;

Procedure CreateFontList(Display : PDisplay);
var
  theFonts : PPChar;
  Tmp,Tmp2 : AnsiString;
  I, N : Integer;
begin
  If Assigned(FontList) then
    Try
      Fontlist.Free;
    except
    end;
  FontList := TStringList.Create;
  theFonts := XListFonts(Display,PChar('-*-*-*-*-*'
    + '-*-*-*-*-*-*-*-*-*'), 10000, @N);
  For I := 0 to N - 1 do
    If theFonts[I] <> nil then begin
      Tmp := AnsiString(theFonts[I]);
      Tmp2 := Copy(tmp,2,Length(Tmp));
      UpdateFont(Tmp2);
    end;
end;
{$EndIF}

Function InvalidWSCombo(Name : AnsiString; W,S : String) : Boolean;
var
  Font : TFontObject;
  I,J : Integer;
begin
  Result := True;
  If (Name = '') then
    exit;
  If (S = '*') and (W = '*') then
    Result := False
  else begin
    I := FontList.IndexOf(UpperCase(Name));
    If I <> -1 then begin
      Font := TFontObject(FontList.Objects[I]);
      I := Font.Weights.IndexOf(UpperCase(W));
      Result := I < 0;
      If not Result then
        If S = '*' then
          exit
        else begin
          Result := TStrings(Font.Weights.Objects[I]).IndexOf(UpperCase(S)) < 0;
        end;
    end;
  end;
end;

Function SlantPreference(W,S : String) : Boolean;
begin
  Result := False;
  If (W = '*') then
    Result := True
  else
    If S = '*' then
      Result := False;
end;

Function GetFontWeight(Name : AnsiString;  Weight : Longint) : String;

  Function FindValidWeight(Res : String) : String;
  begin
    If InvalidWSCombo(Name,Res, '*') then begin
      Repeat
        Inc(Weight,50);
        Result := GetFontWeight(Name,Weight);
      Until (not InvalidWSCombo(Name,Result, '*')) or (Result = '*');
    end else
      Result := Res;
  end;

begin
  Result := '';
  Case Weight of
    FW_THIN..FW_LIGHT:
      begin
        Result := 'THIN';
      End;
    FW_NORMAL..FW_REGULAR:
      begin
        Result := 'REGULAR';
      end;
    FW_MEDIUM:
      begin
        Result := 'MEDIUM';
      end;
    FW_SEMIBOLD:
      begin
        Result := 'SEMIBOLD';
      end;
    FW_DEMIBOLD:
      begin
        Result := 'DEMIBOLD';
      end;
    FW_BOLD :
      begin
        Result := 'BOLD';
      end;
    FW_EXTRABOLD..FW_ULTRABOLD:
      begin
        Result := 'BLACK';
      end;
    FW_HEAVY:
      begin
        Result := 'BLACK';
      end;
    FW_BLACK:
      begin
        Result := 'BLACK';
      end;
    1000..High(Longint):
      Result := '*';  
  end;
  If Result = '' then
    Result := 'BOLD';
  Result := FindValidWeight(Result);
end;

Function GetFontSlant(Name : AnsiString;  Weight : String;
  Italic : Boolean) : String;
begin
  Result := '';
  If Italic then begin
    Repeat
      If Result = '' then
        Result := 'i'
      else
        If Result = 'i' then
          Result := 'o'
        else
          Result := '*';
    Until (not InvalidWSCombo(Name,Weight,Result));
  end else begin
    Result := 'R';
    If InvalidWSCombo(Name,Weight,Result) then
      Result := '*';
  end;
end;

Function GetFontName(Font : TLogFont) : PChar;
var
  Val : AnsiString;
  Style1,
  Style2 : String;
  Name : PChar;
begin
  Result := nil;
  If Not Assigned(FontList) then
    exit;
  If Fontlist.Count = 0 then
    exit;
  With Font do begin
    name := lfFaceName;
    If Name = '' then
      Name := '*';
    If Not InvalidWSCombo(AnsiString(Name),'*','*') then begin
      Style1 := GetFontWeight(AnsiString(Name), lfWeight);
      Style2 := GetFontSlant(AnsiString(Name), Style1, Boolean(lfItalic));
    end
    else begin
     Name := '*';
     Style1 := '*';
     Style2 := '*';
    end;
    Val := '-*-' + Name + '-' + Style1 + '-' + Style2
       +'-*-*-'+ IntToStr(ABS(lfHeight)) + '-*-*-*-*-*-*-*';
    Result := PChar(Copy(Val, 1, Length(Val)));
  end;
end;

Procedure InitDisplay(DisplayName : PChar);
begin
  theDisplay := XOpenDisplay(Displayname);
end;

Procedure CloseDisplay;
begin
  XCloseDisplay(theDisplay);
  theDisplay := nil;
end;

Function XGetCursorPos(Display : PDisplay; var pos : TPoint) : Boolean;
var
  root, child : TWindow;
  rootX, rootY, winX, winY : Integer;
  xstate : Word;
begin
  pos.x := 0;
  pos.y := 0;
  Result := LongBool(XQueryPointer(display, XDefaultRootWindow(Display), @root,
    @child, @rootX, @rootY, @winX, @winY, @xstate));
  If Result then begin
    pos.x := rootX;
    pos.y := rootY;
  end;
end;

Procedure XSetCursorPos(Display : PDisplay; X, Y : Integer);
begin
  XWarpPointer(Display, XDefaultRootWindow(Display),
    XDefaultRootWindow(Display), 0, 0, 0, 0, X, Y);
  XFlush(Display );
end;

initialization
  theDisplay := nil;

end.
